/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * ARGPARSE.h
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 12/22/89
 *   MODIFIED : 01/22/95
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		Prototype for argparse();
 *
 * HISTORY
 *		01/22/95 (dcc) - Changed CHR_ARG from (1 << 15) to (1U << 15) (lint).
 *
*/

#ifndef ECHIDNA_ARGPARSE_H
#define ECHIDNA_ARGPARSE_H

#include <echidna/platform.h>

#include <echidna\listfunc.h>

#ifdef __cplusplus
extern "C" {
#endif

/**************************** C O N S T A N T S ***************************/
#define	STANDARD_ARG	1		/* Like Filename */
#define	KEYWORD_ARG		2		/* Arg that requires a prefix keyword */
#define	SWITCH_ARG		3		/* TRUE if arg keyword is specified */
#define TOGGLE_ARG		4

#define CHRKEYWORD_ARG	(KEYWORD_ARG | CHR_ARG)
#define	CHRSWITCH_ARG	(SWITCH_ARG  | CHR_ARG)
#define	CHRTOGGLE_ARG	(TOGGLE_ARG  | CHR_ARG)

#define EXIT_ARG			(1 <<  9)	/* Argparse stops here			*/
#define	MULTI_ARG			(1 << 10)	/* Many of these args allows	*/
#define	REQUIRED_ARG		(1 << 11)	/* Arg required					*/
#define CASE_SENSITIVE_ARG	(1 << 12)	/* Arg keyword/char case snsitv	*/
#define LIST_ARG			(1 << 13)	/* (multiarg is linked list)	*/
#define EXPAND_ARG			(1 << 14)	/* Expand arg into file list	*/

/** Private: Use above constants **/
#define CHR_ARG				(1U << 15)

#define ARGTYPE(type)		(type & 0x00FF)

/***************************** T Y P E D E F S ****************************/

typedef struct {
	unsigned	 ArgType;
	char		*ArgTokens;
	char		*ArgHelp;
} ArgSpec;

/******************************* M A C R O S ******************************/

#define SWITCH_VALUE(ptr)			((short)((long)(ptr)))
#define MULTI_ARGLIST(ptr)			((char **)(ptr))
#define	MULTI_ARGLINKEDLIST(ptr)	((ListType *)(ptr))

/****************** F U N C T I O N   P R O T O T Y P E S *****************/
#if __NOT_ANSI__		/** Non-ANSI Prototypes **/

#else					/** ANSI Prototypes **/

extern short	 argify (
					char	*line,
					short	 maxargs,
					char	**args);

extern short	 arganoid (
					char	**outv,
					int		*in_argc,
					char	***in_argv,
					ArgSpec	*templat,
					int		 fail,
					int		*multi);

extern char		**argvark(
					int		*in_argc,
					char	***in_argv,
					ArgSpec	*templat,
					int		 fail);

extern char		**argparse(
					int		 in_argc,
					char	**in_argv,
					ArgSpec	*templat);

extern void		 printarghelp (
					char	*usage,
					ArgSpec	*templat);

extern void		 freeargs(
					char	**outv,
					ArgSpec	*templat);

#endif /** Function Prototypes **/

#ifdef __cplusplus
}
#endif

#endif /* ECHIDNA_ARGPARSE_H */

