/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * EVENTS.H
 *
 * PROGRAMMER : Gregg A. Tavares 
 *    VERSION : 00.000
 *    CREATED : 08/08/89 
 *   MODIFIED : 07/26/94
 *       TABS : 05 09
 *
 * DESCRIPTION
 *
 *
 * HISTORY
 *
*/
#ifndef ECHIDNA_EVENTS_H
#define ECHIDNA_EVENTS_H 

#include <echidna/platform.h>

#ifdef __cplusplus
extern "C" {
#endif

#include <echidna/etypes.h>
#include <echidna/listfunc.h>

/**************************** C O N S T A N T S ***************************/

#define ETYPE_MENU		1
#define ETYPE_HITAREA	2
#define ETYPE_KEY		3

typedef enum { 
	inptFocus,  /* Focus (space or enter key binding) changed */
	inptMouse, 	/* Mouse button activity */
	inptCursor, /* Cursor key */
	inptHitKey,	/* Hot key or Focus key (space, enter or escape) hit */
	inptKey, 	/* Other Key board input */
	inptAbility,
} INPT;

/* Focus change flags */
#define FOCUS_SPACE_ON	(1 << 0)
#define FOCUS_SPACE_OFF	(1 << 1)
#define FOCUS_ENTER_ON	(1 << 2)
#define FOCUS_ENTER_OFF	(1 << 3)

/* Cursor key flags */
#define CURSOR_UP		(1 << 0)
#define CURSOR_DN		(1 << 1)
#define CURSOR_LFT		(1 << 2)
#define CURSOR_RGT		(1 << 3)
#define CURSOR_PREV		(1 << 4) /* On if CURSOR_UP or CURSOR_LFT. */
#define CURSOR_NEXT		(1 << 5) /* On if CURSOR_DN or CURSOR_RGT. */



/******************************** T Y P E S *******************************/

/*
** EventInfo
**
** Passed to an event functions
*/
typedef struct {
	struct Event	*Event;
	short			 EventType;
	void			*EventTrigger;
} EventInfo;

/*
 * HitInfo: Passed to function that is attached to a HitArea that got hit.
 * Fields:
 *
 * 	inpt: Input type field. 
 *		Events.H: inptFocus, inptMouse, inptHitKey, inptKey, inptCursor
 *
 * 	fFocus: Focus change flags input field. 
 *		Events.H: FOCUS_SPACE_ON, FOCUS_SPACE_OFF, 
 *				FOCUS_ENTER_ON, FOCUS_ENTER_OFF
 *
 *	fCursor: Cursor key input flags field.
 * 		Events.H: CURSOR_UP, CURSOR_DN, CURSOR_LFT, CURSOR_RGT, 
 *				CURSOR_PREV, CURSOR_NEXT
 *
 *	KeyCode: Key press input field (inptKey and inptHitKey).
 * 		KeyBoard.H: ASCII char or defined constant:
 *				UP_KEY, DOWN_KEY, LEFT_KEY, RIGHT_KEY, PGUP_KEY, PGDN_KEY, 
 *				BKSP_KEY, TAB_KEY, ESC_KEY, RETURN_KEY, DEL_KEY, HOME_KEY, 
 *				END_KEY		
 *
 *  X,Y: Mouse position input fields.
 * 		Offset of hit point into hit area
 *
 *	Buttons: Mouse buttons flags input field.
 *		Mouse.H: MOUSE_LEFTBUTTON, MOUSE_RIGHTBUTTON, MOUSE_FIRSTDOWN,
 *				MOUSE_BEFOREEVENT, MOUSE_AFTEREVENT, MOUSE_RELEASE,	
 *
 *	Shiftflags: Key press (shift, alt, ctrl) and Mouse click (double) 
 *				modifier flags field.
 * 		KeyBoard.H: KEY_SHIFT, KEY_ALT, KEY_CTRL, KEY_DOUBLE 
 *
 *	HitArea: Pointer to HitArea struct for area function being called.
 *		See HitArea.H
 *
*/
typedef struct HitInfo {
	INPT  inpt;
	UWORD fFocus;	
	UWORD fCursor;	
	short KeyCode;
	short X;
	short Y;
	UWORD Buttons; 
	UWORD Shiftflags; 
	struct HitArea		*HitArea;
} HitInfo;

/*
** Function Pointer Type
*/
typedef short (*FuncPtr)(EventInfo *ei);
typedef short (*MouseFunc)(HitInfo *hi);

/*
** EventState
**
** Used to keep track of the state of various menus states.
*/
typedef struct Event {
	NodeType	 Node;
	FuncPtr		 Function;	/* Function to call for this item. */
	short		*StateVar;	/* Pointer to State Variable	   */
	ULONG		 Flags;		/* Flags like CHECKIT, MENUTOGGLE  */
	ListType	*ExcludeList;	/* List of Items to EXCLUDE	   */
} EventState;

/*
** StateTracker.
**
** Used to keep lists of states.
*/
typedef struct {
	NodeType	 Node;
	EventState	*EventState;
} StateTracker;

/****************************** G L O B A L S *****************************/

extern ListType		*EventList;

extern MouseFunc	FN_BeforeEvent;
extern MouseFunc	FN_DontWait;
extern MouseFunc	FN_AfterEvent;

/****************** F U N C T I O N   P R O T O T Y P E S *****************/

/*
** KeyEvent.c
*/
extern char			*FindFirstEventKey (
							char	*eventname,
							USHORT	*qualifier);



/*
** HndlEvnt.c
*/

extern void			 SetStateVar (EventState *es, short state);
extern void			 AfterEvent (void);
extern void			 BeforeEvent (void);
extern EventState	*FindEvent (char *name);
extern short		 TriggerEvent (
							EventState	*es,
							short		 etype,
							void		*etrigger
						);

#ifdef __cplusplus
}
#endif

#endif /* ECHIDNA_EVENTS_H */

