/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * FONTS.H
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 09/10/90
 *   MODIFIED : 10/02/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		Font Routines header file for gfxlib\fonts.c.
 *
 * HISTORY
 *	05/25/91 Saturday (RGM) - Added include for <stddef.h>.
 *	06/01/91 Saturday (RGM) - Fixed clash with TextFont definition and
 *		Amiga OS TextFont definition by not using new TextFont definition
 *		on Amigas (is this right?).
 *	03/02/93 Tuesday (dcc) - added wFitString().
 *
*/

#ifndef ECHIDNA_FONTS_H
#define ECHIDNA_FONTS_H

#include <echidna/platform.h>
#include "switches.h"

#ifdef __cplusplus
extern "C" {
#endif

#include <stdlib.h>
#include <stddef.h>
#include <echidna/etypes.h>
#include <echidna/graphics.h>

/**************************** C O N S T A N T S ***************************/


/******************************** T Y P E S *******************************/

typedef struct Font {
	GShape	**Letters;
	UBYTE	 FirstLetter;
	UBYTE	 LastLetter;
} Font;

#if __AMIGAOS__

/* Amiga already has a TextFont typedef in <graphics/text.h> */

#else

typedef struct TextFont {
	struct Font Font;
} TextFont;

#endif

/****************************** G L O B A L S *****************************/


/******************************* M A C R O S ******************************/

#define GetFontHeight(pfont)		((pfont)->Letters[0]->Height)

#define GetFontCharWidth(pfont,ch)	(								\
	((ch) >= (pfont)->FirstLetter && (ch) <= (pfont)->LastLetter)	\
		? ((pfont)->Letters[(ch)-((pfont)->FirstLetter)]->Width)	\
		: 0															\
)

#define IsDrawChar(pfnt,ch)	(	\
	((ch) >= (pfnt)->FirstLetter) && ((ch) <= (pfnt)->LastLetter)	\
)

/****************** F U N C T I O N   P R O T O T Y P E S *****************/

short	 DrawString (Font *font, short x, short y, char *string);
short	 DrawStringN (Font *font, short x, short y, char *string, size_t len);
short	 StringLen (Font *font, char *string);
short	 StringLenN (Font *font, char *string, size_t len);
short	 wFitString(Font *pFont, char *sz, short wWidth);

#ifdef __cplusplus
}
#endif

#endif /* ECHIDNA_FONTS_H */

