/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * GRAPHICS.H
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 12/18/89
 *   MODIFIED : 10/08/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		
 *
 * HISTORY
 *		05/25/91 Saturday (RGM) - Added include for <stddef.h>.
 *		09/27/93 Monday (dcc) - updated for MS-DOS Watcom C.
 *		10/08/94 (dcc) - updated to use <echidna/alloc.h>
 *
*/
#ifndef ECHIDNA_GRAPHICS_H
#define ECHIDNA_GRAPHICS_H

#include <echidna/platform.h>
#include "switches.h"

#ifdef __cplusplus
extern "C" {
#endif

#include <stdlib.h>
#include <stddef.h>
#include <echidna/etypes.h>
#include <echidna/alloc.h>

#if __TURBOC__
#include <dos.h>
#endif
#if __AMIGAOS__
#include <graphics/rastport.h>
#endif

/**************************** C O N S T A N T S ***************************/
#define MODE_MCGA	0
#define MODE_EGA	1
#define MODE_TANDY	2
#define MODE_CGA	3

#define PCC_TWO_COPIES		1
#define PCC_FOUR_COPIES		2

#if __MSDOS__
	#define MAX_COLORS	255
#elif __AMIGAOS__ 
	#define MAX_COLORS	32
#define MAX_DEPTH	5
#else
#error Error: Platform missing MAX_COLORS define.
#endif

#define DEFAULT_MAX_CYCLE_COLORS		32
#define DEFAULT_PALETTE_UPDATE_HERTZ	30

#define SCREEN_WIDTH	320
#define SCREEN_HEIGHT	200

/******************************** T Y P E S *******************************/

typedef struct {
	short		 X;
	short		 Y;
	short		 Width;
	short		 Height;
	void		*Data;
} SaveAreaType;

/******************************** RemapTableType */
typedef struct {
	UWORD	 Elements;
	UBYTE	*From;
	UBYTE	*To;
} RemapTableType;

/******************************** RemapInfoType */
typedef struct {
	UWORD			 NumRemapTables;
	RemapTableType	*Tables;
} RemapInfoType;

/******************************** ColorType */
#if __AMIGAOS__
	typedef struct {
		UWORD	Color;
	} ColorType;

#elif __MSDOS__
	typedef struct {
		UBYTE	Red;
		UBYTE	Green;
		UBYTE	Blue;
	} ColorType;
#else
#error Error: Platform missing ColorType.
#endif

/******************************** ColorCycleType */
typedef struct {
	WORD NumColors;		/* Number of colors in cycle. */
	WORD NumRegisters;	/* Number of registers in cycle. */
	WORD Hertz;			/* Changes per second */
	WORD Direction;		/* 0=positive; 1=negative */
	WORD Active;		/* TRUE if cycle is active. */
	WORD Timer;			/* Count down timer for timing speed */
	WORD CurrentColor;	/* Index in Colors to put in Registers[0]. */
	void		*Colors;
	UBYTE		*Registers;	
} ColorCycleType;

/******************************** ColorCycleSetType */
typedef struct {
	WORD 			 NumCycles;
	WORD			 Active;
	WORD			 TotalColors;
	WORD			 TotalRegisters;
	ColorCycleType	*Cycles;
	void			*Colors;
	UBYTE			*Registers;
} ColorCycleSetType;


/******************************** PaletteType */
#if __AMIGAOS__
	typedef struct {
		short			 NumColors;
		union {
			ColorType	*RGB;
		} Data;
	} PaletteType;

#elif __MSDOS__
	typedef struct {
		short		NumColors;
		union {
			ColorType	*RGB;
			UBYTE	 	*EGA;
			UWORD	 	 CGA;
		} Data;
	} PaletteType;
#else
#error Error: Platform missing PaletteType.
#endif

/******************************** PaintBoxType */
typedef struct {
	PaletteType			*Palette;
	ColorCycleSetType	*CycleSet;
} PaintBoxType;


#if __AMIGAOS__
	typedef void (*PauseFunc)(void);

	typedef struct RastPort	 RastPort;
	typedef struct TextFont	 TextFont;
	typedef struct BitMap	*GPage;

#endif

#if __MSDOS__
typedef void (far32 *GfxRoutinePtr)();

#if __MSDOS32X__
typedef UBYTE *GPage;
#else
typedef UWORD GPage;
#endif

typedef struct {
	int   far32	*ClipLeft;
	int   far32	*ClipTop;
	int   far32	*ClipWidth;
	int   far32	*ClipHeight;
	int   far32	*ClipRight;
	int   far32	*ClipBottom;
	UBYTE far32	*PenColor;
	UBYTE far32	*PaperColor;
#if __MSDOS32X__
	GPage far32	*GrDraw;
#else
	GPage far32	*GrSeg;
#endif
	GPage far32	*WorkBuffer;
	GPage far32	*BackBuffer;
	GPage far32	*DisplayBuffer;
	UBYTE far32	*EvenPatternTable;
	UBYTE far32	*OddPatternTable;
} GfxDataTbl;

typedef struct RastPort {
	short	 cp_x;
	short	 cp_y;
	short	 TxBaseline;
	void	*Font;
	UBYTE	 DrawMode;
	UBYTE	 pad0;
} RastPort;

#define JAM1	0
#define JAM2	1

#define SetDrMd(rp,mode)	{ (rp)->DrawMode = (mode); }
#define SetFont(rp,font)	{ (rp)->Font     = (font); }


typedef	struct {
	UWORD	 NumPatterns;
	UBYTE	*Even;
	UBYTE	*Odd;
} PatmapTableType;

typedef struct {
	UWORD			 NumPatmapTables;
	PatmapTableType	*PatmapTables;
	UBYTE			*DataBuf;
} PatternInfoType;

#endif

/******************************** GShape Types */
#define USE_SHAPE_TPAT		1	/* Use transparency pattern fields in shapes*/
 
typedef struct {
	UWORD	XOffset;
	UWORD	DWidth;
	UWORD	YOffset;
	UWORD	DHeight;
	UBYTE	PlanePick;
	UBYTE	PlaneOnOff;
} AmigaSDataType;
#define	AMIGASDATASIZE	(2 + 2 + 2 + 2 + 1 + 1)

typedef struct {
	UWORD	XOffset;
	UWORD	DWidth;
	UWORD	YOffset;
	UWORD	DHeight;
} MCGASDataType;
#define MCGASDATASIZE	(2 + 2 + 2 + 2)

typedef struct {
	UWORD	XOffset;
	UWORD	DWidth;
	UWORD	YOffset;
	UWORD	DHeight;
} TGAGShape;
#define TGAGSHAPESIZE	(2 + 2 + 2 + 2)

#define TGA_NUM_GPOSITIONS	2
typedef struct {
#if USE_SHAPE_TPAT
	UBYTE			 TPatEven;	/* Transparent color pattern, even byte */
	UBYTE			 TPatOdd;	/* Transparent color pattern, odd byte */
#endif
	TGAGShape far32	*ts[TGA_NUM_GPOSITIONS];
	UBYTE			 pcc;
	UBYTE			 pad0;
} TGASDataType;
#if USE_SHAPE_TPAT
#define TGASDATASIZE	(1 + 1 + 4 * TGA_NUM_GPOSITIONS + 1 + 1)
#else
#define TGASDATASIZE	(4 * TGA_NUM_GPOSITIONS + 1 + 1)
#endif

typedef struct {
	UWORD	XOffset;
	UWORD	DWidth;
	UWORD	YOffset;
	UWORD	DHeight;
} CGAGShape;
#define CGAGSHAPESIZE	(2 + 2 + 2 + 2)

#define CGA_NUM_GPOSITIONS	4
typedef struct {
#if USE_SHAPE_TPAT
	UBYTE			 TPatEven;	/* Transparent color pattern, even byte */
	UBYTE			 TPatOdd;	/* Transparent color pattern, odd byte */
#endif
	CGAGShape far32	*cs[CGA_NUM_GPOSITIONS];
	UBYTE			 pcc;
	UBYTE			 pad0;
} CGASDataType;
#if USE_SHAPE_TPAT
#define CGASDATASIZE	(1 + 1 + 4 * CGA_NUM_GPOSITIONS + 1 + 1)
#else
#define CGASDATASIZE	(4 * CGA_NUM_GPOSITIONS + 1 + 1)
#endif

typedef struct {
#if USE_SHAPE_TPAT
	UBYTE	TPatEven;	/* Transparent color pattern, even byte */
	UBYTE	TPatOdd;	/* Transparent color pattern, odd byte */
#endif
	UWORD	XOffset;
	UWORD	DWidth;
	UWORD	YOffset;
	UWORD	DHeight;
} EGASDataType;
#if USE_SHAPE_TPAT
#define EGASDATASIZE	(1 + 1 + 2 + 2 + 2 + 2)
#else
#define EGASDATASIZE	(2 + 2 + 2 + 2)
#endif

typedef union {
	AmigaSDataType	Amiga;
	MCGASDataType	MCGA;
	CGASDataType	CGA;
	TGASDataType	TGA;
	EGASDataType	EGA;
} SData;

typedef struct {
	UWORD	Size;
	short	Width;
	short	Height;
	UWORD	TColor;
	SData	Data;
} GShape;
#define GSHAPESIZE		(2 + 2 + 2 + 2)

typedef struct {
	GShape			*ShapeData;
	UBYTE			*ColorMap;
#if __MSDOS__
	PatmapTableType	*PatternMap;
	UBYTE	 		 pcc;
	UBYTE			 ExpandPatterns;
#endif
#if __AMIGAOS__
	UBYTE			 Remap;
	UBYTE			 pad0;
#endif
	short	 		 HFlip;
	short	 		 VFlip;
} ConvertSType;

/****************************** G L O B A L S *****************************/
#if __MSDOS__
extern int   far32		*ClipLeft;
extern int   far32		*ClipTop;
extern int   far32		*ClipWidth;
extern int   far32		*ClipHeight;
extern int   far32		*ClipRight;
extern int   far32		*ClipBottom;
extern UBYTE far32		*PenColor;
extern UBYTE far32		*PaperColor;
#if __MSDOS32X__
extern GPage far32		*GrDraw;
#else
extern GPage far32		*GrSeg;
#endif
extern GPage far32		*WorkBuffer;
extern GPage far32		*BackBuffer;
extern GPage far32		*DisplayBuffer;
extern UBYTE far32		*EvenPatternTbl;
extern UBYTE far32		*OddPatternTbl;
extern PaletteType		*CurrentPalette;
#endif

#if __AMIGAOS__
extern int				 ClipLeft;
extern int				 ClipTop;
extern int				 ClipWidth;
extern int				 ClipHeight;
extern short			 ClipRight;
extern short			 ClipBottom;
extern UBYTE 			 PenColor;
extern UBYTE 			 PaperColor;
extern GPage 			 GrSeg;
extern GPage 			 WorkBuffer;
extern GPage 			 BackBuffer;
extern GPage 			 DisplayBuffer;
extern PaletteType		*CurrentPalette;
extern PauseFunc		 BeforePause;
extern PauseFunc		 AfterPause;
#endif

extern short			 GraphicsMode;	
extern ULONG far32		 VBTimer;

extern short 			 PaletteMaxColors;
extern volatile short	 PaletteIsFading;
extern volatile short	 CycleSetIsFading;
extern short			 ColorCyclingActive; /* Use macros to access this */

/******************************* M A C R O S ******************************/
#define ColorCyclingOn()	ColorCyclingActive = 1
#define ColorCyclingOff()	ColorCyclingActive = 0
#define ColorCyclingState()	(ColorCyclingActive)

#if __MSDOS__
	#define SetClipDimensions(x,y,width,height)		\
		(*ClipLeft		= (x)					),	\
		(*ClipTop 		= (y)					),	\
		(*ClipWidth 	= (width)				),	\
		(*ClipHeight	= (height)				),	\
		(*ClipRight		= ((x) + (width) - 1)	),	\
		(*ClipBottom	= ((y) + (height) - 1)	)	

	#define SetClipEdges(xmin,ymin,xmax,ymax)		\
		(*ClipLeft		= (xmin)				),	\
		(*ClipTop		= (ymin)				),	\
		(*ClipWidth		= ((xmax) - (xmin) + 1)	),	\
		(*ClipHeight	= ((ymax) - (ymin) + 1)	),	\
		(*ClipRight		= (xmax)				),	\
		(*ClipBottom	= (ymax)				)	

	#define SetRemapFlag(cs,flag)
#if __MSDOS32X__
	#define SetWorkPage(pbuf)		(*GrDraw = (*pbuf))
#else
	#define SetWorkPage(seg)		(*GrSeg = (*seg))
#endif
	#define SetPenColor(color)		(*PenColor = (color))
	#define SetPaperColor(color)	(*PaperColor = (color))

	#define GetPenColor()			(*PenColor)
	#define GetPaperColor()			(*PaperColor)
#if __MSDOS32X__
	#define GetDisplayAddress()		(*DisplayBuffer)
#else
	#define GetDisplayAddress()		(MK_FP(*DisplayBuffer, 0))
#endif

	#define GetClipLeft()			(*ClipLeft)
	#define GetClipTop()			(*ClipTop)
	#define GetClipWidth()			(*ClipWidth)
	#define GetClipHeight()			(*ClipHeight)
	#define GetClipRight()			(*ClipRight)
	#define GetClipBottom()			(*ClipBottom)

	#define GetClipDimensions(x,y,width,height)	\
		((x)	 	= *ClipLeft		),			\
		((y)	 	= *ClipTop 		),			\
		((width) 	= *ClipWidth 	),			\
		((height)	= *ClipHeight	)

	#define GetClipEdges(xmin,ymin,xmax,ymax)	\
		((xmin)		= *ClipLeft		),			\
		((ymin)		= *ClipTop		),			\
		((xmax)		= *ClipRight	),			\
		((ymax)		= *ClipBottom	)	

	#define GRoutine(name)			(far32 *name)

#elif __AMIGAOS__
	#define SetClipDimensions(x,y,width,height)		\
		(ClipLeft		= (x)					),	\
		(ClipTop 		= (y)					),	\
		(ClipWidth 		= (width)				),	\
		(ClipHeight		= (height)				),	\
		(ClipRight		= ((x) + (width) - 1)	),	\
		(ClipBottom		= ((y) + (height) - 1)	)	

	#define SetClipEdges(xmin,ymin,xmax,ymax)		\
		(ClipLeft		= (xmin)				),	\
		(ClipTop		= (ymin)				),	\
		(ClipWidth		= ((xmax) - (xmin) + 1)	),	\
		(ClipHeight		= ((ymax) - (ymin) + 1)	),	\
		(ClipRight		= (xmax)				),	\
		(ClipBottom		= (ymax)				)	

	#define SetRemapFlag(cs,flag)	((cs).Remap = (flag))
	#define SetWorkPage(seg)		(GrSeg = (seg))
	#define SetPenColor(color)		(PenColor = (color))
	#define SetPaperColor(color)	(PaperColor = (color))
	#define GetPenColor()			(PenColor)
	#define GetPaperColor()			(PaperColor)
	#define GetDisplayAddress()		(GrSeg)

	#define GetClipLeft()			(ClipLeft)
	#define GetClipTop()			(ClipTop)
	#define GetClipWidth()			(ClipWidth)
	#define GetClipHeight()			(ClipHeight)
	#define GetClipRight()			(ClipRight)
	#define GetClipBottom()			(ClipBottom)

	#define GetClipDimensions(x,y,width,height)	\
		((x)	 	= ClipLeft		),			\
		((y)	 	= ClipTop 		),			\
		((width) 	= ClipWidth 	),			\
		((height)	= ClipHeight	)

	#define GetClipEdges(xmin,ymin,xmax,ymax)	\
		((xmin)		= ClipLeft		),			\
		((ymin)		= ClipTop		),			\
		((xmax)		= ClipRight		),			\
		((ymax)		= ClipBottom	)	

	#define GRoutine(name)			name
#else
#error	Error: Platform missing macro defines for global variable setting.
#endif

/* Only use FreePalette on palettes created by allocating SizeOfPalette() 
 * using CopyPalette(), or palettes created by DupPalette().  Do not
 * use FreePalette to free a palette in a BPI structure.
 */
#define FreePalette(p)	MEM_free (p)

#define FadeToPalette(pal,frames)	\
		FadeToPaletteN ((pal), 0, 0, PaletteMaxColors, (frames))

/* Backward Compatability macros */
#define FadePaletteIn(table, frames)	\
		FadeToPalette ((table), (frames))

#define FadePaletteOut(table, frames)	\
		FadeToPalette (NULL, (frames))

#define ShowScreen()		ShowRect (0, 0, 320, 200)
#define RestoreScreen()		Restore	(0, 0, 320, 200)
#define UnRestoreScreen()	UnRestore	(0, 0, 320, 200)
#define RestoreShowScreen()	RestoreScreen();	\
							ShowScreen()

#define DrawHLine(x,y,len)	DrawRect ((x), (y), (len), 1)
#define DrawVLine(x,y,len)	DrawRect ((x), (y), 1, (len))
#define DrawXorHLine(x,y,len)	DrawXorRect ((x), (y), (len), 1)
#define DrawXorVLine(x,y,len)	DrawXorRect ((x), (y), 1, (len))

#define DrawPatHLine(x,y,len)	DrawPatRect ((x), (y), (len), 1)
#define DrawPatVLine(x,y,len)	DrawPatRect ((x), (y), 1, (len))

#define DrawPatXorHLine(x,y,len)	DrawPatXorRect ((x), (y), (len), 1)
#define DrawPatXorVLine(x,y,len)	DrawPatXorRect ((x), (y), 1, (len))

#if USE_GRAFX
	#define BeforeGraphics()	RealBeforeGraphics()
	#define AfterGraphics()		RealAfterGraphics()
#else
	#define BeforeGraphics()
	#define AfterGraphics()
#endif

/****************** F U N C T I O N   P R O T O T Y P E S *****************/
extern short			 OpenGraphics(
							short mode, 
							size_t max_brush_size,
							short max_palette_colors, 
							short max_cycle_colors, 
							short palette_update_hertz
							);
extern void				 CloseGraphics(void);

extern void				 ResetVTimer(short ticks);
extern short			 CheckVTimer(void);
extern void				 WaitVTimer(void);

#if __MSDOS__
	extern void				 SetGlobalPattern (PatmapTableType *pmt);
	extern void far32 		*GRoutine(GetGShapeLoadBuf)(void);
#elif __AMIGAOS__
	#define SetGlobalPattern(arg)
	#define	GetGShapeLoadBuf()	((void *)(BackBuffer->Planes[0]))
#else
#error Error: Platform missing SetGlobalPattern() and GetGShapeLoadBuffer().
#endif

extern void				 DrawBox (int x, int y, int w, int h);
extern void				 DrawThickBox (int x, int y, int w, int h, int thickness);
extern void				 DrawXorBox (int x, int y, int w, int h);
extern void				 DrawPatBox (int x, int y, int w, int h);
extern void				 DrawPatXorBox (int x, int y, int w, int h);

extern void				 GRoutine(DrawPixel)(int x, int y);
extern void				 GRoutine(DrawRect)(int x, int y, int width, int height);
extern void				 GRoutine(ClearWorkPage)(void);
extern void				 GRoutine(DrawGShape)(GShape far32 *gshape, int x, int y);
extern void				 GRoutine(DrawGShapeNoTrans)(GShape far32 *gshape, int x, int y);
extern void				 GRoutine(Restore)(int x, int y, int width, int height);
extern void				 GRoutine(UnRestore)(int x, int y, int width, int height);
extern void				 GRoutine(ShowRect)(int x, int y, int width, int height);
extern void				 GRoutine(FreeGShape)(GShape *gshapeptr);
extern GShape			*GRoutine(ConvertGShape)(
								ConvertSType	*cs
							);

#if USE_GRAFX

extern void RealBeforeGraphics (void);
extern void RealAfterGraphics (void);

extern void				 GRoutine(ClippedCopyRect)(void *src, int sx, int sy,
										void *dest, int dx, int dy,
										int width, int height);
extern void				 GRoutine(CopyRect)(void *src, int sx, int sy,
										void *dest, int dx, int dy,
										int width, int height);
extern void				 GRoutine(XORRect)(int x, int y, int w, int h);
extern SaveAreaType		*GRoutine(SaveArea) (int x, int y, int width, int height);
extern void				 GRoutine(RestoreArea) (SaveAreaType *sa);

#else
extern size_t			 SizeOfPalette (PaletteType *palette);
extern PaletteType		*CopyPalette (
								void *dstbuf,
								PaletteType *srcpal
							);
extern PaletteType		*DupPalette (PaletteType *palette);

extern void				 WaitForFade (void); 
extern void				 GRoutine(UsePaletteN)(
								PaletteType	*palette,
								short		 src_offset,
								short		 dst_offset,
								short		 numcolors
							);
extern void				 GRoutine(FadeToPaletteN) (
								PaletteType	*palette,
								short		 src_offset,
								short		 dst_offset,
								short		 numcolors,
								short		 hz60frames
							);
extern void				 GRoutine(UseColorCycleSet)(
								ColorCycleSetType	*color_cycle_set,
								short				 activate
							);
extern void				 GRoutine(FadeToColorCycleSet) (
								ColorCycleSetType	*color_cycle_set,
								short		 		 hz60frames
							);
extern void				 GRoutine(UsePaintBoxN) (
								PaintBoxType	*paintbox,
								short			 src_offset,
								short			 dst_offset,
								short			 numcolors, 
								short			 activate_color_cycling
							);
extern void				 GRoutine(FadeToPaintBoxN) (
								PaintBoxType	*paintbox,
								short			 src_offset,
								short			 dst_offset,
								short			 numcolors,
								short			 hz60frames
							);
#endif /* ELSE USE_GRAFX */

#ifdef __cplusplus
}
#endif

#endif /* ECHIDNA_GRAPHICS_H */

