/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * MEMORY.H
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 03/18/90
 *   MODIFIED : 11/08/95
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		Memory routines because MS-DOS C memory allocation sucks!
 *
 * HISTORY
 *		05/25/91 (RGM) - Added include for <stddef.h>.
 *		08/27/92 (GAT) - Added farmalloc, farcalloc, farfree
 *		09/20/94 (dcc) - simplify farmalloc, farcalloc, farfree so that
 *						 they do not collide with 32-bit Borland C macros
 *		10/08/94 (dcc) - updated to use <echidna/alloc.h>
 *		11/08/95 (dcc) - #undef farmalloc, farcalloc, and farfree 
 *                       so 32-bit Borland C works.
 *
*/

#ifndef ECHIDNA_MEMORY_H
#define ECHIDNA_MEMORY_H

#include <echidna/platform.h>
#include "switches.h"

#ifdef __cplusplus
extern "C" {
#endif

#include <stdlib.h>
#include <stddef.h>
#include <echidna/etypes.h>
#include <echidna/alloc.h>

#if __TURBOC__
#include <alloc.h>
#endif

#if (__ZTC__ && __MSDOS__)
#include <dos.h>
#endif

/**************************** C O N S T A N T S ***************************/


/******************************** T Y P E S *******************************/
#if __MSDOS__
typedef void *(*AllocMemFuncPtr)(unsigned long size);
typedef void (*FreeSomeFuncPtr)(unsigned long size);
#endif

/****************************** G L O B A L S *****************************/


/******************************* M A C R O S ******************************/
#if __MSDOS__
#define ClearExtraMemFuncs()	SetExtraMemFuncs (NULL,NULL)
#endif

#if (__MSDOS32X__ || __AMIGAOS__ || __MACOS__ || __EGGS_SGI__)

#if __BORLANDC__
#ifdef farmalloc
#undef farmalloc
#endif
#ifdef farcalloc
#undef farcalloc
#endif
#ifdef farfree
#undef farfree
#endif
#endif

#define	farmalloc				MEM_malloc
#define	farcalloc				MEM_calloc
#define	farfree					MEM_free
#endif

/****************** F U N C T I O N   P R O T O T Y P E S *****************/

#if __MSDOS32X__

#define memfree()				error "memfree() not implemented"
#define freesome()				error "freesome() not implemented"
#define	SetExtraMemFuncs(a,b)	error "SetExtraMemFuncs() not implemented"
#define	UseExtraMemory(a,b)		error "UseExtraMemory not implemented"
#define OpenMemory()				(TRUE)
#define	CloseMemory()
#define	CreateTempBuffer			MEM_malloc
#define DestroyTempBuffer(mem,size)	(MEM_free(mem))
#define AllocMemOnPara				MEM_malloc
#define FreeMemOnPara				MEM_free

#else	/* !__MSDOS32X__ */
#if __NOT_ANSI__						/** Non-ANSI Prototypes **/

#else	/* !__NOT_ANSI__ */				/** ANSI Prototypes **/

extern ULONG	 memfree (void);
extern void		 freesome (size_t size);
extern short	 OpenMemory (void);
extern void		 CloseMemory (void);

#if			DEBUG

 void far32 *__CreateTempBuffer (size_t size, char *pFile, int iLine);
 void __DestroyTempBuffer(void far *mem, size_t size, char *pFile, int iLine);

 #define CreateTempBuffer(size)			__CreateTempBuffer(size,__FILE__,__LINE__)
 #define DestroyTempBuffer(mem,size)	__DestroyTempBuffer(mem,size,__FILE__,__LINE__)

#else	/* !DEBUG */

 void far32 *CreateTempBuffer (size_t size);
 void DestroyTempBuffer(void far *mem, size_t size);

#endif	/* !DEBUG */

#if __MSDOS__
extern void		 SetExtraMemFuncs (AllocMemFuncPtr memfunc, FreeSomeFuncPtr fsfunc);
extern void		 UseExtraMemory (void *mem, size_t size);

#if			DEBUG

 void far32 *__AllocMemOnPara (ULONG size, char *pFile, int iLine);
 void __FreeMemOnPara(void *mem, char *pFile, int iLine);

 #define AllocMemOnPara(size)			__AllocMemOnPara(size,__FILE__,__LINE__)
 #define FreeMemOnPara(mem)				__FreeMemOnPara(mem,__FILE__,__LINE__)

#else	/* !DEBUG */

 void far32 *AllocMemOnPara (ULONG size);
 void FreeMemOnPara(void *mem);

#endif	/* !DEBUG */

#endif	/* __MSDOS__ */

#if __AMIGAOS__
extern void		*mallocChipMem (size_t size);
extern void		*callocChipMem (size_t num, size_t size);
#endif	/* __AMIGAOS__ */

#endif	/* !__NOT_ANSI__ */				/** Function Prototypes **/
#endif	/* !__MSDOS32X__ */

#ifdef __cplusplus
}
#endif

#endif /* ECHIDNA_MEMORY_H */

