/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * MENUS2.H
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 10/09/90
 *   MODIFIED : 10/12/93
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		Generic Menu Definitions.
 *
 * HISTORY
 *
*/

#ifndef ECHIDNA_MENUS2_H
#define ECHIDNA_MENUS2_H

#include <echidna/platform.h>

#ifdef __cplusplus
extern "C" {
#endif

#include <echidna/listfunc.h>
#include <echidna/events.h>
#include <echidna/grafx.h>
#include <echidna/fonts.h>
#include <echidna/readini.h>
#include <echidna/mouse.h>
#include <echidna/windows.h>

/**************************** C O N S T A N T S ***************************/

#define MNU_HAS_CHECK	(1U <<  0U)
#define MNU_HAS_RADIO	(1U <<  2U)
#define MNU_LINED		(1U <<  4U)
#define MNU_OFF			(1U <<  5U)
#define MNU_GHOSTED		(1U <<  6U)
#define MNU_POPUP		(1U <<  7U)
#define MNU_SEPARATOR	(1U <<  8U)
#define MNU_BREAK		(1U <<  9U)
#define MNU_BARBREAK	(1U << 10U)
#define MNU_SHIFT		(1U << 11U)
#define MNU_CTRL		(1U << 12U)
#define MNU_ALT			(1U << 13U)

/******************************** T Y P E S *******************************/

typedef struct MenuItem {
	NodeType		 Node;
	short			 X;
	short			 Y;
	short			 Width;
	short			 Height;
	char			*LeftText;
	char			*RightText;
	char			*KeyText;
	USHORT			 Flags;
	EventState		*Event;
} MenuItem;

typedef struct PopupItem {
	MenuItem			 Item;
	short				 PopX;
	short				 PopY;
	short				 PopWidth;
	short				 PopHeight;
	short				 PopCount;
	struct PopupItem	*Parent;
	ListType			 MenuItemListX;
	ListType			*MenuItemList;
} PopupItem;

/****************************** G L O B A L S *****************************/

extern PopupItem		*MainPopup;
extern short			 MenuBarHeight;
extern short			 MenuSticky;

/******************************* M A C R O S ******************************/


/****************** F U N C T I O N   P R O T O T Y P E S *****************/

extern void			 DrawMenuBar (void);
extern short		 ComputeMenuBarHeight (void);
extern short		 LastMenuEvent (EventInfo *ei);
extern short		 InitMenus (IniList *inilist);
extern void			 SetMenuColors (short white, short black);
extern void			 SetMenuInfo (
							short	 swidth,
							short	 sheight,
							Font	*font,
							GShape	**symbols);
extern short		 CheckMenuKeys (USHORT keyvalue);
extern short		 HandleMenus (Window *win, MouseInfo *mouse);
extern void			 SetShowKeyCmds(BOOL f);
extern short		 ParsePopup (SectionTracker *sec, PopupItem *popup, short goodscan);
extern short		 DoMenus (MenuItem *mi, MouseInfo *mouse);
extern short		 DoLocalMenus (PopupItem *pop, MenuItem *mi, MouseInfo *mouse);
extern MenuItem		*RealCheckMenuKeys (PopupItem *pop, USHORT keyvalue, short alt);

#ifdef __cplusplus
}
#endif

#endif /* ECHIDNA_MENUS2_H */

