/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * READINI.H
 *
 * PROGRAMMER : Dan Chang
 *    VERSION : 00.000
 *    CREATED : 01/19/92
 *   MODIFIED : 09/21/94
 *       TABS : 04 07
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		
 *
 * HISTORY
 *
 *		02/27/94 Sunday (GAT) - Added CreateINI, AddINISection, AddINILine
 *		03/02/94 Wednesday (GAT) - Changed ReadINI to AppendINI and added
 *							ReadINI macro
 *	
*/

#ifndef ECHIDNA_READINI_H
#define ECHIDNA_READINI_H

#include <echidna\platform.h>

#ifdef __cplusplus
extern "C" {
#endif

#include <echidna/etypes.h>
#include <echidna\listfunc.h>

/**************************** C O N S T A N T S ***************************/


/******************************** T Y P E S *******************************/

typedef ListType IniList;

typedef struct
{
	NodeType	Node;
	ListType	LineList;
} Section;

typedef struct
{
	NodeType	Node;
	UWORD		LineNo;
} ConfigLine;

typedef struct
{
	ListType *ltSection;
	ConfigLine *clCurrentLine;
} SectionTracker;

/****************************** G L O B A L S *****************************/


/******************************* M A C R O S ******************************/

#define szSectionName NodeName
#define szConfigLine NodeName

#define GetConfigLineNo(configline)	((configline)->LineNo)
#define ReadINI(filename)				AppendINI(NULL,(filename))

/****************** F U N C T I O N   P R O T O T Y P E S *****************/

extern void SetINICaseSensitive(BOOL f);	/* case sensitive section merge, FindSection & FindNextINILine */
extern void SetINISaveBlankLines(BOOL f);	/* Set TRUE to save blank lines as well */
extern void SetINITrimWhiteSpace(BOOL f);	/* Set TRUE to strip leading & trailing WS */
extern void SetINIStripComments(BOOL f);	/* Set TRUE to strip comments starting w/ <szComment> */
extern void SetINICommentAnywhere(BOOL f);/* Set TRUE to strip from <szComment> to end of line */
extern void SetINIMergeSections(BOOL f);	/* Set TRUE to merge lines with same section heading */

extern void SetINIComment(char *sz);		/* string to match for a comment. Eg., szComment = "//" */
extern void SetINISectionMarker(char *sz);/* lines beginning w/ <szSectionMarker> start a section */

extern UWORD GetINIWarnings(void);	/* returns number of warnings in parsing INI file */

extern void FreeINI(IniList *pIni);
extern void PrintINI(IniList *pIniList);
extern IniList *AppendINI(IniList *pIniList, char *filename);

extern IniList *CreateINI(char *pszName);
extern Section *AddINISection (IniList *pIniList, char *pszSectionName);
extern ConfigLine *AddINILine (Section *pst, char *pszIniLine, UWORD lineNo);

extern void ResetSection(SectionTracker *pst);
extern SectionTracker *FindSection(SectionTracker *pst, IniList *pIniList, char *sz);
extern ConfigLine *GetNextINILine(SectionTracker *pst);
extern WORD CountINILines(SectionTracker *pst);
extern char *FindNextINILine(SectionTracker *pst, char *sz);
extern char *FindFirstINILine (SectionTracker *pst, char *sz);

extern BOOL GetFirstBool (SectionTracker *pst, char *sz, BOOL *dst);
extern BOOL GetFirstULong (SectionTracker *pst, char *sz, ULONG *dst);
extern BOOL GetFirstLong (SectionTracker *pst, char *sz, long *dst);
extern BOOL GetFirstUShort (SectionTracker *pst, char *sz, USHORT *dst);
extern BOOL GetFirstShort (SectionTracker *pst, char *sz, short *dst);
extern BOOL GetFirstString (SectionTracker *pst, char *sz, char *dst, size_t maxlen);
extern BOOL GetNextBool (SectionTracker *pst, char *sz, BOOL *dst);
extern BOOL GetNextULong (SectionTracker *pst, char *sz, ULONG *dst);
extern BOOL GetNextLong (SectionTracker *pst, char *sz, long *dst);
extern BOOL GetNextUShort (SectionTracker *pst, char *sz, USHORT *dst);
extern BOOL GetNextShort (SectionTracker *pst, char *sz, short *dst);
extern BOOL GetNextString (SectionTracker *pst, char *sz, char *dst, size_t maxlen);


#ifdef __cplusplus
}
#endif

#endif /* ECHIDNA_READINI_H */
