/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * STRINGS.h
 *
 * PROGRAMMER : Gregg A. Tavares 
 *    VERSION : 00.000
 *    CREATED : 11/01/89
 *   MODIFIED : 10/11/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w-----
 *	 E C H I D N A
 *	--------------
 *
 * DESCRIPTION
 *		Routines for dealing with strings.
 *
 * HISTORY
 *		05/25/91 Saturday (RGM) - Added include for <stddef.h>.
 *		10/06/94 (dcc) - updated to use <echidna/alloc.h>
 *		10/10/94 (dcc) - make DEBUG dupstr() call our routine (call MEM_malloc())
 *
*/

#ifndef ECHIDNA_STRINGS_H
#define ECHIDNA_STRINGS_H

#include <echidna\platform.h>
#include "switches.h"

#ifdef __cplusplus
extern "C" {
#endif

#include <echidna/alloc.h>
#include <string.h>
#include <stdlib.h>
#include <stddef.h>

/******************************* M A C R O S ******************************/
#if (__TURBOC__ || (__ZTC__ && __MSDOS__) || __MACSC__ || __WATCOMC__ || __EGGS_SGI__)
#if DEBUG
extern char		*dupstr (char *s);
#else	/* !DEBUG */
#define dupstr			strdup
#endif	/* !DEBUG */
#endif

#define freestr			MEM_free

/***************************** T Y P E D E F S ****************************/


/****************************** G L O B A L S *****************************/


/****************** F U N C T I O N   P R O T O T Y P E S *****************/
#if __NOT_ANSI__		/** Non-ANSI Prototypes **/



#else					/** ANSI Prototypes **/

extern char		*stristr (char *data, char *key);

extern int		 lchinstr (
							char	 ch,
							char	*str
							);
extern int		 rchinstr (
							char	 ch,
							char	*str
							);
extern int		 rstrchinstr (
							char	*chset,
							char	*str
							);
extern char		*TrimWhiteSpace (char *str);
extern void		 StripComment (char	*s);
extern char		*UnEscStr (char *orig);
extern short	 repstr (char **dest, char *src);
extern short	fstrpcmp (
							char *fpat,
							char *fstr
						);

extern void		*memcpysrc (void *dst, void *src, size_t len);
extern void		*memcpydst (void *dst, void *src, size_t len);
extern short	 eggs_atos (char *s);
extern long		 eggs_atol (char *s);

extern void		 pCharAppend(unsigned char p1, unsigned char *p2);
extern void		 pStrAppend(unsigned char * src,unsigned char * dest);
extern void		 pStrCopy(unsigned char *src,unsigned char *dest);


#if (AZTEC_C)
extern char		*dupstr (char *s);
extern short	 stricmp (char *s1, char *s2);
extern short	 strnicmp (char *s1, char *s2, size_t len);
extern char		*strupr (char *s);
extern char		*itoa (int value, char *string, int radix);
extern char		*ltoa (long value, char *string, int radix);
#endif

#if (__EGGS_SGI__)
#define itoa ltoa
extern char		*ltoa (long value, char *string, int radix);
#endif

#if ( __MACSC__ || __EGGS_SGI__ )
extern int		 stricmp (const char *s1, const char *s2);
extern int		 strnicmp (const char *s1, const char *s2, size_t len);
#endif

#endif /** Function Prototypes **/

#ifdef __cplusplus
}
#endif

#endif /* ECHIDNA_STRINGS_H */

