/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * tUMEsup.h
 *
 * PROGRAMMER : Juan M. Alvarado
 *    VERSION : 00.000
 *    CREATED : 11/06/89
 *   MODIFIED : 07/26/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		Headers and defines for image loading routines.
 *
 * HISTORY
 *	11/30/89 Thursday - Added support for using ExtendedLoadGameMap. (RGM)
 *	12/17/89 Sunday   - Revised for new naming/error standards. (JMA)
 *	12/18/89 Monday   - Added support for 'conversion rooms'. (RGM)
 *	01/02/90 Tuesday  - Added support for no conversion table entry
 *	                    flag (NO_OFFSET). (RGM)
 *	01/15/90 Monday   - Added DrawPartOfRoom(). (GAT)
 *	01/28/90 Sunday   - Added Amiga support for GET_WORD(). (GAT)
*/

#ifndef ECHIDNA_TUMESUP_H
#define ECHIDNA_TUMESUP_H

/***************************** I N C L U D E S ****************************/
#include <echidna/platform.h>

#ifdef __cplusplus
extern "C" {
#endif

#include <echidna/etypes.h>
#include <echidna/graphics.h>

/******************************* M A C R O S ******************************/
#if __iAPx86__
#define GET_WORD(word, hibyte, lobyte)			\
	(*((UBYTE *)&(word))  = (lobyte),			\
	((UBYTE *)&(word))[1] = (hibyte))
#else
#define GET_WORD(word, hibyte, lobyte)			\
	((UBYTE *)&(word))[1] = (lobyte),			\
	(*((UBYTE *)&(word))  = (hibyte))
#endif

/**************************** C O N S T A N T S ***************************/

#define IMAGE_BLOCK_MAX 65536UL /* max image storage chunk allocatable */

#define MAX_ROOM_COPIES	2
#define ROOMNAME_MAXLEN	30

#define NO_OFFSET -1
#define NO_IMAGE 0

/***************************** T Y P E D E F S ****************************/

typedef struct {
#if __MSDOS__
	PatternInfoType		 PatternInfo;
#endif
	UWORD	 			 NumTiles;
	UWORD	 			 NumGlobalTiles;
	UWORD				 NumGlobalTileSets;
	short				 HasGlobals;
	UWORD				*RecoverTable;
	WORD				*TileSetOffsets;
	ULONG				*TileSetAttributes;
	GShape				**TileShapes;
} CTIType;

typedef UBYTE		*LayerType;
typedef LayerType	*RoomType;

typedef struct {
	WORD			 RoomWidth;
	WORD			 RoomHeight;
	WORD			 NumLayers;
	WORD			 TileWidth;
	WORD			 TileHeight;
	UBYTE			 NumRoomCopies;
	PaletteType		 Palette;
	CTIType			*CTI;
	RoomType		*Rooms;
	char			 Name[ROOMNAME_MAXLEN];
} tUMEPackRoomType;

typedef struct {
	UWORD	Layers;		/* number of layers */
	UWORD	Entries;	/* number of tile entries */
	UWORD	Adjustment;	/* subtract from CTI number to get entry index */
	UBYTE	**High;		/* CTptr->High[layer - 1] [CTI - CTptr->Adjustment]*/
	UBYTE	**Low;		/* CTptr->Low[layer - 1] [CTI - CTptr->Adjustment] */
} ConversionTableType;

/* stupid definition to keep at four byte adddress, */
/* needed for the conversion 'in place' in LoadConversionRoom () */
typedef ConversionTableType far32 *ConTablePtrType;

typedef struct {
	UWORD	  			Entries;		/* number of actual tileset entries */
	UWORD	  			Adjustment;		/* - from logic id gets entry index */
	ConTablePtrType		*ConTable;		/* CTptr = CRptr->ConTable[tset] */
	void				*MemBuff;		/* kludge to free up mem */
} ConversionRoomType;

/****************************** G L O B A L S *****************************/

extern CTIType	*GlobalCTIPtr;

/******************************* M A C R O S ******************************/

#define DrawRoom(tproom, room, xoff, yoff)	\
	DrawPartOfRoom ((tproom),				\
					(room),					\
					(xoff),					\
					(yoff),					\
					0,						\
					0,						\
					(tproom)->RoomWidth,	\
					(tproom)->RoomHeight)

#define LoadGlobalCTI(filename, fpfindex)	\
	(GlobalCTIPtr = LowLoadCTI ((filename), (fpfindex), NULL, TRUE))

#define LoadLocalCTI(filename, fpfindex)	\
	LowLoadCTI ((filename), (fpfindex), GlobalCTIPtr, FALSE)

#define LoadCTI(filename, fpfindex)	\
	LowLoadCTI ((filename), (fpfindex), NULL, FALSE)

#define FreeGlobalCTI()		LowFreeCTI (GlobalCTIPtr, TRUE)
#define FreeLocalCTI(cti)	LowFreeCTI ((cti), FALSE)
#define FreeCTI(cti)		LowFreeCTI ((cti), FALSE)

/****************** F U N C T I O N   P R O T O T Y P E S *****************/
#if __NOT_ANSI__		/** Non-ANSI Prototypes **/



#else					/** ANSI Prototypes **/

extern CTIType *LowLoadCTI (
	char	*filename,
	short		 fpfindex,
	CTIType	*globalcti,
	short		 isglobal
	);

extern void	LowFreeCTI (
	CTIType	*cti,
	short		 freeglobal
);	

extern tUMEPackRoomType	*LoadtUMEPackRoom (
	char				*filename,
	short					 fpfindex,
	CTIType				*cti,
	tUMEPackRoomType	*tproomptr,
	short				 numcopies
	);

extern void	FreetUMEPackRoom (
	tUMEPackRoomType *tproomptr
	);

extern ConversionRoomType *LoadConversionRoom (
	char	*filename,
	short		 fpfindex
	);

extern void	FreeConversionRoom (
	ConversionRoomType	*conroom
	);

extern void DrawPartOfRoom (
	tUMEPackRoomType	*tproom,
	short				 room,
	short				 xoff,
	short				 yoff,
	short				 txoff,
	short				 tyoff,
	short				 twidth,
	short				 theight
);

#endif /** Function Prototypes **/

#ifdef __cplusplus
}
#endif

#endif /* ECHIDNA_TUMESUP_H */

