/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * READTGA.C
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 02/16/94
 *   MODIFIED : 03/13/95
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oO///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		Load a Targa file into a BlockO24BitPixels structure.
 *		This code is designed for 32-bit platforms.
 *
 * HISTORY
 *		09/15/94 (GAT) - add support to read Photoshop files
 *
*/

#include <echidna\platform.h>
#include "switches.h"
#include <echidna\proc.h>

#include <stdio.h>
#include <echidna\strings.h>
#include <echidna\etypes.h>
#include <echidna\checkglu.h>
#include <echidna\eerrors.h>
#include <echidna\eio.h>

#include <echidna\read24.h>

#include "photoshp.h"		// GAT 9/15/94

/**************************** C O N S T A N T S ***************************/


/******************************** T Y P E S *******************************/

typedef struct
{
	UBYTE	ID;				// Byte 0
	UBYTE	ctype;			// Byte 1
	UBYTE	itype;			// Byte 2
	UBYTE	mincolorl;		// Byte 3
	UBYTE	mincolorh;		// Byte 4
	UBYTE	colorsl;		// Byte 5
	UBYTE	colorsh;		// Byte 6
	UBYTE	colorsize;		// Byte 7
	UBYTE	originxl;		// Byte 8
	UBYTE	originxh;		// Byte 9
	UBYTE	originyl;		// Byte 10
	UBYTE	originyh;		// Byte 11
	UBYTE	widthl;			// Byte 12
	UBYTE	widthh;			// Byte 13
	UBYTE	heightl;		// Byte 14
	UBYTE	heighth;		// Byte 15
	UBYTE	bpp;			// Byte 16
	UBYTE	idesc;			// Byte 17
} TGAHeader;

/****************************** G L O B A L S *****************************/


/******************************* M A C R O S ******************************/


/***************************** R O U T I N E S ****************************/

/*********************************************************************
 *
 * unloadFile
 *
 * SYNOPSIS
 *		void  unloadFile (void *fileBuffer, char *ID)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
PROC_LOCAL (void , unloadFile, (void *fileBuffer, char *ID))
{
	CHK_DeallocateMemory (fileBuffer, ID);
}
PROC_END // unloadFile


/*********************************************************************
 *
 * loadFile
 *
 * SYNOPSIS
 *		void * loadFile (char *filename, long *size)
 *
 * PURPOSE
 *		Load an entire file into memory
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
PROC_LOCAL (void *, loadFile, (char *filename, long *size))
{
	int		 fh;
	void	*data;
	
	fh = CHK_ReadOpen (filename);

	*size = CHK_FileLength (fh);

	data = CHK_AllocateMemory (*size, filename);

	CHK_Read (fh, data, *size);

	CHK_Close (fh);

	return data;
}
PROC_END // loadFile


/*********************************************************************
 *
 * loadUncompressedTGA
 *
 * SYNOPSIS
 *		void  loadUncompressedTGA (BlockO24BitPixels *blockPtr, unsigned char *fileBufferPtr)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
PROC_LOCAL (void , loadUncompressedTGA, (BlockO24BitPixels *blockPtr, unsigned char *fileBufferPtr))
{

	UBYTE	*alphaPtr;
	UBYTE	*alphaStart;
	UBYTE	*bufferStart;
	UBYTE	*buffer;

	long	 i;
	long	 loop;
	long	 lineSize;
	long	 alphaSize;
	long	 bufferSize;
	long	 bufferWidth;
	long	 bufferHeight;

	bufferWidth  = blockPtr->width;
	bufferHeight = blockPtr->height;
	lineSize     = bufferWidth * 3;
	alphaSize    = bufferWidth * bufferHeight;
	bufferSize   = lineSize * bufferHeight;
	
	bufferStart = (UBYTE *)blockPtr->rgb + bufferSize - lineSize;

	if (blockPtr->channels)
	{
		alphaStart  = (UBYTE *)blockPtr->channel[0] + alphaSize  - bufferWidth;
	}
	
	buffer = bufferStart;
	
	if (!blockPtr->channels)
	{
		// No Alpha / Uncompressed
		
		for (loop = 0; loop < bufferHeight; loop++)
		{
			buffer = bufferStart;
			
			for (i = 0; i < bufferWidth; i++)
			{
				*buffer++ = fileBufferPtr[2];
				*buffer++ = fileBufferPtr[1];
				*buffer++ = fileBufferPtr[0];
				fileBufferPtr += 3;
			}
			
			bufferStart -= lineSize;
		}
	}
	else
	{
		// Alpha / Uncompressed
		
		for (loop = 0; loop < bufferHeight; loop++)
		{
			buffer = bufferStart;
			alphaPtr = alphaStart;
			
			for (i = 0; i < bufferWidth; i++)
			{
				*buffer++ = fileBufferPtr[2];
				*buffer++ = fileBufferPtr[1];
				*buffer++ = fileBufferPtr[0];
				*alphaPtr = fileBufferPtr[3];
				
				alphaPtr++;
				fileBufferPtr += 4;
			}
			
			bufferStart -= lineSize;
			alphaStart  -= bufferWidth;
		}
	}

}
PROC_END // loadUncompressedTGA

/*********************************************************************
 *
 * flipBuffer
 *
 * SYNOPSIS
 *		void  flipBuffer (void *buffer, long rowSize, long rows)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
PROC_LOCAL (void , flipBuffer, (void *buffer, long rowSize, long rows))
{

	UBYTE	*temp;
	UBYTE	*firstRow;
	UBYTE	*lastRow;
			
	long	 count;
	
	temp = (UBYTE *)CHK_AllocateMemory (rowSize, "flipBuffer");
	
	firstRow = (UBYTE *) buffer;
	lastRow  = firstRow + rowSize * (rows - 1);
	
	count = (rows + 1) / 2;
	
	while (count)
	{
		memcpy (temp, lastRow, rowSize);
		memcpy (lastRow, firstRow, rowSize);
		memcpy (firstRow, temp, rowSize);
		
		firstRow += rowSize;
		lastRow  -= rowSize;
		
		count--;
	}
	
	CHK_DeallocateMemory (temp, "flipBuffer");

}
PROC_END // flipBuffer

/*********************************************************************
 *
 * loadCompressedTGA
 *
 * SYNOPSIS
 *		void  loadCompressedTGA (BlockO24BitPixels *blockPtr, UBYTE *fileBufferPtr)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *		02/28/95 (GAT) - Fixed not reading alpha channel on compressed targas.
 *
 * SEE ALSO
 *
*/
PROC_LOCAL (void , loadCompressedTGA, (BlockO24BitPixels *blockPtr, UBYTE *fileBufferPtr))
{
	UBYTE			a;
	UBYTE			r;
	UBYTE			g;
	UBYTE			b;
	UBYTE			i;
	UBYTE			count;
	UBYTE			*alphaBuffer;
	UBYTE			*buffer;

	long			pixelCount;
	long			totalPixels;
	long			bufferWidth;
	long			bufferHeight;

	bufferWidth  = blockPtr->width;
	bufferHeight = blockPtr->height;
	totalPixels  = bufferWidth * bufferHeight;
	buffer       = (UBYTE *) blockPtr->rgb;
	pixelCount   = 0;

	if (blockPtr->channels)
	{
		alphaBuffer = blockPtr->channel[0];
	}
	else
	{
		alphaBuffer = NULL;
	}
	
	while (pixelCount < totalPixels)
	{
		i = *fileBufferPtr++;
		count = (UBYTE) ((0x7F & i) + 1);
		
		pixelCount += count;
		
		if (pixelCount > totalPixels)
		{
			ErrMess ("Pixel overflow in targa decompression\n");
		}
		
		if (i & 0x80)
		{
			// run data

			b = *fileBufferPtr++;
			g = *fileBufferPtr++;
			r = *fileBufferPtr++;

			if (alphaBuffer)
			{
				a = *fileBufferPtr++;
			}

			while (count--)
			{
				*buffer++ = r;
				*buffer++ = g;
				*buffer++ = b;
				if (alphaBuffer)
				{
					*alphaBuffer++ = a;
				}
			}
		}
		else
		{
			// dump data

			while (count--)
			{
				b = *fileBufferPtr++;
				g = *fileBufferPtr++;
				r = *fileBufferPtr++;
			
				*buffer++ = r;
				*buffer++ = g;
				*buffer++ = b;
			
				if (alphaBuffer)
				{
					*alphaBuffer++ = *fileBufferPtr++;
				}
			}
		}
	}
	
	flipBuffer (blockPtr->rgb, blockPtr->width * 3, blockPtr->height);
	
	if (blockPtr->channels)
	{
		flipBuffer (blockPtr->channel[0], blockPtr->width, blockPtr->height);
	}
}
PROC_END // loadCompressedTGA


/*********************************************************************
 *
 * loadTGA
 *
 * SYNOPSIS
 *		void loadTGA (BlockO24BitPixels *blockPtr, char *fileName)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
PROC_LOCAL (void, loadTGA, (BlockO24BitPixels *blockPtr, char *fileName))
{

	TGAHeader		*tgaHeader;
	
	UBYTE			*fileBufferPtr;
	UBYTE			*fileBuffer;

	long			fileSize;
	long			alphachannels;
	long			bufferWidth;
	long			bufferHeight;

	fileBuffer    = (UBYTE *) loadFile (fileName, &fileSize);
	tgaHeader     = (TGAHeader *) fileBuffer;
	fileBufferPtr = fileBuffer + sizeof (TGAHeader);
	
	if (tgaHeader->idesc & 0xf0)
	{
		ErrMess ("Invalid TGA file '%s'", fileName);
	}

	#if 0
	{
		printf ("\n");
		printf ("\t\ttgaHeader->ID          = %d\n", tgaHeader->ID);
		printf ("\t\ttgaHeader->ctype       = %d\n", tgaHeader->ctype);
		printf ("\t\ttgaHeader->itype       = %d\n", tgaHeader->itype);
		printf ("\t\ttgaHeader->mincolorl   = %d\n", tgaHeader->mincolorl);
		printf ("\t\ttgaHeader->mincolorh   = %d\n", tgaHeader->mincolorh);
		printf ("\t\ttgaHeader->colorsl     = %d\n", tgaHeader->colorsl);
		printf ("\t\ttgaHeader->colorsh     = %d\n", tgaHeader->colorsh);
		printf ("\t\ttgaHeader->colorsize   = %d\n", tgaHeader->colorsize);
		printf ("\t\ttgaHeader->originxl    = %d\n", tgaHeader->originxl);
		printf ("\t\ttgaHeader->originxh    = %d\n", tgaHeader->originxh);
		printf ("\t\ttgaHeader->originyl    = %d\n", tgaHeader->originyl);
		printf ("\t\ttgaHeader->originyh    = %d\n", tgaHeader->originyh);
		printf ("\t\ttgaHeader->widthl      = %d\n", tgaHeader->widthl);
		printf ("\t\ttgaHeader->widthh      = %d\n", tgaHeader->widthh);
		printf ("\t\ttgaHeader->heightl     = %d\n", tgaHeader->heightl);
		printf ("\t\ttgaHeader->heighth     = %d\n", tgaHeader->heighth);
		printf ("\t\ttgaHeader->bpp         = %d\n", tgaHeader->bpp);
		printf ("\t\ttgaHeader->idesc       = %d\n", tgaHeader->idesc);
		printf ("\n");
	}
	#endif
	
	fileBufferPtr += tgaHeader->ID; // Skip User Info
	
	if (tgaHeader->bpp == 32)
	{
		alphachannels = TRUE;
	}
	else
	{
		alphachannels = FALSE;
	}
		
	bufferWidth  = ((long)tgaHeader->widthl  + (long)tgaHeader->widthh * 256L);
	bufferHeight = ((long)tgaHeader->heightl + (long)tgaHeader->heighth * 256L);
	
	{
		long	alphaSize;
		long	bufferSize;
				
		alphaSize  = bufferWidth * bufferHeight;
		bufferSize = alphaSize * 3;
		
		blockPtr->rgb    = (UBYTE *) CHK_AllocateMemory (bufferSize, "loadTGA image");
		blockPtr->width  = bufferWidth;
		blockPtr->height = bufferHeight;
		
		if (alphachannels)
		{
			blockPtr->channels   = 1;
			blockPtr->channel    = (UBYTE **)CHK_CallocateMemory (sizeof (UBYTE *), "channel pointers");
			blockPtr->channel[0] = (UBYTE *)CHK_CallocateMemory (alphaSize, "alpha channel");
		}
		
	}
	
	switch (tgaHeader->itype)
	{
	case 2:
		if (((tgaHeader->bpp != 24) && (tgaHeader->bpp != 32)))
		{
			ErrMess ("Unsupported bit depth (%d)", tgaHeader->bpp);
		}
		else
		{
// printf ("load uncompressed tga\n");
			loadUncompressedTGA (blockPtr, fileBufferPtr);
		}
		break;
	
	case 10:
		if (((tgaHeader->bpp != 24) && (tgaHeader->bpp != 32)))
		{
			ErrMess ("Unsupported bit depth (%d)", tgaHeader->bpp);
		}
		else
		{
// printf ("load compressed tga\n");
			loadCompressedTGA (blockPtr, fileBufferPtr);
		}
		break;
	default:
		ErrMess ("Unsupported TGA file type");
		break;
	}
	
	unloadFile (fileBuffer, "loadTGA");

}
PROC_END // loadTGA


/*********************************************************************
 *
 * P24_Read24BitPicture
 *
 * SYNOPSIS
 *		BlockO24BitPixels * P24_Read24BitPicture (char *filename)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *		09/15/94 (GAT) - add support to read Photoshop files
 *
 * SEE ALSO
 *
*/
PROC (BlockO24BitPixels *, P24_Read24BitPicture, (char *filename))
{
	BlockO24BitPixels	*b24;

	b24 = CHK_CallocateMemory (sizeof (BlockO24BitPixels), "BlockO24BitPixels");

	// GAT 9/15/94 ---VVV---
	if (!stricmp (".PSD", EIO_Ext (filename)))
	{
		loadPhotoshop (b24, filename);
	}
	else
	{
		loadTGA (b24, filename);
	}
	// GAT 9/15/94 ---^^^---

	return b24;
}
PROC_END // P24_Read24BitPicture

