/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * ELVTGADG.C
 *
 * PROGRAMMER : Juan M. Alvarado
 *    VERSION : 00.000
 *    CREATED : 02/07/92
 *   MODIFIED : 02/05/95
 *       TABS : 04 08
 *
 *	     \|///-_
 *	     \oO///_
 *	-----w/-w-----
 *	 E C H i D N A
 *	--------------
 *
 * DESCRIPTION
 *		Routines to manage elevator gadget.
 *
 * HISTORY
 *		02/05/95 (dcc) - Comment out calls to SetRectPat() to silence warnings.
 *						 (All are marked with '// KLUDGE ')
 *
*/

#include <echidna/platform.h>
#include "switches.h"

#include <echidna/etypes.h>
#include <echidna/grafx.h>
#include <echidna/hitareas.h>
#include <echidna/eui.h>
#include <echidna/events.h>
#include <echidna/mouse.h>
#include <echidna/scic.h>
#include <echidna/gadgets.h>
#include <echidna/cmdgadg.h>
#include <echidna/elvtgadg.h>
#include <echidna/naming.h>
#include <echidna/eerrors.h>


/**************************** C O N S T A N T S ***************************/

enum {
	iUpArrow, iDnArrow,	iSlider
};

#define iLftArrow	iUpArrow
#define iRgtArrow	iDnArrow	

/******************************** T Y P E S *******************************/
// KLUDGE /* Crazy macros for being able to make an 8 bit mask with 0s and 1s */
// KLUDGE #define _7bit_	0
// KLUDGE #define _7bitX	(128)
// KLUDGE #define _6bit_	0
// KLUDGE #define _6bitX	(64)
// KLUDGE #define _5bit_	0
// KLUDGE #define _5bitX	(32)
// KLUDGE #define _4bit_	0
// KLUDGE #define _4bitX	(16)
// KLUDGE #define _3bit_	0
// KLUDGE #define _3bitX	(8)
// KLUDGE #define _2bit_	0
// KLUDGE #define _2bitX	(4)
// KLUDGE #define _1bit_	0
// KLUDGE #define _1bitX	(2)
// KLUDGE #define _0bit_	0
// KLUDGE #define _0bitX	(1)
// KLUDGE 
// KLUDGE #define _8bitMask(f7,f6,f5,f4,f3,f2,f1,f0)				\
// KLUDGE 	_7bit##f7 | _6bit##f6 | _5bit##f5 | _4bit##f4 | 	\
// KLUDGE 	_3bit##f3 | _2bit##f2 | _1bit##f1 | _0bit##f0 
// KLUDGE  
/****************************** G L O B A L S *****************************/
// KLUDGE 
// KLUDGE #if 0
// KLUDGE static UBYTE rgubShaftPat[8] = {
// KLUDGE 	_8bitMask ( _, _, X, _, _, X, _, _),	
// KLUDGE                                            /**/
// KLUDGE 	_8bitMask ( _, _, X, _, _, X, _, _),	
// KLUDGE                                            /**/
// KLUDGE 	_8bitMask ( X, X, _, _, _, _, X, X),	
// KLUDGE                                            /**/
// KLUDGE 	_8bitMask ( _, _, _, X, X, _, _, _),	
// KLUDGE                                            /**/
// KLUDGE 	_8bitMask ( _, _, _, X, X, _, _, _),	
// KLUDGE                                            /**/
// KLUDGE 	_8bitMask ( X, X, _, _, _, _, X, X),	
// KLUDGE                                            /**/
// KLUDGE 	_8bitMask ( _, _, X, _, _, X, _, _),	
// KLUDGE                                            /**/
// KLUDGE 	_8bitMask ( _, _, X, _, _, X, _, _),	
// KLUDGE                                            /**/
// KLUDGE };
// KLUDGE #elif 0
// KLUDGE static UBYTE rgubShaftPat[8] = {
// KLUDGE 	_8bitMask ( _, _, X, _, _, _, X, _),	
// KLUDGE                                            /**/
// KLUDGE 	_8bitMask ( _, _, X, _, _, _, X, _),	
// KLUDGE                                            /**/
// KLUDGE 	_8bitMask ( X, X, X, _, X, X, X, _),	
// KLUDGE                                            /**/
// KLUDGE 	_8bitMask ( _, _, _, _, _, _, _, _),	
// KLUDGE                                            /**/
// KLUDGE 	_8bitMask ( X, _, _, _, X, _, _, _),	
// KLUDGE                                            /**/
// KLUDGE 	_8bitMask ( X, _, _, _, X, _, _, _),	
// KLUDGE                                            /**/
// KLUDGE 	_8bitMask ( X, _, X, X, X, _, X, X),	
// KLUDGE                                            /**/
// KLUDGE 	_8bitMask ( _, _, _, _, _, _, _, _),	
// KLUDGE                                            /**/
// KLUDGE };
// KLUDGE #else
// KLUDGE static UBYTE rgubShaftPat[8] = {
// KLUDGE 	_8bitMask ( _, X, _, _, _, X, _, _),	
// KLUDGE                                            /**/
// KLUDGE 	_8bitMask ( X, X, _, _, X, X, _, _),	
// KLUDGE                                            /**/
// KLUDGE 	_8bitMask ( _, _, _, X, _, _, _, X),	
// KLUDGE                                            /**/
// KLUDGE 	_8bitMask ( _, _, X, X, _, _, X, X),	
// KLUDGE                                            /**/
// KLUDGE 	_8bitMask ( _, X, _, _, _, X, _, _),	
// KLUDGE                                            /**/
// KLUDGE 	_8bitMask ( X, X, _, _, X, X, _, _),	
// KLUDGE                                            /**/
// KLUDGE 	_8bitMask ( _, _, _, X, _, _, _, X),	
// KLUDGE                                            /**/
// KLUDGE 	_8bitMask ( _, _, X, X, _, _, X, X),	
// KLUDGE                                            /**/
// KLUDGE };
// KLUDGE 
// KLUDGE #endif
/******************************* M A C R O S ******************************/


/***************************** R O U T I N E S ****************************/

void DrawShaftRect (short x, short y, short cx, short cy) {
/* Assumes caller has already called BeforeGraphics(). */
	SetPenColor (GetActiveWindow()->White);
	DrawRect (x, y, cx, cy);
	SetPenColor (GetActiveWindow()->DarkGray);
// KLUDGE 	SetRectPat (rgubShaftPat);
	DrawPatRect (x, y, cx, cy);
}

void MoveSlider (EGAD *pegad, short dpelNew) {

	BeforeGraphics ();

	if (pegad->axis == VERTICAL) {
		short	dy = pegad->S.V.dy;
		short	cx = pegad->cx;
		short	x, yTop;

		if (pegad->fDrawn) {
			/* Erase old slider */
			DrawShaftRect ((x = pegad->x) + 1, 
				(yTop = pegad->S.V.yTop) + dy + (dy == 0),
				cx - 2, 
				cx - (dy == pegad->S.V.cySlack));

			/* Draw new slider */
			DrawBTN (&pegad->rgbtn[iSlider], x, yTop + dpelNew);
			/*cx, cx,
				bdkBlank, NULL, 0);*/
		}
		pegad->S.V.dy = dpelNew;

	} else {
		short dx = pegad->S.H.dx;
		short cy = pegad->cy;
		short y, xLft;

		if (pegad->fDrawn) {
			/* Erase old slider */
			DrawShaftRect ((xLft = pegad->S.H.xLft) + dx + (dx == 0), 
				(y = pegad->y) + 1,
				cy - (dx == pegad->S.H.cxSlack), 
				cy - 2);

			/* Draw new slider */
			DrawBTN (&pegad->rgbtn[iSlider], xLft + dpelNew, y);
			/*, cy, cy,
				bdkBlank, NULL, 0); */
		}
		pegad->S.H.dx = dpelNew;
	}

	AfterGraphics();
} /* MoveSlider */

/*********************************************************************
 *
 * DrawEGAD
 *
 * SYNOPSIS
 *		void	DrawEGAD (EGAD *pegad, short x, short y)
 *
 * PURPOSE
 *		To draw an elevator gadget according to the parameters in the struct.
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void	DrawEGAD (EGAD *pegad, short x, short y)
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "DrawEGAD";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
{
	short cx = pegad->cx;
	short cy = pegad->cy;
	short cpelShaft;
	/* SCIC *pscic = pegad->rgscic;*/
	Window	*pwin = GetActiveWindow ();

	pegad->x = x;
	pegad->y = y;

	BeforeGraphics ();

	/* Outline */
	SetPenColor (pwin->Black);
	DrawBox (x, y, cx, cy);
	
	if (pegad->axis == VERTICAL) {

		pegad->S.V.x = x;
		pegad->S.V.yTop = (y + cx) - 1;
		pegad->S.V.yBotBtn = y + cy - cx; 

		/* Up arrow */
		DrawBTN (&pegad->rgbtn[iUpArrow], x, y); 
		/*, cx, cx,
			bdkSCIC, pscic++, 0);*/

		/* Down arrow */
		DrawBTN (&pegad->rgbtn[iDnArrow], x, pegad->S.V.yBotBtn);
		/*, cx, cx,
			bdkSCIC, pscic, 0);*/

		/* Vertical Slider */
		if ((cpelShaft = cy - 2 * cx) > 0) { 
			DrawShaftRect (x + 1, y + cx, cx - 2, cpelShaft);
		}
		if (pegad->S.V.cySlack > 0) {
			DrawBTN (&pegad->rgbtn[iSlider], 
				x, pegad->S.V.yTop + pegad->S.V.dy);
			/*, cx, cx,
				bdkBlank, NULL, 0);*/
			
		}

	} else 
#if USE_H_ELEVATORS
	/* Draw horizontal elevator if any */
	{
		pegad->S.H.y = y;
		pegad->S.H.xLft = (x + cy) - 1;
		pegad->S.H.xRgtBtn = x + cx - cy;

		/* Left arrow */
		DrawBTN (&pegad->rgbtn[iLftArrow], x, y);
		/*, cy, cy,
			bdkSCIC, pscic++, 0);*/

		/* Right arrow */
		DrawBTN (&pegad->rgbtn[iRgtArrow], pegad->S.H.xRgtBtn, y);
		/*, cy, cy,
			bdkSCIC, pscic, 0);*/

		/* Horizontal Slider */
		if ((cpelShaft = cx - 2 * cy) > 0) { 
			DrawShaftRect (x + cy, y + 1, cpelShaft, cy - 2);
		}
		if (pegad->S.H.cxSlack > 0) {
			DrawBTN (&pegad->rgbtn[iSlider], 
				pegad->S.H.xLft + pegad->S.H.dx, y);
			/*, cy, cy,
				bdkBlank, NULL, 0);*/
			
		}
	}	
#endif

	AfterGraphics ();
	pegad->fDrawn = TRUE;

}
} /* DrawEGAD */


/*********************************************************************
 *
 * Scale
 *
 * SYNOPSIS
 *		short Scale (short Range1, short Range2, i1)
 *
 * PURPOSE
 *		Return i2 where i1 is to Range1 as i2 is to Range2.
 *
 * INPUT
 *		Range1:	Must be non-zero.
 *		Range2:	Must be non-zero.
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short Scale (short Range1, short Range2, short i1)
{
	return (i1 * Range2) / Range1;
} /* Scale */

/*********************************************************************
 *
 * short	SetFloorPositionEGAD
 *
 * SYNOPSIS
 *		extern short	SetFloorPositionEGAD (
 *			EGAD *pegad, 
 *			short FloorsMax,
 *			short iFloorCrnt,
 *			short fDraw,
 *		)
 *
 * PURPOSE
 *		To allow user to set the number of floors and current floor of 
 *		elvator and to redraw it if fDraw is TRUE.	
 *
 * INPUT
 *
 *
 * AFFECTS
 *		Calls user function if one available.
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short	SetFloorPositionEGAD (
	EGAD *pegad, 
	short FloorsMax,
	short iFloorCrnt
)
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "short	SetFloorPositionEGAD";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
{

	if (FloorsMax < 2) return TRUE;

	pegad->FloorsMax = FloorsMax;
	pegad->iFloorCrnt = iFloorCrnt;


	if (pegad->axis == VERTICAL && pegad->S.V.cySlack > 0) {
		short dyNew;
		
		if ((dyNew = Scale (FloorsMax-1, pegad->S.V.cySlack, iFloorCrnt)) !=
			pegad->S.V.dy) 
		{
			MoveSlider (pegad, dyNew);
		}
	} else if (pegad->S.H.cxSlack > 0) {
		short dxNew;

		if ((dxNew = Scale (FloorsMax-1, pegad->S.H.cxSlack, iFloorCrnt)) !=
			pegad->S.H.dx)
		{
			MoveSlider (pegad, dxNew);
		}
	}

	if (pegad->pfnegad) { /* Call user function */
		pegad->pfnegad(pegad->pvUserData);
	}

	return TRUE;
}
} /* SetFloorPositionEGAD */

short EGAD_UpArrow (HitInfo *phi) {
	EGAD 	*pegad = (EGAD *)phi->HitArea->UserData;
	short	 Buttons;

	if ((Buttons = phi->Buttons) & MOUSE_FIRSTDOWN) {
		EUICountDownTimer = pegad->EUITicksTilRepeat;
	}

	if ((Buttons & MOUSE_FIRSTDOWN) || !EUICountDownTimer) {
		if (pegad->iFloorCrnt > 0) {
			SetFloorPositionEGAD (pegad, 
				pegad->FloorsMax, pegad->iFloorCrnt-1);	
		}
	}
	return TRUE;
}

short EGAD_DnArrow (HitInfo *phi) {
	EGAD 	*pegad = (EGAD *)phi->HitArea->UserData;
	short	 Buttons;



	if ((Buttons = phi->Buttons) & MOUSE_FIRSTDOWN) {
		EUICountDownTimer = pegad->EUITicksTilRepeat;
	} 

	if ((Buttons & MOUSE_FIRSTDOWN) || !EUICountDownTimer) {
		if (pegad->iFloorCrnt < pegad->FloorsMax-1) {
			SetFloorPositionEGAD (pegad, 
				pegad->FloorsMax, pegad->iFloorCrnt+1);	
		}
	}
	return TRUE;
}

short EGAD_VSlider (HitInfo *phi) {
	EGAD 	*pegad = (EGAD *)phi->HitArea->UserData;
	short	 dpel = pegad->cx;
	short	 cySlack = pegad->S.V.cySlack;
	short	 FloorsMax = pegad->FloorsMax;
	short	 dyS = pegad->S.V.dy;
	short	 dyHit = phi->Y;

	if (dyHit >= dyS && dyHit < dyS + dpel) 
	{ 
		/* pointer on slider */
		MouseInfo	mi;
		short y = pegad->y;
		short yBotEx = y + pegad->cy;
		short x = pegad->x;
		short xRgtEx = x + dpel;
		short ySTop = pegad->S.V.yTop;
 		short yS = ySTop + dyS;
		short ySBot = ySTop + cySlack;
		short ySStart = yS;
		short dyStart = dyS - dyHit;
		short ySNew = yS; /* make sure they are same the first time */
		short fFirstTime = TRUE;
		short iFloor = pegad->iFloorCrnt;

		do {
			/* Draw ghost slider */
			if (ySNew != yS || fFirstTime) {
				BeforeGraphics ();
				DrawXorBox (x, ySNew, dpel, dpel);  
				AfterGraphics ();
				yS = ySNew;
				fFirstTime = FALSE;
			}
			ReadMouse (&mi);
			if ((ySNew = mi.Y + dyStart) < ySTop) {
				ySNew = ySTop;
			} else if (ySNew > ySBot) {
				ySNew = ySBot;
			}

			if (ySNew != yS || !(mi.Buttons & MOUSE_LEFTBUTTON)) { 
				/* erase old box */
				BeforeGraphics ();
				DrawXorBox (x, yS, dpel, dpel);  
				/* Move slider if RealTime */
				if (pegad->fRealTimeSlider) {
					short iFloorNew = Scale (cySlack, FloorsMax-1, 
						ySNew - ySTop); 
					if (iFloorNew != iFloor) {
						SetFloorPositionEGAD (pegad, FloorsMax, iFloorNew);
						iFloor = iFloorNew;
					}
				}
				AfterGraphics ();
			}
		} while (mi.Buttons & MOUSE_LEFTBUTTON);

		if (yS != ySStart &&
			(mi.Y >= y && mi.Y < yBotEx && mi.X >= x && mi.X < xRgtEx) 
		)
		{
			/* Move slider */
			SetFloorPositionEGAD (pegad, FloorsMax,
				Scale (cySlack, FloorsMax-1, yS - ySTop)); 
		}
	} else { /* Move slider to spot pointed to */
		short dyNew;

		if ((dyNew = dyHit)  > cySlack) {
			dyNew = cySlack;
		} else if (dyNew < 0) {
			dyNew = 0;
		}
		/* Move slider */
		SetFloorPositionEGAD (pegad, FloorsMax,
			Scale (cySlack, FloorsMax-1, dyNew)); 
	}
	return TRUE;
}

#if USE_H_ELEVATORS
short EGAD_LftArrow (HitInfo *phi) {
	EGAD 	*pegad = (EGAD *)phi->HitArea->UserData;
	short	 Buttons;

	if ((Buttons = phi->Buttons) & MOUSE_FIRSTDOWN) {
		EUICountDownTimer = pegad->EUITicksTilRepeat;
	} 

	if ((Buttons & MOUSE_FIRSTDOWN) || !EUICountDownTimer) {
		if (pegad->iFloorCrnt > 0) {
			SetFloorPositionEGAD (pegad, 
				pegad->FloorsMax, pegad->iFloorCrnt-1);	
		}
	}
	return TRUE;
}

short EGAD_RgtArrow (HitInfo *phi) {
	EGAD 	*pegad = (EGAD *)phi->HitArea->UserData;
	short	 Buttons;

	if ((Buttons = phi->Buttons) & MOUSE_FIRSTDOWN) {
		EUICountDownTimer = pegad->EUITicksTilRepeat;
	}

	if ((Buttons & MOUSE_FIRSTDOWN) || !EUICountDownTimer) {
		if (pegad->iFloorCrnt < pegad->FloorsMax-1) {
			SetFloorPositionEGAD (pegad, 
				pegad->FloorsMax, pegad->iFloorCrnt+1);	
		}
	}

	return TRUE;
}

short EGAD_HSlider (HitInfo *phi) {
	EGAD 	*pegad = (EGAD *)phi->HitArea->UserData;
	short	 dpel = pegad->cy;
	short	 cxSlack = pegad->S.H.cxSlack;
	short	 FloorsMax = pegad->FloorsMax;
	short	 dxS = pegad->S.H.dx;
	short	 dxHit = phi->X;

	if (dxHit >= dxS && dxHit < dxS + dpel) 
	{ 
		/* pointer on slider */
		MouseInfo	mi;
		short y = pegad->y;
		short yBotEx = y + dpel;
		short x = pegad->x;
		short xRgtEx = x + pegad->cx;
		short xSLft = pegad->S.H.xLft;
 		short xS = xSLft + dxS;
		short xSEnd = xSLft + cxSlack;
		short xSStart = xS;
		short dxStart = dxS - dxHit;
		short xSNew = xS; /* make sure they are same the first time */
		short fFirstTime = TRUE;
		short iFloor = pegad->iFloorCrnt;

		do {
			/* Draw ghost slider */
			if (xSNew != xS || fFirstTime) {
				BeforeGraphics ();
				DrawXorBox (xSNew, y, dpel, dpel);  
				AfterGraphics ();
				xS = xSNew;
				fFirstTime = FALSE;
			}
			ReadMouse (&mi);
			if ((xSNew = mi.X + dxStart) < xSLft) {
				xSNew = xSLft;
			} else if (xSNew > xSEnd) {
				xSNew = xSEnd;
			}

			if (xSNew != xS || !(mi.Buttons & MOUSE_LEFTBUTTON)) { 
				/* erase old box */
				BeforeGraphics ();
				DrawXorBox (xS, y, dpel, dpel);  
				/* Move slider if RealTime */
				if (pegad->fRealTimeSlider) {
					short iFloorNew = Scale (cxSlack, FloorsMax-1,
						xSNew - xSLft); 
					if (iFloorNew != iFloor) {
						SetFloorPositionEGAD (pegad, FloorsMax, iFloorNew);
						iFloor = iFloorNew;
					}
				}
				AfterGraphics ();
			}
		} while (mi.Buttons & MOUSE_LEFTBUTTON);

		if (xS != xSStart && 
			(mi.Y >= y && mi.Y < yBotEx && mi.X >= x && mi.X < xRgtEx) 
		)
		{
			/* Move slider */
			SetFloorPositionEGAD (pegad, FloorsMax,
				Scale (cxSlack, FloorsMax-1, xS - xSLft)); 
		}
	} else { /* Move slider to spot pointed to */
		short dxNew;

		if ((dxNew = dxHit)  > cxSlack) {
			dxNew = cxSlack;
		} else if (dxNew < 0) {
			dxNew = 0;
		}
		/* Move slider */
		SetFloorPositionEGAD (pegad, FloorsMax,
			Scale (cxSlack, FloorsMax-1, dxNew)); 
	}
	return TRUE;
}
#endif


/*********************************************************************
 *
 * InitEGAD
 *
 * SYNOPSIS
 *		short	InitEGAD (
 *			EGAD	*pegad,
 *			AXIS	 axis,
 *			short	 cpelLnth,	
 *			short	 cpelWdth,	
 *			short	 FloorsMax,
 *			UWORD	 Flags,
 *			short	 c60hzTicksTilRepeat, 
 *			PFNEGAD	 pfnegad,	
 *			void	*pvUserData	
 *		)	
 *
 * PURPOSE
 *		To initialize an elevator gadget.
 *
 * INPUT
 *		axis:	type of elevator (HORIZONTAL or VERTICAL)
 *		cpelLnth:	Length of shaft along axis.
 *		cpelWdth:	Width of shaft perpendicular to axis.
 *		iFloorsMax:		Number of floors elevator services.
 *		Flags:		EGAD_REALTIME_SLIDER 
 *		c60hzTicksTilRepeat:	For arrow buttons. < 0 means no repeat.		
 *		pfnegad:	User function to call when floor changes value.	
 *		pvUserData	User data to pass to user function.
 *	
 * AFFECTS
 *
 *
 * RETURN VALUE
 *		TRUE if succesful.
 *		FALSE if size is too small for gadget.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short	InitEGAD (
	EGAD	*pegad,
	AXIS	 axis,
	short	 cpelLnth,	
	short	 cpelWdth,
	short	 FloorsMax,
	UWORD	 Flags,
	short	 c60hzTicksTilRepeat, 
	PFNEGAD	 pfnegad,	
	void	*pvUserData	
)	
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "InitEGAD";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
{
	HitArea		*pha;
	SCIC		*pscic;
	BTN			*pbtn;
	UWORD		 mskRepeatArrow = ((c60hzTicksTilRepeat >= 0) ?
								HTA_REPEAT : 0);

	/* Shaft must be wide enough for a button and long enough for two 
	 * buttons 
	 */
	if (
		(cpelWdth < cpelEGADButtonMin) ||
		(cpelLnth < (2 * cpelWdth - 1))
	)
	{	
		return FALSE;
	}

	memset (pegad, 0, sizeof (EGAD));
	pegad->axis = axis;
	pegad->FloorsMax = FloorsMax;
	pegad->pfnegad = pfnegad;
	pegad->pvUserData = pvUserData;
	pegad->EUITicksTilRepeat = (short) (((ULONG)c60hzTicksTilRepeat * (ULONG)EUI_TIMER_HERTZ) / 60UL);
	pegad->fRealTimeSlider = Flags & EGAD_REALTIME_SLIDER;

	pha = pegad->rgha;
	pscic = pegad->rgscic;
	pbtn = pegad->rgbtn;
	/* Vertical elevator */
	if (axis == VERTICAL) {
		short cxVArrow = cxVArrowMin + 
				2 * ((cpelWdth - cxPushBTNMin - cxVArrowMin)/ 4);
		short cyVArrow = cyVArrowMin + 
				2 * ((cpelWdth - cyPushBTNMin - cyVArrowMin)/ 4);

		pegad->cx = cpelWdth;
		pegad->cy = cpelLnth;

		/* Up arrow button */
		pscic->pfnscic = Draw_SCIC_UpArrow;
		pscic->cx = cxVArrow;
		pscic->cy = cyVArrow;
		pbtn->cx = cpelWdth;
		pbtn->cy = cpelWdth;
		pbtn->bdk = bdkSCIC;
		pbtn->pvDecal = pscic;
		pbtn->rlpsDecal = rlpsOn;
		pbtn->jstfDecal = jstfCenter;
		pbtn->btnk = btnkPush;
		phaInitCGADHitArea (1, 1, 
			&pegad->rgbtn[iUpArrow], 
			&pegad->rgghi[iUpArrow], 
			EGAD_UpArrow,
			pegad,
			pha,
			HTA_DOWN | mskRepeatArrow,
			0
		);
		pha->Height = pha->Width = cpelWdth - 2;
		++pha;
		++pscic;
		++pbtn;

		/* bottom arrow button */
		pscic->pfnscic = Draw_SCIC_DnArrow;
		pscic->cx = cxVArrow;
		pscic->cy = cyVArrow;;
		pbtn->cx = cpelWdth;
		pbtn->cy = cpelWdth;
		pbtn->bdk = bdkSCIC;
		pbtn->pvDecal = pscic;
		pbtn->rlpsDecal = rlpsOn;
		pbtn->jstfDecal = jstfCenter;
		pbtn->btnk = btnkPush;
		phaInitCGADHitArea (1, (cpelLnth - cpelWdth) + 1, 
			&pegad->rgbtn[iDnArrow], 
			&pegad->rgghi[iDnArrow], 
			EGAD_DnArrow,
			pegad,
			pha,
			HTA_DOWN | mskRepeatArrow,
			0
		);
		pha->Height = pha->Width = cpelWdth - 2;
		++pha;
		++pbtn;

		/* slider */
		pegad->S.V.cySlack = cpelLnth - 3 * cpelWdth + 2;
		if (pegad->S.V.cySlack > 0) {
			pbtn->cx = cpelWdth;
			pbtn->cy = cpelWdth;
			pbtn->bdk = bdkBlank;
			pbtn->rlpsDecal = rlpsOn;
			pbtn->jstfDecal = jstfCenter;
			pbtn->btnk = btnkPush;
			pha->X = 1;
			pha->Y = cpelWdth;
			pha->Width = cpelWdth - 2;
			pha->Height = cpelLnth - 2 * cpelWdth;
			pha->Flags = HTA_DOWN;
			pha->Data = EGAD_VSlider;
			pha->UserData = pegad;
			++pha;
		}
	} else 
#if USE_H_ELEVATORS
	{
		short cxHArrow =  cxHArrowMin +
				2 * ((cpelWdth - cxPushBTNMin - cxHArrowMin) / 4);
		short cyHArrow = cyHArrowMin + 
				2 * ((cpelWdth - cyPushBTNMin - cyHArrowMin) / 4);

		pegad->cx = cpelLnth;
		pegad->cy = cpelWdth;
		/* Left arrow button */
		pscic->pfnscic = Draw_SCIC_LftArrow;
		pscic->cx = cxHArrow;
		pscic->cy = cyHArrow;
		pbtn->cx = cpelWdth;
		pbtn->cy = cpelWdth;
		pbtn->bdk = bdkSCIC;
		pbtn->pvDecal = pscic;
		pbtn->rlpsDecal = rlpsOn;
		pbtn->jstfDecal = jstfCenter;
		pbtn->btnk = btnkPush;
		phaInitCGADHitArea (1, 1, 
			&pegad->rgbtn[iLftArrow], 
			&pegad->rgghi[iLftArrow], 
			EGAD_LftArrow,
			pegad,
			pha,
			HTA_DOWN | mskRepeatArrow,
			0
		);
		pha->Height = pha->Width = cpelWdth - 2;
		++pha;
		++pscic;
		++pbtn;

		/* Right arrow button */
		pscic->pfnscic = Draw_SCIC_RgtArrow;
		pscic->cx = cxHArrow;
		pscic->cy = cyHArrow;
		pbtn->cx = cpelWdth;
		pbtn->cy = cpelWdth;
		pbtn->bdk = bdkSCIC;
		pbtn->pvDecal = pscic;
		pbtn->rlpsDecal = rlpsOn;
		pbtn->jstfDecal = jstfCenter;
		pbtn->btnk = btnkPush;
		phaInitCGADHitArea ((cpelLnth - cpelWdth) + 1, 1, 
			&pegad->rgbtn[iRgtArrow], 
			&pegad->rgghi[iRgtArrow], 
			EGAD_RgtArrow,
			pegad,
			pha,
			HTA_DOWN | mskRepeatArrow,
			0
		);
		pha->Height = pha->Width = cpelWdth - 2;
		++pha;
		++pbtn;

		/* slider */
		pegad->S.H.cxSlack = cpelLnth - 3 * cpelWdth + 2;
		if (pegad->S.H.cxSlack > 0) {
			pbtn->cx = cpelWdth;
			pbtn->cy = cpelWdth;
			pbtn->bdk = bdkBlank;
			pbtn->rlpsDecal = rlpsOn;
			pbtn->jstfDecal = jstfCenter;
			pbtn->btnk = btnkPush;
			pha->X = cpelWdth;
			pha->Y = 1;
			pha->Width = cpelLnth - 2 * cpelWdth;
			pha->Height = cpelWdth - 2;
			pha->Flags = HTA_DOWN;
			pha->Data = EGAD_HSlider;
			pha->UserData = pegad;
			++pha;
		}
	}
#endif

	/* end area */
	MAKE_END_HITAREA (pha);


	return TRUE;

}
} /* InitEGAD */

