/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * FILEREQ.C
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 12/11/90
 *   MODIFIED : 10/17/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		
 *
 * HISTORY
 *		12/04/92 (GAT) - Added SetDirSortType()
 *		12/04/92 (GAT) - Made list move to match what's typed.
 *		12/04/92 (GAT) - Added PgUp/PgDn support
 *		03/16/93 (dcc) - fix typing 'z' moving list too far (crashed).
 *		03/22/93 (dcc) - press [Backspace] at start of line move up a dir.
 *					   - press [SPACE] completes name
 *					   - highlighter changes box even if dir or drive
 *					   - pressing [Enter] w/ sub-dir enters sub-dir
 *					   - drives do not default to root dir (no end '\\')
 *					   - press '\\' at start of line moves to root dir
 *					   - press '\\' after sub-dir moves into sub-dir
 *		03/23/93 (dcc) - use .. instead of [bksp] at start of line to
 *						 move up a directory
 *		04/01/93 (dcc) - move highlighter to nearest match on entry
 *		04/01/93 (dcc) - add RestoreOriginalDrive(), which allows
 *						 you to restore the original drive & path
 *						 when you exit the program (broken 03/22/93)
 *		10/12/93 (dcc) - updated to support MS-DOS Watcom C32.
 *		10/06/94 (dcc) - updated to use <echidna/alloc.h>
 *
 * TODO
 *		* Fix edit path to show all lower dirs like Mac.
*/

#include <echidna/platform.h>
#include "switches.h"

#include <stdio.h>
#include <ctype.h>

#include <echidna/alloc.h>
#include <echidna/eio.h>
#include <echidna/strings.h>
#include <echidna/listfunc.h>
#include <echidna/eerrors.h>
#include <echidna/grafx.h>
#include <echidna/fonts.h>
#include <echidna/hitareas.h>
#include <echidna/windows.h>
#include <echidna/eui.h>
#include <echidna/mouse.h>
#include <echidna/edraw.h>
#include <echidna/keyboard.h>

#include <echidna/emacros.h>

#include "edit.h"
#if __WATCOMC__
#include "diskp.h"
#endif

/**************************** C O N S T A N T S ***************************/

#if 0
#define FILE_TYPE	0
#define DRAWER_TYPE	1
#define DISK_TYPE	2
#else
#define FILE_TYPE	1
#define DRAWER_TYPE	0
#define DISK_TYPE	2
#endif

#define LOCAL static

#define FR_LEFT		20
#define FR_TOP		20
#define FR_WIDTH	189
#define FR_HEIGHT	119

#define FR_WHITE	0
#define FR_BLACK	1
#define FR_BLUE		2
#define FR_ORANGE	3

#define PATH_X		3
#define PATH_Y		14
#define PATH_WIDTH	182
#define PATH_HEIGHT	11
#define PATH_ID		1

#define NAME_X		3
#define NAME_Y		103
#define NAME_WIDTH	182
#define NAME_HEIGHT	12
#define NAME_ID		2

#define LIST_X		3
#define LIST_Y		27
#define LIST_WIDTH	124
#define LIST_HEIGHT	74
#define LIST_ID		3
#define LIST_LINES	8
#define LINE_HEIGHT	9

#define ARROW_X	 		(LIST_X + 113)
#define ARROW_TOP		(LIST_Y + 10)
#define ARROW_BOT		(LIST_Y + 63)
#define ARROW_WIDTH		11
#define ARROW_HEIGHT	11
#define UPARROW_ID		4
#define DOWNARROW_ID	5
#define ELEVATOR_ID		6

#define ELEVATOR_X		(ARROW_X + 1)
#define ELEVATOR_TOP	(ARROW_TOP + 1)
#define ELEVATOR_WIDTH	(ARROW_WIDTH - 2)
#define ELEVATOR_MIN	4
#define ELEVATOR_HEIGHT	(ARROW_BOT - ARROW_TOP - 1)

#define BUTTS_X			129
#define BUTTS_WIDTH		56
#define BUTTS_HEIGHT	12
#define FILES_Y			33
#define DIRS_Y			47
#define DISKS_Y			61
#define CANCEL_Y		75
#define OKAY_Y			89

#define FILES_ID		11
#define DIRS_ID			10
#define DISKS_ID		7
#define CANCEL_ID		8
#define OKAY_ID			9

/******************************** T Y P E S *******************************/

typedef struct {
	NodeType	 Node;
	UWORD		 Type;
} FNNode;

/****************************** G L O B A L S *****************************/


/******************************* M A C R O S ******************************/


/****************************** E X T E R N S *****************************/


/************************ G L O B A L   L O C A L S ***********************/

LOCAL ListType	 NameListX;
LOCAL ListType	*NameList = &NameListX;

LOCAL Window		*FRWin;
LOCAL Font			*FRFont;
LOCAL EasyFileReq	*FileReq;
LOCAL char			 FRPath[EIO_MAXPATH];
LOCAL char			 FRName[EIO_MAXPATH];
LOCAL char			 LastUsedPath[EIO_MAXPATH];
LOCAL short			 FRdone;
LOCAL short			 FRabort;

LOCAL short			 DiskPos;

LOCAL short			 ELheight;
LOCAL short			 ELarea;
LOCAL short			 ELpos;
LOCAL short			 ELfirst;
LOCAL short			 ELorigp;
LOCAL short			 ELin;

LOCAL short			 DirsAtTop;

LOCAL short			 HLpos;
LOCAL FNNode		*HLnode;

LOCAL EditString NameString = {
	FRName, (NAME_WIDTH - 5) / 8, FR_LEFT + NAME_X + 2, FR_TOP + NAME_Y + 2,
};

LOCAL int			 wOriginalDrive;

/******************************* T A B L E S ******************************/

LOCAL char *s0 = "Disks";
LOCAL char *s1 = "Cancel";
LOCAL char *s2 = "Okay";
LOCAL char *s3 = "Dirs";
LOCAL char *s4 = "Files";

LOCAL char **FrStr[] = {
#define DISKS_STR	0
	&s0,
#define CANCEL_STR	1
	&s1,
#define OKAY_STR	2
	&s2,
#define DIRS_STR	3
	&s3,
#define FILES_STR	4
	&s4,
};

LOCAL struct TextFont **FrFonts[] = {
	(struct TextFont **)&FRFont,
};

LOCAL EPic FileEPic[] = {
	ED_Font (0),
	ED_DrawMode (JAM2),
	ED_FgColorTbl (FR_WHITE),
	ED_Rect (0, 0, FR_WIDTH, FR_HEIGHT),
	ED_FgColorTbl (FR_ORANGE),
	ED_Rect (1, 1, FR_WIDTH - 3, 10),
	ED_DrawMode (JAM1),
	ED_FgColorTbl (FR_BLACK),
	ED_Rect (0, 0, FR_WIDTH - 1, FR_HEIGHT - 1),
	ED_RelVLine (FR_WIDTH - 1, 1, FR_HEIGHT - 1),
	ED_RelHLine (1,FR_WIDTH - 1, FR_HEIGHT - 1),
	ED_RelHLine (1,FR_WIDTH - 1, 11),
	ED_Rect (PATH_X, PATH_Y, PATH_WIDTH, PATH_HEIGHT),
	ED_Rect (NAME_X, NAME_Y, NAME_WIDTH, NAME_HEIGHT),
	ED_Rect (LIST_X, LIST_Y, LIST_WIDTH, LIST_HEIGHT),
	ED_RelVLine (ARROW_X, LIST_Y, LIST_HEIGHT),
	ED_RelHLine (ARROW_X, 10, ARROW_TOP),
	ED_RelHLine (ARROW_X, 10, ARROW_BOT),
	ED_DrawMode (JAM2),
	ED_Rect (ARROW_X + 4, ARROW_TOP - 2, 3, -6),
	ED_Rect (ARROW_X + 4, ARROW_BOT + 2, 3,  6),
	ED_DrawMode (JAM1),

	ED_Plot (ARROW_X + 2, ARROW_TOP - 5),
	ED_Plot (ARROW_X + 3, ARROW_TOP - 5),
	ED_Plot (ARROW_X + 3, ARROW_TOP - 6),
	ED_Plot (ARROW_X + 5, ARROW_TOP - 8),
	ED_Plot (ARROW_X + 7, ARROW_TOP - 6),
	ED_Plot (ARROW_X + 7, ARROW_TOP - 5),
	ED_Plot (ARROW_X + 8, ARROW_TOP - 5),

	ED_Plot (ARROW_X + 2, ARROW_BOT + 5),
	ED_Plot (ARROW_X + 3, ARROW_BOT + 5),
	ED_Plot (ARROW_X + 3, ARROW_BOT + 6),
	ED_Plot (ARROW_X + 5, ARROW_BOT + 8),
	ED_Plot (ARROW_X + 7, ARROW_BOT + 6),
	ED_Plot (ARROW_X + 7, ARROW_BOT + 5),
	ED_Plot (ARROW_X + 8, ARROW_BOT + 5),

	ED_Rect (BUTTS_X, DISKS_Y, BUTTS_WIDTH - 1, BUTTS_HEIGHT - 1),
	ED_RelHLine (BUTTS_X + 1, BUTTS_WIDTH - 1, DISKS_Y + BUTTS_HEIGHT - 1),
	ED_RelVLine (BUTTS_X + BUTTS_WIDTH - 1, DISKS_Y + 1, BUTTS_HEIGHT - 1),
	ED_PosText (BUTTS_X + 2, DISKS_Y + 2, DISKS_STR), 

	ED_Rect (BUTTS_X, OKAY_Y, BUTTS_WIDTH - 1, BUTTS_HEIGHT - 1),
	ED_RelHLine (BUTTS_X + 1, BUTTS_WIDTH - 1, OKAY_Y + BUTTS_HEIGHT - 1),
	ED_RelVLine (BUTTS_X + BUTTS_WIDTH - 1, OKAY_Y + 1, BUTTS_HEIGHT - 1),
	ED_PosText (BUTTS_X + 2, OKAY_Y + 2, OKAY_STR), 

	ED_Rect (BUTTS_X, CANCEL_Y, BUTTS_WIDTH - 1, BUTTS_HEIGHT - 1),
	ED_RelHLine (BUTTS_X + 1, BUTTS_WIDTH - 1, CANCEL_Y + BUTTS_HEIGHT - 1),
	ED_RelVLine (BUTTS_X + BUTTS_WIDTH - 1, CANCEL_Y + 1, BUTTS_HEIGHT - 1),
	ED_PosText (BUTTS_X + 2, CANCEL_Y + 2, CANCEL_STR), 

	ED_Rect (BUTTS_X, DIRS_Y, BUTTS_WIDTH - 1, BUTTS_HEIGHT - 1),
	ED_RelHLine (BUTTS_X + 1, BUTTS_WIDTH - 1, DIRS_Y + BUTTS_HEIGHT - 1),
	ED_RelVLine (BUTTS_X + BUTTS_WIDTH - 1, DIRS_Y + 1, BUTTS_HEIGHT - 1),
	ED_PosText (BUTTS_X + 2, DIRS_Y + 2, DIRS_STR), 

	ED_Rect (BUTTS_X, FILES_Y, BUTTS_WIDTH - 1, BUTTS_HEIGHT - 1),
	ED_RelHLine (BUTTS_X + 1, BUTTS_WIDTH - 1, FILES_Y + BUTTS_HEIGHT - 1),
	ED_RelVLine (BUTTS_X + BUTTS_WIDTH - 1, FILES_Y + 1, BUTTS_HEIGHT - 1),
	ED_PosText (BUTTS_X + 2, FILES_Y + 2, FILES_STR), 

	ED_End,
};

LOCAL UBYTE FrColors[4];

LOCAL EPicture FREPicture = {
	FileEPic,
	FrFonts,
	FrStr,
	NULL,
	NULL,
	FrColors,
};

LOCAL EPic PointEP[] = {
	ED_FgColorTbl (FR_BLACK),
	ED_RelVLine (0, 0, 5),
	ED_RelVLine (1, 1, 3),
	ED_Plot (2, 2),
	ED_End,
};

LOCAL EPic *FREPics[] = {
	PointEP,
};

LOCAL EPic DrawerEPic[] = {
	ED_FgColorTbl (FR_BLACK),
	ED_DrawMode (JAM1),
	ED_Rect (0, 1, 10, 6),
	ED_RelHLine (3, 4, 4),
	ED_Plot (3, 3),
	ED_Plot (6, 3),
	ED_Move (11, 1),
	ED_Gosub (0),
	ED_End,
};

LOCAL EPicture FRDrawerEP = {
	DrawerEPic,
	NULL,
	NULL,
	FREPics,
	NULL,
	FrColors,
};

LOCAL EPic DiskEPic[] = {
	ED_FgColorTbl (FR_BLACK),
	ED_DrawMode (JAM1),
	ED_Rect (2, 0, 8, 7),
	ED_Rect (4, 3, 4, 4),
	ED_FgColorTbl (FR_WHITE),
	ED_Plot (2, 6),
	ED_Move (11, 1),
	ED_Gosub (0),
	ED_End,
};

LOCAL EPicture FRDiskEP = {
	DiskEPic,
	NULL,
	NULL,
	FREPics,
	NULL,
	FrColors,
};

/***************************** R O U T I N E S ****************************/


/*********************************************************************
 *
 * UpdatePath
 *
 * SYNOPSIS
 *		LOCAL short UpdatePath (EasyFileReq *fr)
 *
 * PURPOSE
 *		Display current path
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL short UpdatePath (EasyFileReq *fr)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "UpdatePath";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		char	 work[EIO_MAXPATH];
		short	 p;

		fr = fr;
		EIO_fnsplit (FRPath, work, NULL, NULL);

		p = strlen(work) - ((PATH_WIDTH - 4) / 8);
		p = max (p, 0);

		SetPenColor (FrColors[FR_WHITE]);
		DrawRect (FR_LEFT + PATH_X + 1, FR_TOP + PATH_Y + 1, PATH_WIDTH - 2, PATH_HEIGHT - 2);
		DrawString (FRFont, FR_LEFT + PATH_X + 2, FR_TOP + PATH_Y + 2, work + p);
	}

	return TRUE;

} /* UpdatePath */

/*********************************************************************
 *
 * UpdateElevator
 *
 * SYNOPSIS
 *		LOCAL void UpdateElevator (EasyFileReq *fr)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL void UpdateElevator (EasyFileReq *fr)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "UpdateElevator";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		short	fcount;
		short	fpos;

		fcount   = fr->FileCount;
		fpos     = fr->Position;
		ELheight = (ELEVATOR_HEIGHT * LIST_LINES) / fcount;
		ELheight = min (ELheight, ELEVATOR_HEIGHT);
		ELheight = max (ELheight, ELEVATOR_MIN);
		ELarea   = ELEVATOR_HEIGHT - ELheight;
		if (ELarea) {
			ELpos = (fpos * ELarea) / (fcount - LIST_LINES);
		} else {
			ELpos = 0;
		}

		SetPenColor (FrColors[FR_WHITE]);
		DrawRect (FR_LEFT + ELEVATOR_X, FR_TOP + ELEVATOR_TOP, ELEVATOR_WIDTH, ELEVATOR_HEIGHT);
		SetPenColor (FrColors[FR_BLACK]);
		if (ELpos + ELheight - 1 > ELEVATOR_HEIGHT) {
			ELpos = ELpos;
		}
		DrawBox (FR_LEFT + ELEVATOR_X, FR_TOP + ELEVATOR_TOP + ELpos, ELEVATOR_WIDTH, ELheight);
	}
} /* UpdateElevator */

/*********************************************************************
 *
 * DrawFileName
 *
 * SYNOPSIS
 *		LOCAL void DrawFileName (short position, FNNode *fn)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL void DrawFileName (short position, FNNode *fn)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "DrawFileName";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		struct RastPort	rp;

		position *= LINE_HEIGHT;
		SetPenColor (FrColors[FR_WHITE]);
		DrawRect (FR_LEFT + LIST_X + 1, FR_TOP + LIST_Y + 1 + position, LIST_WIDTH - ARROW_WIDTH - 1, LINE_HEIGHT);
		if (fn) {
			switch (fn->Type) {
			case DRAWER_TYPE:
				StartEPicture (&rp, &FRDrawerEP, FR_LEFT + LIST_X + 2, FR_TOP + LIST_Y + 2 + position);
				break;
			case DISK_TYPE:
				StartEPicture (&rp, &FRDiskEP, FR_LEFT + LIST_X + 2, FR_TOP + LIST_Y + 2 + position);
				break;
			}
			DrawString (FRFont, FR_LEFT + LIST_X + 17, FR_TOP + LIST_Y + 2 + position, NodeName (fn));
		}
	}

} /* DrawFileName */

/*********************************************************************
 *
 * HiliteName
 *
 * SYNOPSIS
 *		LOCAL void HiliteName (short position)
 *
 * PURPOSE
 *		Hilite selected name
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL void HiliteName (short position)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "HiliteName";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		position *= LINE_HEIGHT;
		SetPenColor (FrColors[FR_WHITE] ^ FrColors[FR_BLACK]);
		XORRect (FR_LEFT + LIST_X + 1, FR_TOP + LIST_Y + 1 + position, LIST_WIDTH - ARROW_WIDTH - 1, LINE_HEIGHT);
	}

} /* HiliteName */

/*********************************************************************
 *
 * UpdateNameList
 *
 * SYNOPSIS
 *		short UpdateNameList (EasyFileReq *fr)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL short UpdateNameList (EasyFileReq *fr)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "UpdateNameList";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		short	 position;
		short	 hpos;
		FNNode	*fn;

		position = fr->Position;
		if (position > fr->FileCount - LIST_LINES) {
			position = fr->FileCount - LIST_LINES;
		}
		position     = max (position, 0);
		fr->Position = position;
		hpos         = position;

		fn = Head (NameList);
		while (position) {
			fn = Next (fn);
			position--;
		}
		fr->Top = fn;

		BeforeGraphics ();
		position = 0;
		HLnode   = NULL;
		while (position < LIST_LINES && !IsEOList (fn)) {
			DrawFileName (position, fn);
			if (hpos == HLpos) {
				HiliteName (position);
				HLnode = fn;
			}
			fn = Next (fn);
			hpos++;
			position++;
		}
		while (position < LIST_LINES) {
			DrawFileName (position, NULL);
			position++;
		}
		UpdateElevator (fr);
		AfterGraphics ();
	}
	return TRUE;

} /* UpdateNameList */

/*********************************************************************
 *
 * CreateNameList
 *
 * SYNOPSIS
 *		LOCAL short CreateNameList (void)
 *
 * PURPOSE
 *		Read filenames from Current Directory into linked list
 *		and add drive id's.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *	04/29/93 Thursday (dcc) - fix function so it finds network drives again
 *						(originally fixed 05/13/92, broke 02/18/93) (sigh)
 *
 * SEE ALSO
 *
*/
LOCAL short CreateNameList (EasyFileReq *fr)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "CreateNameList";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		short		 aokay   = TRUE;

		HLpos = -1;
		UpdatePath (fr);
		EmptyList (NameList);
		fr->FileCount = 0;
		{
			FNNode		*fn;
			FNNode		*sh;
			DirTracker	*dt      = NULL;
			char		*path;

			path = FRPath;
			dt = EIO_GetFirstFile (path, TRUE);
			if (dt) {
				while (aokay && dt->Status) {
					if (strcmp (dt->Path, ".") && strcmp (dt->Path, "..")) {
						fn = CreateNode (sizeof (FNNode), dt->Path);
						if (!fn) {
							SetGlobalErr (ERR_OUT_OF_MEMORY);
							GEcatf ("\nCreateNameList: OOM FNNode");
							break;
						}
						fr->FileCount++;
						fn->Type = (dt->IsDir) ? DRAWER_TYPE : FILE_TYPE;
						sh = Head (NameList);
						for (;;) {
							if (IsEOList (sh)) {
								break;
							}
							if (DirsAtTop)
							{
								if (fn->Type < sh->Type) {
									break;
								}
								if (fn->Type == sh->Type &&
									stricmp (NodeName(fn), NodeName(sh)) < 0) {
									break;
								}
							}
							else
							{
								if (stricmp (NodeName(fn), NodeName(sh)) < 0) {
									break;
								}
							}
							sh = Next (sh);
						}
						InsertBefore (sh, fn);
					}
					aokay = EIO_GetNextFile (dt);
				}
				EIO_FreeDirTracker (dt);
			}
		}
		{
			BOOL		 fOneFloppy;
			FNNode		*fn;
			int		 drives;
			int		 curdrive;
			int		 i;
			char		 s[3] = "A:";

			fOneFloppy = SingleDrive();		/* Is this an one floppy machine? */
			s[0] = 'A';		SetFloppy(s);	/* Yes, force drive one to "A:" */

			drives = setdisk(curdrive = getdisk());	/* Find total number of drives */

			for (i=0; i <= drives; i++) {
				if (!fOneFloppy || i != 1) {/* If one floppy & "B:", skip it */
					DiskPos   = fr->FileCount;

					setdisk(i);
					if (getdisk() == i) {
						s[0] = i + 'A';
						fn   = CreateNode (sizeof (FNNode), s);
						if (!fn) {
							SetGlobalErr (ERR_OUT_OF_MEMORY);
							GEcatf ("\nCreateNameList: OOM FNNode (drives)");
							break;
						}
						fr->FileCount++;
						fn->Type = DISK_TYPE;
						AddTail (NameList, fn);
					}
				}
			}
			setdisk(curdrive);			/* restore current drive setting */
		}
		#if 0
		{
			FNNode	*sh;

			printf ("Dir of '%s'\n", path);
			sh = Head (NameList);
			while (!IsEOList (sh)) {
				printf ("\t'%14s', %d\n", NodeName (sh), sh->Type);
				sh = Next (sh);
			}
		}
		#endif

		return aokay;
	}
} /* CreateNameList */

/*********************************************************************
 *
 * ScrollUp
 *
 * SYNOPSIS
 *		LOCAL short ScrollUp (HitInfo *hi)
 *
 * PURPOSE
 *		Scroll Down the list (or move up the list)
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL short ScrollUp (HitInfo *hi)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ScrollUp";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		static short first = TRUE;

		if (hi->Buttons) {
			if (first) {
				first = FALSE;
				EUICountDownTimer = 0;
			} else {
				if (!EUICountDownTimer && FileReq->Position) {
					FileReq->Position--;
					UpdateNameList (FileReq);
					EUICountDownTimer = EUI_TIMER_HERTZ / 20;
				}
			}
		} else {
			first = TRUE;
		}
	}
	return TRUE;

} /* ScrollUp */

/*********************************************************************
 *
 * ScrollDown
 *
 * SYNOPSIS
 *		LOCAL short ScrollDown (HitInfo *hi)
 *
 * PURPOSE
 *		Scroll List Up (or move down the list)
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL short ScrollDown (HitInfo *hi)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ScrollDown";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		static short first = TRUE;

		if (hi->Buttons) {
			if (first) {
				first = FALSE;
				EUICountDownTimer = 0;
			} else {
				if (!EUICountDownTimer && FileReq->Position < FileReq->FileCount - LIST_LINES) {
					FileReq->Position++;
					UpdateNameList (FileReq);
					EUICountDownTimer = EUI_TIMER_HERTZ / 20;
				}
			}
		} else {
			first = TRUE;
		}
	}
	return TRUE;

} /* ScrollDown */

/*********************************************************************
 *
 * Elevator
 *
 * SYNOPSIS
 *		LOCAL short Elevator (HitInfo *hi)
 *
 * PURPOSE
 *		Allow user to drag elevator and re-display list accordingly.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL short Elevator (HitInfo *hi)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "Elevator";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		static short first = TRUE;
		short		 fcount;
		short		 fpos;
		short		 oldfp;
		short		 y;

		y = hi->Y;
		if (hi->Buttons) {
			if (first) {
				first   = FALSE;
				ELfirst = y;
				ELorigp = ELpos;
				if (y >= ELpos && y < ELpos + ELheight) {
					ELin = TRUE;
				} else {
					ELin = FALSE;
					if (y < ELpos) {
						FileReq->Position -= 8;
					} else {
						FileReq->Position += 8;
					}
					UpdateNameList (FileReq);
				}
			} else {
				if (ELin && ELarea) {
					ELpos  = ELorigp + (y - ELfirst);
					ELpos  = max (0, ELpos);
					ELpos  = min (ELpos, ELarea);
					fcount = FileReq->FileCount;
					fpos   = FileReq->Position;
					oldfp  = fpos;
					fpos   = ((fcount - LIST_LINES) * ELpos) / ELarea;
					if (fpos > fcount - LIST_LINES) {
						fpos = fcount - LIST_LINES;
					}
					if (oldfp != fpos) {
						FileReq->Position = fpos;
						UpdateNameList (FileReq);
					}
				}
			}
		} else {
			first = TRUE;
		}
	}

	return TRUE;
} /* Elevator */

/*********************************************************************
 *
 * ShowDisks
 *
 * SYNOPSIS
 *		LOCAL short ShowDisks (HitInfo *hi)
 *
 * PURPOSE
 *		Display Available Disks in List Area
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL short ShowDisks (HitInfo *hi)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ShowDisks";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	hi = hi;
	{
		short	fpos;
		short	fcount;

		fpos   = DiskPos;
		fcount = FileReq->FileCount;
		if (fpos > fcount - LIST_LINES) {
			fpos = fcount - LIST_LINES;
		}
		FileReq->Position = fpos;
		UpdateNameList (FileReq);
	}

	return TRUE;
} /* ShowDisks */

/*********************************************************************
 *
 * ShowDirs
 *
 * SYNOPSIS
 *		LOCAL short ShowDirs (HitInfo *hi)
 *
 * PURPOSE
 *		Display Available Dirs in List Area
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL short ShowDirs (HitInfo *hi)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ShowDirs";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	hi = hi;
	{
		short	 fpos;
		FNNode	*fn;

		fpos = 0;
		fn   = Head (NameList);
		while (!IsEOList (fn)) {
			if (fn->Type == DRAWER_TYPE) {
				FileReq->Position = fpos;
				UpdateNameList (FileReq);
				break;
			}
			fpos++;
			fn = Next (fn);
		}

	}

	return TRUE;
} /* ShowDirs */

/*********************************************************************
 *
 * ShowFiles
 *
 * SYNOPSIS
 *		LOCAL short ShowFiles (HitInfo *hi)
 *
 * PURPOSE
 *		Display Available Files in List Area
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL short ShowFiles (HitInfo *hi)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ShowFiles";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	hi = hi;
	{
		short	 fpos;
		FNNode	*fn;

		fpos = 0;
		fn   = Head (NameList);
		while (!IsEOList (fn)) {
			if (fn->Type == FILE_TYPE) {
				FileReq->Position = fpos;
				UpdateNameList (FileReq);
				break;
			}
			fpos++;
			fn = Next (fn);
		}

	}

	return TRUE;
} /* ShowFiles */


/*********************************************************************
 *
 * MoveToNewDrive
 *
 * SYNOPSIS
 *		LOCAL short MoveToNewDrive(char *sz)
 *
 * PURPOSE
 *		Move to new disk drive specified by <sz> and rescan files.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *		Returns TRUE if able to switch to new drive.
 *
 * HISTORY
 *		10/12/93 Tuesday (dcc) - updated to use SetSysErrorFunc() instead
 *				of setvect() to work with new definition of IgnoreErrors().
 *
 * SEE ALSO
 *
*/
LOCAL short MoveToNewDrive(char *sz)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "MoveToNewDrive";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
{
	union REGS inregs;
	union REGS outregs;
	INT far32 * intfunc;

	intfunc = GetSysErrorVector();
	SetSysErrorFunc(IgnoreErrors);

	inregs.h.ah = 0x1C;
	inregs.h.dl = toupper(*sz) - '@';
#if __WATCOMC__
	int386 (0x21, &inregs, &outregs);
#elif __TURBOC__
	int86 (0x21, &inregs, &outregs);
#endif

	SetSysErrorVector (intfunc);

	if (outregs.h.al != 0xFF)
	{
		setdisk(toupper(*sz) - 'A');
		EIO_CurrentDir (FRPath);
		EIO_InsureEndSlash (FRPath);
		strcat (FRPath, "*.*");
		FileReq->Position = 0;
		CreateNameList (FileReq);
		UpdateNameList (FileReq);
		return TRUE;
	}
	return FALSE;
}
} /* MoveToNewDrive */


/*********************************************************************
 *
 * MoveToRoot
 *
 * SYNOPSIS
 *		LOCAL short MoveToRoot(void)
 *
 * PURPOSE
 *		Move to the root directory of the current drive.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *		Always returns TRUE.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL short MoveToRoot(void)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "MoveToRoot";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	FRPath[3] = 0;
	strcat (FRPath, "*.*");
	FileReq->Position = 0;
	CreateNameList (FileReq);
	UpdateNameList (FileReq);
	return TRUE;

} /* MoveToRoot */


/*********************************************************************
 *
 * MoveOutADir
 *
 * SYNOPSIS
 *		LOCAL short MoveOutADir(void)
 *
 * PURPOSE
 *		Move to parent directory and rescan files.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *		Always returns TRUE.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL short MoveOutADir(void)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "MoveOutADir";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		char	 work[EIO_MAXPATH];
		char	 dir[EIO_MAXPATH];
		char	*path;

		path = FRPath;
		if (strlen (path) > 3 + 3) {
			EIO_fnsplit (path, work, NULL, NULL);
			work[strlen(work) - 1] = '\0';
			EIO_fnsplit (work, dir, NULL, NULL);
			EIO_InsureEndSlash (dir);
			EIO_fnmerge (path, dir, "*.*", NULL);

			FileReq->Position = 0;
			CreateNameList (FileReq);
			UpdateNameList (FileReq);
		}
	}
	return TRUE;

} /* MoveOutADir */


/*********************************************************************
 *
 * MoveIntoADir
 *
 * SYNOPSIS
 *		LOCAL short MoveIntoADir(char *sz)
 *
 * PURPOSE
 *		Move into sub-directory <sz> and rescan files.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *		Returns TRUE if moved into <sz>, FALSE if <sz> is not a sub-dir.
 *
 * SEE ALSO
 *
*/
LOCAL short MoveIntoADir(char *sz)
{
	char	 work[EIO_MAXPATH];
	char	 dir[EIO_MAXPATH];

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "MoveIntoADir";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	EIO_fnsplit (FRPath, dir, NULL, NULL);
	EIO_fnmerge (work, dir, sz, NULL);
	if (EIO_FileType(work) == EIO_TYPE_DIRECTORY)
	{
		EIO_InsureEndSlash (work);
		EIO_fnmerge (FRPath, work, "*.*", NULL);
		FileReq->Position = 0;
		CreateNameList (FileReq);
		UpdateNameList (FileReq);
		return TRUE;
	}
	return FALSE;

} /* MoveIntoADir */


/*********************************************************************
 *
 * EditPath
 *
 * SYNOPSIS
 *		LOCAL short EditPath (HitInfo *hi)
 *
 * PURPOSE
 *		This function is called when user clicks on current directory box.
 *		It calls MoveOutADir to move to parent directory and rescan files.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL short EditPath (HitInfo *hi)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "EditPath";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	hi = hi;

	MoveOutADir();

	return TRUE;

} /* EditPath */

/*********************************************************************
 *
 * EditList
 *
 * SYNOPSIS
 *		LOCAL short EditList (HitInfo *hi)
 *
 * PURPOSE
 *		Choose a name from list
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL short EditList (HitInfo *hi)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "EditList";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		short	oldfp = FileReq->Position;
		short	oldhp = HLpos;
		short	y;
		UWORD	buttons;
		static	skip  = FALSE;

		y          = hi->Y;
		buttons    = hi->Buttons;

		if (!skip) {
			y = (y - 1) / LINE_HEIGHT;
			if (y < 0) {
				if (FileReq->Position) {
					FileReq->Position--;
				}
			} else if (y > LIST_LINES - 1) {
				if (FileReq->Position < FileReq->FileCount - LIST_LINES) {
					FileReq->Position++;
				}
			}

			y = max (y, 0);
			y = min (y, LIST_LINES - 1);

			HLpos = FileReq->Position + y;
		
			if (HLpos != oldhp || FileReq->Position != oldfp) {
				UpdateNameList (FileReq);
			}

			if (buttons && (buttons & MOUSE_DOUBLE) && HLpos == oldhp) {
				if (HLnode) {
					switch (HLnode->Type) {
					case FILE_TYPE:
						FRdone = TRUE;
						break;
					case DRAWER_TYPE:
						skip = MoveIntoADir(NodeName(HLnode));
						if (skip)
						{
							FRName[0] = 0;
							SelectString (&NameString);
						}
						break;
					case DISK_TYPE:
						skip = MoveToNewDrive(NodeName(HLnode));
						if (skip)
						{
							FRName[0] = 0;
							SelectString (&NameString);
						}
						break;
					}
				}
			}
		}
		if (!buttons) {
			skip = FALSE;
			if (HLnode /*&& HLnode->Type == FILE_TYPE*/) {
				strcpy (FRName, NodeName (HLnode));
				SelectString (&NameString);
			}
		}
	}
	return TRUE;

} /* EditList */


/*********************************************************************
 *
 * UpdateNearestNameList
 *
 * SYNOPSIS
 *		LOCAL void UpdateNearestNameList(EasyFileReq *fr, char *sz)
 *
 * PURPOSE
 *		Find closest match to <sz>, move highlighter in list to
 *		closest match.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL void UpdateNearestNameList(EasyFileReq *fr, char *sz)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "UpdateNearestNameList";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		short	 newhp = 0;
		FNNode	*newfn;

		newfn = Head (NameList);
		while (newhp < fr->FileCount &&
			((DirsAtTop && newfn->Type != FILE_TYPE) || stricmp (NodeName (newfn), sz) < 0))
		{
			newhp++;
			newfn = Next (newfn);
		}

		if (newhp != HLpos)
		{
			fr->Position = newhp;
			HLpos = newhp;
			UpdateNameList (fr);
		}
	}
} /* UpdateNearestNameList */


/*********************************************************************
 *
 * CheckKeys
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *		05/13/93 (dcc) - Fix trying to save to certain filenames
 *						 (such as C.LBM) on empty drive switching drives.
 *		10/17/94 (dcc) - Allow filename to remain if new drive inserted
 *						 in front of filename.
 *		10/17/94 (dcc) - Fix '\' at start not moving to the root directory
 *						 if there is a subsequent filename.
 *
 * SEE ALSO
 *
*/
void CheckKeys (EasyFileReq *fr)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "CheckKeys";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		short	status;
		short	key;

		status = CheckEditKeys (&key);
		if (status) {
			if (status < 0)
			{
				FRdone = TRUE;
				FRabort = TRUE;
			}
			else
			{
				if (strcmp(FRName, "..") == 0)
				{
					MoveOutADir();
					FRName[0] = 0;
					SelectString (&NameString);
				}
				else if (HLnode)		// hiliter over a file in list?
				{
					switch (HLnode->Type)
					{
						case FILE_TYPE:	// hilite over filename, just return
							FRdone = TRUE;
							break;
						case DRAWER_TYPE:	// see if move into sub-dir
							if (MoveIntoADir(FRName))
							{
								FRName[0] = 0;
								SelectString (&NameString);
							}
							else
							{
								FRdone = TRUE;
							}
							break;
						case DISK_TYPE:	// see if change disks
							if (FRName[1] == ':' && FRName[2] == 0 &&
								MoveToNewDrive(FRName))
							{
								FRName[0] = 0;
								SelectString (&NameString);
							}
							else
							{
								FRdone = TRUE;
							}
					}
				}
				else
				{
					FRdone = TRUE;
				}
			}
		} else {
			if (key > 32 && key < 128 && FRName[1] == ':')
			{
				if (MoveToNewDrive(FRName))
				{
					/* Delete initial 'x:'. */
					strcpy(FRName, &FRName[2]);
					SelectString (&NameString);
				}
			}
			switch (key) {
			case EASCII_UP:
			case EASCII_DOWN:
			case EASCII_PGUP:
			case EASCII_PGDN:
				{
					short	newhp;
					short	add;

					if (HLpos < 0 ||
						HLpos < fr->Position ||
						HLpos > fr->Position + LIST_LINES - 1) {
						newhp = fr->Position + LIST_LINES / 2 - 1;
						if (newhp >= fr->FileCount) {
							newhp = fr->FileCount - 1;
						}
					} else {
						newhp = HLpos;
					}

					switch (key) {
					case EASCII_UP:
						add = -1;
						break;
					case EASCII_DOWN:
						add = 1;
						break;
					case EASCII_PGUP:
						add = -(LIST_LINES - 1);
						break;
					case EASCII_PGDN:
						add = (LIST_LINES - 1);
						break;
					}

					newhp += add;
					newhp = max (0, newhp);
					newhp = min (newhp, fr->FileCount - 1);

					if (HLpos != newhp) {
						if (newhp < fr->Position) {
							fr->Position = newhp;
						} else if (newhp > fr->Position + LIST_LINES - 1) {
							fr->Position = newhp - (LIST_LINES - 1);
						}
						HLpos = newhp;
						UpdateNameList (fr);
						if (HLnode /* && HLnode->Type == FILE_TYPE*/) {
							strcpy (FRName, NodeName (HLnode));
							SelectString (&NameString);
						}
					}
				}
				break;
			case 0:
				break;
			case ' ':
				{
					if (HLnode)
					{
						switch (HLnode->Type)
						{
							case FILE_TYPE:	// hilite over filename, complete name
								strcpy(FRName, NodeName(HLnode));
								SelectString (&NameString);
								break;
							case DRAWER_TYPE:	// hilite over sub-dir, move into it
								MoveIntoADir(NodeName(HLnode));
								FRName[0] = 0;
								SelectString (&NameString);
								break;
							case DISK_TYPE:	// hilite over disk, change drive
								MoveToNewDrive(NodeName(HLnode));
								FRName[0] = 0;
								SelectString (&NameString);
								break;
						}
					}
				}
				break;
#if 0
			case ':':
				{
					if (FRName[2] == 0)
					{
						if (MoveToNewDrive(FRName))
						{
							FRName[0] = 0;
							SelectString (&NameString);
						}
					}
				}
				break;
#endif
			case '\\':
				{
					if (FRName[0] == '\\')
					{
						if (MoveToRoot())
						{
							/* Delete initial '\'. */
							strcpy(FRName, &FRName[1]);
							SelectString (&NameString);
						}
					}
					else
					{
						char *pb = strchr(FRName, '\\');

						if (strcmp(FRName, "..\\") == 0)
						{
							MoveOutADir();
							/* Delete initial '..\'. */
							strcpy(FRName, pb+1);
							SelectString (&NameString);
						}
						else
						{
							*pb = 0;
							MoveIntoADir(FRName);
							/* Delete initial directory name. */
							strcpy(FRName, pb+1);
							SelectString (&NameString);
						}
					}
				}
				break;
			default:
				//
				// String has been edited so move list to reflect.
				//
				UpdateNearestNameList (fr, FRName);
				break;
			}
		}
	}
} /* CheckKeys */

/********************************* Gadgets ********************************/

LOCAL HitArea FRHitArea[] = {
	{ ARROW_X, ARROW_TOP - ARROW_HEIGHT + 1, ARROW_WIDTH, ARROW_HEIGHT,
		HTA_DOWN | HTA_REPEAT | HTA_INSURE_UP | HTA_COMPLEMENT, UPARROW_ID, ScrollUp},
	{ ARROW_X, ARROW_BOT, ARROW_WIDTH, ARROW_HEIGHT,
		HTA_DOWN | HTA_REPEAT | HTA_INSURE_UP | HTA_COMPLEMENT, DOWNARROW_ID, ScrollDown},
	{ ARROW_X, ARROW_TOP + 1, ARROW_WIDTH, ARROW_BOT - ARROW_TOP - 1,
		HTA_DOWN | HTA_REPEAT | HTA_INSURE_UP, ELEVATOR_ID, Elevator, },
	{ PATH_X, PATH_Y, PATH_WIDTH, PATH_HEIGHT,
		NULL, PATH_ID, EditPath, },
	{ NAME_X, NAME_Y, NAME_WIDTH, NAME_HEIGHT,
		HTA_DOWN | HTA_REPEAT | HTA_INSURE_UP, NAME_ID, HighlightString, &NameString, },
	{ LIST_X, LIST_Y, LIST_WIDTH, LIST_HEIGHT,
		HTA_DOWN | HTA_REPEAT | HTA_INSURE_UP, LIST_ID, EditList, },
	{ BUTTS_X, DISKS_Y, BUTTS_WIDTH, BUTTS_HEIGHT,
		HTA_COMPLEMENT, DISKS_ID, ShowDisks, },
	{ BUTTS_X, CANCEL_Y, BUTTS_WIDTH, BUTTS_HEIGHT,
		HTA_COMPLEMENT, CANCEL_ID, NULL, },
	{ BUTTS_X, OKAY_Y, BUTTS_WIDTH, BUTTS_HEIGHT,
		HTA_COMPLEMENT, OKAY_ID, NULL, },
#define DIRS_BUTTON		9
	{ BUTTS_X, DIRS_Y, BUTTS_WIDTH, BUTTS_HEIGHT,
		HTA_COMPLEMENT, DIRS_ID, ShowDirs, },
#define FILES_BUTTON	10
	{ BUTTS_X, FILES_Y, BUTTS_WIDTH, BUTTS_HEIGHT,
		HTA_COMPLEMENT, FILES_ID, ShowFiles, },
	{ HTA_LAST_AREA, HTA_LAST_AREA, HTA_LAST_AREA, HTA_LAST_AREA, },
};

/****************************** MainRoutines ******************************/

/*********************************************************************
 *
 * LocalGetFileName
 *
 * SYNOPSIS
 *		char *LocalGetFileName (EasyFileReq *fr, char *path)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *		10/10/94 (dcc) - Make sure <ha> pointer is != NULL B4 using it.
 *
 * SEE ALSO
 *
*/
LOCAL char *LocalGetFileName (EasyFileReq *fr, char *path)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "LocalGetFileName";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		char				*filename = NULL;
		SaveAreaType		*sa = NULL;
		HitArea				*ha;
		Window				*win;
		struct	RastPort	 rp = { NULL, };

		InitList (NameList);

		win     = fr->Window;
		FRWin   = win;
		FRFont  = win->Font;
		FileReq = fr;
		FrColors[FR_WHITE ] = win->White;
		FrColors[FR_BLACK ] = win->Black;
		FrColors[FR_BLUE  ] = win->Blue;
		FrColors[FR_ORANGE] = win->Orange;
		FRdone  = FALSE;
		FRabort = FALSE;
		if (!fr->Used) {
			fr->Used = TRUE;
			if (!strlen (fr->Path) || !strcmp (fr->Path, "*.*")) {
				if (LastUsedPath[0]) {
					strcpy (fr->Path, LastUsedPath);
				} else {
					EIO_CurrentDir (fr->Path);
					EIO_InsureEndSlash (fr->Path);
					strcat (fr->Path, "*.*");
				}
			}
		}

		if (PushWindowClipValues (win)) {
			sa = SaveArea (FR_LEFT, FR_TOP, FR_WIDTH, FR_HEIGHT);
			if (!sa) {
				GEcatf ("OOM: FileReq Area");
				goto dfrcleanup;
			}

			StartEPicture (&rp, &FREPicture, 20, 20);
			{
				short len;

				len = StringLen (FRFont, fr->Title);
				DrawString (FRFont, FR_LEFT + ((FR_WIDTH - 3) / 2) - (len / 2), FR_TOP + 3, fr->Title);
			}

			if (DirsAtTop)
			{
				FRHitArea[DIRS_BUTTON ].Width = BUTTS_WIDTH;
				FRHitArea[FILES_BUTTON].Width = BUTTS_WIDTH;
			}
			else
			{
				FRHitArea[DIRS_BUTTON ].Width = 0;
				FRHitArea[FILES_BUTTON].Width = 0;

				SetPenColor (FrColors[FR_WHITE]);
				DrawRect (FR_LEFT + BUTTS_X,
						  FR_TOP  + FILES_Y,
						  BUTTS_WIDTH,
						  (BUTTS_HEIGHT + 1) * 2);
			}

			strcpy (FRPath, fr->Path);
			strcpy (FRName, fr->Name);

			CreateNameList (fr);
			if (FRName[0])
			{
				UpdateNearestNameList (fr, FRName);	// highlight nearest match
			}
			else
			{
				UpdateNameList (fr);
			}
			UpdateStrings (FRWin, &NameString, 1, NULL);

			while (!FRdone) {
				ha = CheckHitAreas (FRHitArea, FR_LEFT, FR_TOP);
				if (ha && ha->ID == OKAY_ID) {
					break;
				}
				if (ha && ha->ID == CANCEL_ID) {
					FRabort = TRUE;
					break;
				}
				CheckKeys (fr);
			}

			strcpy (LastUsedPath, FRPath);
			if (!FRabort) {
				strcpy (fr->Path, FRPath);
				strcpy (fr->Name, FRName);
				EIO_fnsplit (fr->Path, FRPath, NULL, NULL);
				EIO_fnmerge (path, FRPath, FRName, NULL);
				filename = path;
			}

dfrcleanup:
			EmptyList (NameList);
			if (sa)		RestoreArea (sa);
			FlushKeyboardBuffer ();

			PopClipValues ();
		}
		return filename;
	}
} /* LocalGetFileName */

/*********************************************************************
 *
 * GetFileName
 *
 * SYNOPSIS
 *		char *GetFileName (Window *win, EasyFileReq *fr)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
char *GetFileName (Window *win, EasyFileReq *fr)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "GetFileName";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		char	*s;

		fr->Window = win;
		s = LocalGetFileName (fr, fr->FileName);
		if (s) {
			if (!strlen (fr->Name)) {
				return NULL;
			}
		}
		return s;
	}

} /* GetFileName */

/*********************************************************************
 *
 * GetPathName
 *
 * SYNOPSIS
 *		char *GetPathName (Window *win, EasyFileReq *fr)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
char *GetPathName (Window *win, EasyFileReq *fr)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "GetPathName";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		char	*s;

		fr->Window = win;
		s = LocalGetFileName (fr, fr->FileName);

		return s;
	}
} /* GetPathName */


/*********************************************************************
 *
 * SetFreqTitle
 *
 * SYNOPSIS
 *		void SetFreqTitle (
 *			EasyFileReq	*ezfrq,
 *			char		*title
 *		)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void SetFreqTitle (
	EasyFileReq	*ezfrq,
	char		*title
) {

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "SetFreqTitle";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	strcpy (ezfrq->Title, title);

} /* SetFreqTitle */

/*********************************************************************
 *
 * InitFileReq
 *
 * SYNOPSIS
 *		void InitFileReq (EasyFileReq *fr)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void InitFileReq (EasyFileReq *fr)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "InitFileReq";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	fr->Position = 0;
	fr->Used     = 0;
	EIO_CurrentDir (fr->Path);
	EIO_InsureEndSlash (fr->Path);
	strcat (fr->Path, "*.*");
	strcpy (fr->Name, "");
	strcpy (fr->Title, "Select File");

} /* InitFileReq */

/*********************************************************************
 *
 * InitFileReqs
 *
 * SYNOPSIS
 *		EasyFileReq **InitFileReqs (
 *			FileReqInt		*fri;
 *		) 
 *
 * PURPOSE
 *		Setup and initialize a table of EasyFileReqs
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
EasyFileReq **InitFileReqs (
	FileReqInt		*fri
) {

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "InitFileReqs";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		int			 	 cnt;
		FileReqInt		*fi;
		EasyFileReq		**ezfrq;
		EasyFileReq		**ezfrqptr;
		EasyFileReq		*wfrp;

		cnt = 0;
		fi  = fri;
		while (fi->Title) {
			cnt++;
			fi++;
		}

#if REQUIRE
		if (!cnt) {
			SetGlobalErr (ERR_REQUIREMENTS);
			GEprintf ("InitFileReqs: Zero EasyFileReqs requested");
			return NULL;
		}
#endif

		if ((ezfrq = MEM_calloc (cnt, sizeof (EasyFileReq *))) == NULL) {
			SetGlobalErr (ERR_OUT_OF_MEMORY);
			GEprintf ("OOM: EasyFileReq Table");
			return NULL;
		}

		ezfrqptr = ezfrq;
		while (cnt > 0) {
			if ((wfrp = MEM_calloc (1, sizeof (EasyFileReq))) == NULL) {
				SetGlobalErr (ERR_OUT_OF_MEMORY);
				GEprintf ("OOM: EasyFileReq");
				return NULL;
			}
			*ezfrqptr = wfrp;
			InitFileReq (wfrp);
			SetFreqTitle (wfrp, fri->Title);
			strcpy (wfrp->Path,  fri->Path);
			strcpy (wfrp->Name,  fri->FileName);
			fri++;
			ezfrqptr++;
			cnt--;
		}
		/* Remember which drive we're on before MoveToNewDrive() changes it */
		{
			char szOriginalPath[EIO_MAXPATH];

			EIO_CurrentDir (szOriginalPath);
			wOriginalDrive = toupper(szOriginalPath[0] - 'A');
		}

		return ezfrq;
	}
} /* InitFileReqs */

/*********************************************************************
 *
 * SetDirSortType
 *
 * SYNOPSIS
 *		void SetDirSortType (short type)
 *
 * PURPOSE
 *		Sets whether or not directories appear at the top of the
 *		list or intermixed with the files.
 *
 * INPUT
 *		type	: FALSE = intermixed
 *				: TRUE  = Directories at top of list.
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void SetDirSortType (short type)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "SetDirSortType";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	DirsAtTop = type;

} /* SetDirSortType */


/*********************************************************************
 *
 * RestoreOriginalDrive
 *
 * SYNOPSIS
 *		void RestoreOriginalDrive(void)
 *
 * PURPOSE
 *		MoveToNewDrive() may change the default drive.
 *		This function restores the drive that was active when
 *		InitFileReqs() was called.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void RestoreOriginalDrive(void)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "RestoreOriginalDrive";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	setdisk(wOriginalDrive);
} /* RestoreOriginalDrive */

