/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * HITAREAS.C
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 10/10/90
 *   MODIFIED : 10/10/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		Routines to deal with Hit Areas (Gadgets?)
 *
 * HISTORY
 *		10/12/93 Tuesday (dcc) - updated to support MS-DOS Watcom C32.
 *
 * TODO
 *
 * * Add some way to know when button is up if mouse is not on button.
 *   HTA_BUTTONVERIFY?
*/

#include <echidna/platform.h>
#include "switches.h"

#include <ctype.h>
#include <dos.h>
#include <echidna/eerrors.h>
#include <echidna/memory.h>
#include <echidna/hitareas.h>
#include <echidna/keyboard.h>
#include <echidna/grafx.h>
#include <echidna/mouse.h>
#include <echidna/events.h>

#define DrawXorRect	XORRect	/* GAT: Kludge */

#if HITAREAS_USE_BLITREC
#include "blitrec.h"
#endif
/**************************** C O N S T A N T S ***************************/


/******************************** T Y P E S *******************************/

typedef struct {
	/* Terms first,last,next,previous refer to order in area array/list */

	HitArea	*pha;					/* Traversal pointer. */
	HitArea *phaDefaultEnter;		/* Current default enter focused area. */
	HitArea	*phaFirstSpaceAble;		/* First space focusable area. */
	HitArea	*phaFirstEnterAble;		/* First enter focusable area. */
	HitArea *phaPrevSpaceAble;		/* Nearest (to current focus ) previous 
									 *    space focusable area. */
	HitArea *phaPrevEnterAble;		/* Nearest (to current focus) previous 
									 *    enter focusable area. */
	HitArea	*phaSpace;				/* Current space focused area. */
	HitArea *phaEnter;				/* Current enter focused area. */
	HitArea *phaNextSpaceAble;		/* Nearest (to current focus) next space 
									 *    focusable area. */
	HitArea *phaNextEnterAble;		/* Nearest (to current focus) next enter
									 *    focusable area. */
	HitArea *phaLastSpaceAble;		/* Last space focusable area. */
	HitArea *phaLastEnterAble;		/* Last enter focusable area. */

} FCT; /* Focus Changing Tracker */


typedef enum {
	hkMouse, 
	hkSpace, 
	hkEnter, 
	hkEscape, 
	hkKey, 		/* Any key pressed for an active area (space focused) 
				 * that wants key input (HitArea.Flags & HTA_KEYS)
				 */
	hkHotKey, 	/* When a hot key is pressed */
	hkCursor
} HK; /* Hit Kind */

typedef struct {
	HK	hk;	/* Hit kind */

	FCT			*pfct;		/* Focus Change Tracker */
	short		 dx, dy;	/* offsets for hkMouse hit kind */
	UWORD		 fShift;	/* ShiftFlags() for KeyCode */
	short		 KeyCode;	/* ASCII key value for Key hit kind and 
							 * ASCII key value for HotKey hit kind and
							 * Cursor key value for Cursor hit kind.
							 */

} HKI;	/* Hit kind info */

/****************************** G L O B A L S *****************************/


/******************************* M A C R O S ******************************/


#define fCursorFromKey(Key)											\
	(																	\
		(((Key) == UP_KEY)		? (CURSOR_UP  | CURSOR_PREV) : 0) |	\
		(((Key) == DOWN_KEY)	? (CURSOR_DN  | CURSOR_NEXT) : 0) |	\
		(((Key) == LEFT_KEY)	? (CURSOR_LFT | CURSOR_PREV) : 0) |	\
		(((Key) == RIGHT_KEY)	? (CURSOR_RGT | CURSOR_NEXT) : 0)		\
	)																	



/***************************** R O U T I N E S ****************************/

/*********************************************************************
 *
 * gettickcount
 *
 * SYNOPSIS
 *		long gettickcount (void)
 *
 * PURPOSE
 *		return OS ticks?
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
long gettickcount (void)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "gettickcount";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

#if (__WATCOMC__ && __MSDOS32X__)
	{
		union REGS regs;

		regs.h.ah = 0x00;

		int386 (0x1A, &regs, &regs);

		return ((long)((((ULONG)regs.w.cx) << 16) | (ULONG)regs.w.dx));
	}
#elif __TURBOC__
	{
		union REGS inregs;
		union REGS outregs;

		inregs.h.ah = 0x00;

		int86 (0x1A, &inregs, &outregs);

		return ((long)((((ULONG)outregs.x.cx) << 16) | (ULONG)outregs.x.dx));
	}
#else
#error Need Something Here!
#endif

} /* gettickcount */

/*********************************************************************
 *
 * FillOutFCT
 *
 * SYNOPSIS
 *		void FillOutFCT (FCT *pfct) 
 *
 * PURPOSE
 *		To fill in the entries in an FCT struct by traversing the array
 *		of areas pointed to by fct->pha and making recursive calls on
 *		areas that have subareas.
 *
 * INPUT
 *		pfct:	Pointer to fct struct with 'pha' field initialized to point 
 *				to the current entry of the traversal.
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void FillOutFCT (FCT *pfct) 
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "FillOutFCT";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
{
	FCT		 fct;	/* local copy of struct to avoid dereferncing */

	/* Find Currently Focused areas and prev and next focusAble areas */
	for (fct = *pfct; 
		!END_HITAREA (fct.pha); 
		fct.pha = NEXT_HITAREA (fct.pha)
	)
	{
		UWORD Flags = fct.pha->Flags;
		UWORD fState = fct.pha->fState;

		if (fState & HA_DISABLED) continue;

		if (Flags & HTA_SUBAREAS) {
			HitArea	*pha = fct.pha;
			fct.pha = pha->Data;
			FillOutFCT (&fct);
			fct.pha = pha;
		} else {
			if ((Flags= fct.pha->Flags) & HTA_ENTER_FOCUS) {
				if (!fct.phaDefaultEnter && (Flags & HTA_DEFAULT_ENTER)) {
					fct.phaDefaultEnter = fct.pha;
				}
				if (!fct.phaFirstEnterAble) {
					fct.phaFirstEnterAble = fct.pha;
				}

				if (!fct.phaNextEnterAble && fct.phaEnter) {
					fct.phaNextEnterAble = fct.pha;
				}
				if (!fct.phaEnter && (fState & HA_ENTER)) {
					fct.phaEnter = fct.pha;
					if (fct.phaLastEnterAble) {
						fct.phaPrevEnterAble = fct.phaLastEnterAble;
					}
				}
				fct.phaLastEnterAble = fct.pha;
			}

			if ((Flags & HTA_SPACE_FOCUS)) {
				if (!fct.phaFirstSpaceAble) {
					fct.phaFirstSpaceAble = fct.pha;
				}

				if (!fct.phaNextSpaceAble && fct.phaSpace) {
					fct.phaNextSpaceAble = fct.pha;
				}
				if (!fct.phaSpace && (fState & HA_SPACE)) {
					fct.phaSpace = fct.pha;
					if (fct.phaLastSpaceAble) {
						fct.phaPrevSpaceAble = fct.phaLastSpaceAble;
					}
				}
				fct.phaLastSpaceAble = fct.pha;
			}
		}
	}

	*pfct = fct; /* copy results back to original struct */

}} /* FillOutFCT */

/*********************************************************************
 *
 * ChangeFocus
 *
 * SYNOPSIS
 *		void ChangeFocus (
 *			HitArea *phaSpace, HitArea *phaEnter,
 *			HitArea *phaNewSpace, HitArea *phaNewEnter,
 *			HitArea *phaDefaultEnter
 *		)
 *
 * PURPOSE
 *		To make calls to unfocus old area and focus new area.
 *
 * INPUT
 *		phaSpace:	Currently space focused area.
 *		phaEnter:	Currently enter focused area.
 *		phaNewSpace:	New area to get space focused.
 *		phaNewEnter:	New area to get enter focused.
 *		phaDefaultEnter: Default enter focus area (this gets enter
 *					focus if phaNewEnter is not enter focusable).
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *		10/09/94 (dcc) - Make sure pointers are != NULL B4 using 'em.
 *
 * SEE ALSO
 *
*/
void ChangeFocus (
	HitArea *phaSpace, HitArea *phaEnter,
	HitArea *phaNewSpace, HitArea *phaNewEnter,
	HitArea *phaDefaultEnter
)
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ChangeFocus";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
{
	HitInfo hi;

	/* Is new space focus area not space focusable */
	if (!(phaNewSpace->Flags & HTA_SPACE_FOCUS)) { /* Yes */
		phaNewSpace = phaSpace; 
	}

	/* Is new enter focus area not enter focusable */
	if (!(phaNewEnter->Flags & HTA_ENTER_FOCUS)) { /* Yes */
		phaNewEnter = phaDefaultEnter;
	}

	/* Only allow focus/unfocus calls if something is going to change */
	if (phaSpace == phaNewSpace) {
		phaSpace = phaNewSpace = NULL;
	}
	if (phaSpace)
		if (!(phaSpace->Flags & HTA_SPACE_FOCUS)) phaSpace = NULL;
	if (phaNewSpace)
		if (!(phaNewSpace->Flags & HTA_SPACE_FOCUS)) phaNewSpace = NULL;

	if (phaEnter == phaNewEnter) {
		phaEnter = phaNewEnter = NULL;
	}
	if (phaEnter)
		if (!(phaEnter->Flags & HTA_ENTER_FOCUS)) phaEnter = NULL;
	if (phaNewEnter)
		if (!(phaNewEnter->Flags & HTA_ENTER_FOCUS)) phaNewEnter = NULL;

	hi.inpt = inptFocus;
	/* Make current area(s) lose focus */
	if (phaSpace || phaEnter) {
		
		hi.fFocus = FOCUS_SPACE_OFF;
		hi.HitArea    = phaSpace;
		if (phaSpace == phaEnter) {
			phaSpace->fState &= ~((UWORD)HA_SPACE | (UWORD)HA_ENTER);
			hi.fFocus    |= FOCUS_ENTER_OFF;
			((MouseFunc)(phaSpace->Data))(&hi);
		} else {
			if (phaSpace) {
				phaSpace->fState &= ~((UWORD)HA_SPACE);
				((MouseFunc)(phaSpace->Data))(&hi);
			} 
			if (phaEnter) {
				hi.fFocus = FOCUS_ENTER_OFF;
				hi.HitArea    = phaEnter;
				phaEnter->fState &= ~((UWORD)HA_ENTER);
				((MouseFunc)(phaEnter->Data))(&hi);
			}
		}
	}

	/* Make New area(s) get focus */
	if (phaNewSpace || phaNewEnter) {

		hi.fFocus = FOCUS_SPACE_ON;
		hi.HitArea    = phaNewSpace;
		if (phaNewSpace == phaNewEnter) {
			hi.fFocus |= FOCUS_ENTER_ON;
			phaNewSpace->fState |= (UWORD)HA_SPACE | (UWORD)HA_ENTER;
			((MouseFunc)(phaNewSpace->Data))(&hi);
		} else {
			if (phaNewSpace) {
				phaNewSpace->fState |= (UWORD)HA_SPACE;
				((MouseFunc)(phaNewSpace->Data))(&hi);
			} 
			if (phaNewEnter) {
				hi.fFocus = FOCUS_ENTER_ON;
				hi.HitArea    = phaNewEnter;
				phaNewEnter->fState |= (UWORD)HA_ENTER;
				((MouseFunc)(phaNewEnter->Data))(&hi);
			}
		}
	}

}} /* ChangeFocus */


/*********************************************************************
 *
 * MoveFocus
 *
 * SYNOPSIS
 *		void MoveFocus (HitArea *pha, short fForward)
 *
 * PURPOSE
 *		Change the focus to next area in direction dFocus.
 *		Call the functions for unfocused and newly focused areas.
 *
 * INPUT
 *		dForward: 	True if direction is forward, else direction is backward.
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void MoveFocus (FCT *pfct, short fForward)
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "MoveFocus";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
{
	HitArea *phaNew = NULL;

	if (fForward) {
		if (pfct->phaNextSpaceAble) {
				phaNew = pfct->phaNextSpaceAble;
		} else if (pfct->phaFirstSpaceAble) { /* Wrap to first focsuable */
				phaNew = pfct->phaFirstSpaceAble;
		}
	} else { /* Backward */
		if (pfct->phaPrevSpaceAble) {
			phaNew = pfct->phaPrevSpaceAble;
		} else if (pfct->phaLastSpaceAble) { /* Wrap to Last focsuable */
			phaNew = pfct->phaLastSpaceAble;
		}
	}

	if (phaNew) {
		ChangeFocus (
			pfct->phaSpace, pfct->phaEnter, 
			phaNew, phaNew,
			pfct->phaDefaultEnter
		);
	}

}} /* MoveFocus */


/*------------------------------------------------------------------------*/
/**# MODULE:HITAREAS_CheckHitAreas                                        */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * HitThisArea
 *
 * SYNOPSIS
 *		void	HitThisArea (HitArea *pha, short x, short y, HKI *phki)
 *
 * PURPOSE
 *		Hit the area pointed at by pha in the way indicated by phki->hk (
 *		key hit or mouse hit).
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void	HitThisArea (HitArea *pha, HKI *phki)
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "HitThisArea";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
{
	HitInfo		 hi;
	MouseInfo	 mi;
	short		 fEvent;
	short		 fComplement;
	short		 fToggle;
	FCT			*pfct = phki->pfct;
	short		 flags = pha->Flags;
	short		 hk = phki->hk;

	fComplement	= flags & HTA_COMPLEMENT;
	fEvent		= flags & HTA_EVENT;
	fToggle		= flags & HTA_TOGGLE;

	if (hk == hkMouse) {
		ReadMouse (&mi);
	}

	ChangeFocus (
		pfct->phaSpace, pfct->phaEnter,
		pha, pha,
		pfct->phaDefaultEnter
	);

	hi.HitArea = pha;

	if (fComplement) {
		SetPenColor (HIT_COMPCOLOR);
		BeforeGraphics();
		DrawXorRect (pha->X + phki->dx, pha->Y + phki->dy,
			pha->Width, pha->Height);
		AfterGraphics();
	}

	if (fToggle) {
		pha->fState ^= HA_TOGGLE_STATE;
	}

	/* Call for button DOWN */
	if (hk == hkMouse) {
		if ((flags & HTA_DOWN) && (pha->Data)) {
			if (!fEvent) {
				static long		oldticks = 0;
				long			newticks;
				long			diff;
				newticks = gettickcount();
				diff     = newticks - oldticks;
				oldticks = newticks;
			
				hi.inpt = inptMouse;
				hi.Buttons = mi.Buttons | MOUSE_FIRSTDOWN |
					((diff < 9) ? MOUSE_DOUBLE : 0);
				hi.Shiftflags = ShiftFlags();
				hi.X = mi.X - phki->dx - pha->X;
				hi.Y = mi.Y - phki->dy - pha->Y;

				((MouseFunc)(pha->Data))(&hi);
			}
		}

		/* Call for Repeat */
		hi.inpt = inptMouse;
		for (;;) {
			ReadMouse (&mi);
			if (!mi.Buttons) {
				break;
			}

			if (pha->Data && !fEvent && (flags & HTA_REPEAT)) {
				hi.X          = mi.X - phki->dx - pha->X;
				hi.Y          = mi.Y - phki->dy - pha->Y;
				hi.Buttons    = mi.Buttons;
				hi.Shiftflags = ShiftFlags();
				((MouseFunc)(pha->Data))(&hi);
			}
		}
	}

	if (fComplement && !fToggle) {
		SetPenColor (HIT_COMPCOLOR);
		BeforeGraphics();
		DrawXorRect (pha->X + phki->dx, pha->Y + phki->dy,
			pha->Width, pha->Height);
		AfterGraphics();
	}

	/* Call for button UP */
	if (pha->Data) {
		if (fEvent) {
			TriggerEvent (pha->Data, ETYPE_HITAREA, pha);
		} else if (!(flags & HTA_NO_UP)) {
			short fHit = TRUE;
			switch (hk) {
			case hkMouse: {
					short x = mi.X - phki->dx;
					short y = mi.Y - phki->dy;
					if ((fHit = (flags & HTA_INSURE_UP) || (
						x >= pha->X && x < pha->X + pha->Width &&
						y >= pha->Y && y < pha->Y + pha->Height)
						) != FALSE)
					{
						hi.Shiftflags = ShiftFlags();
						hi.inpt = inptMouse;
						hi.X = mi.X - phki->dx - pha->X;
						hi.Y = mi.Y - phki->dy - pha->Y;
				 		hi.Buttons = mi.Buttons;
					}
				}
				break;
			default:
				hi.Shiftflags = phki->fShift;
				hi.KeyCode = phki->KeyCode;
				switch (hk) {
				case hkCursor:
					hi.inpt = inptCursor;
					hi.fCursor = fCursorFromKey (phki->KeyCode);
					break;
				case hkKey:
					hi.inpt = inptKey;
					break;
				default:
				/* case hkHotKey, hkSpace, hkEnter, hkEscape: */
					hi.inpt = inptHitKey;
					break;
				}
				break;
			}
			if (fHit) {
				((MouseFunc)(pha->Data))(&hi);
			}
		}
	}

}} /* HitThisArea */

/*********************************************************************
 *
 * FindAndHitAnArea
 *
 * SYNOPSIS
 *		HitArea *FindAndHitAnArea (
 *			HitArea *pha, 
 *			HKI		*phki
 *		)
 *
 * PURPOSE
 *		To find and hit an area in accordance with the kind of hit (mouse,
 *		space key, enter key, escape key, other ascii key press).
 *
 * INPUT
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
HitArea *FindAndHitAnArea (
	HitArea *pha, 
	HKI	*phki
)
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "HitCoordArea";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
{
	MouseInfo mi;
	short fLeftButton;
	short x, y;
	HK	hk = phki->hk;	/* Hit kind */

	if (hk == hkMouse) {
		ReadMouse (&mi);
		if (!(fLeftButton = mi.Buttons & MOUSE_LEFTBUTTON) &&
			!(mi.Buttons & MOUSE_RIGHTBUTTON)) 
		{
			/* No mouse buttons pressed so quit */
			return NULL;
		}
		x = mi.X - phki->dx;
		y = mi.Y - phki->dy;
	}

	while (!END_HITAREA (pha)) {
		short fHit = FALSE;

		if ((pha->Flags & HTA_SUBAREAS) && hk != hkMouse) {
			HKI		hki = *phki;
			HitArea *phaSA;

			if ((phaSA = FindAndHitAnArea (pha->Data, &hki)) != NULL) {
				pha = phaSA;
				break;
			}
		}

		/* Determine if its a hit */
		if (!(pha->fState & HA_DISABLED)) {
			switch (hk) {
			case hkMouse:
				fHit = ((x >= pha->X && x < pha->X + pha->Width &&
					y >= pha->Y && y < pha->Y + pha->Height)
					&&
					(fLeftButton || pha->Flags & HTA_REPORTRIGHT));
				break;
			case hkHotKey:
				fHit = (toupper (pha->KeyCode) == toupper(phki->KeyCode));
				break;
			case hkSpace:
				fHit = (pha->fState & HA_SPACE);
				break;
			case hkEnter:
				fHit = (pha->fState & HA_ENTER);
				break;
			case hkEscape:
				fHit = (pha->Flags & HTA_ESCAPE);
				break;
			default:
				return NULL;
			}
		} 

		if (fHit) {
			if ((pha->Flags & HTA_SUBAREAS) && hk == hkMouse) {
				HKI		hki = *phki;
				HitArea *phaSA;

				hki.dx += pha->X;
				hki.dy += pha->Y;
				if ((phaSA = FindAndHitAnArea (pha->Data, &hki)) != NULL) {
					pha = phaSA;
					break;
				}
			} else {
				HitThisArea (pha, phki);
				break;
			}
		} 
		pha = NEXT_HITAREA (pha);
	}
	
	return (END_HITAREA(pha)) ? NULL : pha;

}} /* FindAndHitAnArea */

/*********************************************************************
 *
 * CheckHitAreas
 *
 * SYNOPSIS
 *		HitArea *CheckHitAreas (HitArea *pha, short xoff, short yoff)
 *
 * PURPOSE
 *		To check MOUSE BUTTON ACTIVITY and call functions for hit area
 *		that mouse is over if necessary.
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
HitArea *CheckHitAreas (HitArea *pha, short xoff, short yoff)
{
	FCT	fct;
	HKI	hki;

	memset (&fct, 0, sizeof (FCT));
	fct.pha = pha;
	FillOutFCT (&fct);

	hki.hk = hkMouse;
	hki.dx = xoff; 
	hki.dy = yoff;
	hki.pfct = &fct;

	return FindAndHitAnArea (pha, &hki);

} /* CheckHitAreas */

/*********************************************************************
 *
 * CheckKeyHitAreas
 *
 * SYNOPSIS
 *		HitArea *CheckKeyHitAreas (
 *			HitArea *pha, 
 *			short KeyCode,
 *			UWORD fShift
 *		)
 *
 * PURPOSE
 *		To update hit areas or call functions based on key press. Tab and 
 *		arrow keys cause 'Enter Focus' and 'Space Focus' to change.
 *		
 *
 * INPUT
 *		pha:	Pointer to HitArea Array.
 *		KeyCode: GetKey();
 *		fShift:	ShiftFlags();
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
HitArea *CheckKeyHitAreas (
	HitArea *pha, 
	UWORD KeyCode
)
{
	HitArea		*phaHit;
	FCT			 fct;
	HKI			 hki;
	short		 dFocus = 0; /* Change in focus: -1 = back; +1 = forward */
	short		 fShift;

	fShift = KeyCode >> 8;
	memset (&fct, 0, sizeof (FCT));
	fct.pha = pha;
	hki.pfct = &fct;
	FillOutFCT (&fct);

	switch (KeyCode) {
	case UP_KEY: case LEFT_KEY: case DOWN_KEY: case RIGHT_KEY:
		if (!fct.phaSpace || !(fct.phaSpace->Flags & HTA_CURSOR)) {
			dFocus = (KeyCode == UP_KEY || KeyCode == LEFT_KEY) ? -1 : +1;
		} /* else cursor key will get passed to area routine */
		break;
	case TAB_KEY:
		dFocus = (fShift & KEY_SHIFT) ? -1 : +1;
		break;
	}

	if (dFocus) {
		MoveFocus (&fct, (dFocus > 0));
		phaHit = NULL;
	} else {
		hki.KeyCode = KeyCode;
		hki.fShift = fShift;
		switch (KeyCode) {
		case UP_KEY: case DOWN_KEY: case LEFT_KEY: case RIGHT_KEY:
			phaHit = fct.phaSpace;
			hki.hk = hkCursor;
			HitThisArea (fct.phaSpace, &hki);
			break;
		case RETURN_KEY:
			hki.hk = hkEnter;
			phaHit = FindAndHitAnArea (pha, &hki);
			break;
		case ' ':
			if (fct.phaSpace && (fct.phaSpace->Flags & HTA_KEYS)) {
				/* Give space input to space focused area that wants keys */
				hki.hk = hkKey;
				HitThisArea (fct.phaSpace, &hki);
			} else {
				hki.hk = hkSpace;
				phaHit = FindAndHitAnArea (pha, &hki);
			}
			break;
		case ESC_KEY:
			hki.hk = hkEscape;
			phaHit = FindAndHitAnArea (pha, &hki);
			break;
		default:
			if (fct.phaSpace && fct.phaSpace->Flags & HTA_KEYS) {
				/* Give key input to space focused area that wants keys */
				hki.hk = hkKey;
				HitThisArea (fct.phaSpace, &hki);
			} else { /* Find hot key area */
				hki.hk = hkHotKey;
				phaHit = FindAndHitAnArea (pha, &hki);
			}
			break;
		}
	}

	return phaHit;

} /* CheckKeyHitAreas */

