/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * HNDLEVNT.C
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 08/08/89 
 *   MODIFIED : 02/05/95
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		
 *
 * HISTORY
 *		08/08/90 (RGM) - Added Lattice 5.0x support.
 *		02/05/95 (dcc) - #include <echidna/events.h> to verify 
 *						 function prototypes.
 *
*/

#include <echidna/platform.h>
#include "switches.h"

#include <echidna/eerrors.h>
#include <echidna/events.h>				//verify function prototypes
#include <echidna/strings.h>
#include <echidna/listfunc.h>
#include <echidna/menus2.h>
#include <echidna/mouse.h>
#include <echidna/keyboard.h>
#include <echidna/hitareas.h>

/**************************** C O N S T A N T S ***************************/


/******************************** T Y P E S *******************************/


/****************************** G L O B A L S *****************************/

MouseFunc	FN_AfterEvent;
MouseFunc	FN_DontWait;
MouseFunc	FN_BeforeEvent;

static short BeforeCount = 0;

/******************************* M A C R O S ******************************/


/***************************** R O U T I N E S ****************************/

/*********************************************************************
 *
 * BeforeEvent
 *
 * SYNOPSIS
 *		void BeforeEvent (void)
 *
 * PURPOSE
 *		Before an event happens clean up?
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void BeforeEvent (void)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "BeforeEvent";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	if (!BeforeCount && FN_BeforeEvent) {
		MouseInfo	 mi;
		HitInfo		 hi;

		ReadMouse (&mi);
		hi.X          = mi.X;
		hi.Y          = mi.Y;
		hi.Buttons    = mi.Buttons | MOUSE_BEFOREEVENT;
		hi.Shiftflags = ShiftFlags();
		FN_BeforeEvent (&hi);
	}
	BeforeCount++;

} /* BeforeEvent */

/*********************************************************************
 *
 * AfterEvent
 *
 * SYNOPSIS
 *		void AfterEvent (void)
 *
 * PURPOSE
 *		After an event happens clean up?
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void AfterEvent (void)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "AfterEvent";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	BeforeCount--;
	if (!BeforeCount && FN_AfterEvent) {
		MouseInfo	 mi;
		HitInfo		 hi;

		ReadMouse (&mi);
		hi.X          = mi.X;
		hi.Y          = mi.Y;
		hi.Buttons    = mi.Buttons | MOUSE_AFTEREVENT;
		hi.Shiftflags = ShiftFlags();
		FN_AfterEvent (&hi);
	}

} /* AfterEvent */

/**************************************************************************
 *
 * FindEvent
 *
 * SYNOPSIS
 *   EventState *FindEvent (char *event);
 *
 * FUNCTION
 *   Finds an EventState by it's "name/label"
 *
 * INPUTS
 *
 *
 * RESULTS
 *
 *
 *
*/
EventState *FindEvent (char *name)
{
	EventState	*es = Head(EventList);

	while (!IsEOList (es)) {
		if (!stricmp (name, NodeName(es))) {
			return es;
		}
		es = Next (es);
	}
	return NULL;
}

/*********************************************************************
 *
 * ExcludeItems
 *
 * SYNOPSIS
 *		void ExcludeItems (
 *			ListType				*items
 *		)
 *
 * PURPOSE
 * Turn 'off' items in list (state variables)
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void ExcludeItems (
	ListType				*items
)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ExcludeItems";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		register StateTracker	*st;
		register EventState		*es;

		if (items) {
			st = Head(items);
			while (!IsEOList (st)) {

				es = st->EventState;
				/*
				** If Menu has a state variable set it to FALSE.
				*/
				if (es->StateVar) {
					*es->StateVar = FALSE;
				}

				st = Next (st);
			}
		}
	}
} /* ExcludeItems */

/*********************************************************************
 *
 * SetStateVar
 *
 * SYNOPSIS
 *		void SetStateVar (EventState *es, short value)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void SetStateVar (EventState *es, short value)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "SetStateVar";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	if (es->StateVar) {
		*(es->StateVar) = value;

		if (value) {
			ExcludeItems (es->ExcludeList);
		}
	}

} /* SetStateVar */


/**************************************************************************
 *
 * TriggerEvent
 *
 * SYNOPSIS
 *   int TriggerEvent (EventState *event);
 *
 * FUNCTION
 *   Called to "Do" an event.
 *
 * INPUTS
 *
 *
 * RESULTS
 *
 *
 * HISTORY
 *
 *
*/
short TriggerEvent (
	EventState		*estate,
	short			 etype,
	void			*edata
) {
	register short	*statevar;
	short			 flag  = TRUE;
	EventInfo		 ei;

	ei.Event        = estate;
	ei.EventType    = etype;
	ei.EventTrigger = edata;
	statevar        = estate->StateVar;

	/*
	** If 'it' has a state variable.....
	*/
	if (statevar) {

		/*
		** If item doesn't have a checkmark but it's MENUTOGGLE
		** flag is set then toggle it's state variable.
		*/
		if (estate->Flags & MNU_HAS_CHECK) {
			SetStateVar (estate, !*statevar);
		/*
		** Otherwise just set its state to TRUE.
		*/
		} else {
			SetStateVar (estate, TRUE);
		}
	}

	/*
	** It state has a function then call it.
	*/
	if (estate->Function) {
		BeforeEvent ();
		flag &= (*estate->Function)(&ei);
		AfterEvent ();
	}

	return flag;
}

