/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * HOTTEXT.C
 *
 * PROGRAMMER : Juan M. Alvarado
 *    VERSION : 00.000
 *    CREATED : 02/18/92
 *   MODIFIED : 02/05/95
 *       TABS : 04 08
 *
 *	     \|///-_
 *	     \oO///_
 *	-----w/-w-----
 *	 E C H i D N A
 *	--------------
 *
 * DESCRIPTION
 *		HotText routines.
 *
 * HISTORY
 *		02/05/95 (dcc) - #include <echidna/hottext.h> to verify
 *						 function prototypes.
 *
*/

#include <echidna/platform.h>
#include "switches.h"

#include <stdlib.h>
#include <stddef.h>
#include <echidna/etypes.h>
#include <echidna/eerrors.h>
#include <echidna/graphics.h>
#include <echidna/fonts.h>
#include <echidna/eui.h>
#include <echidna/hottext.h>			// verify function prototypes
#include <echidna/naming.h>

/**************************** C O N S T A N T S ***************************/


/******************************** T Y P E S *******************************/


/****************************** G L O B A L S *****************************/


/******************************* M A C R O S ******************************/


/***************************** R O U T I N E S ****************************/


/*********************************************************************
 *
 * DrawHotTextN
 *
 * SYNOPSIS
 *		short DrawHotTextN (short x, short y, char *psz, size_t len)
 *
 * PURPOSE
 *		To draw only the first 'len' chars of hottext.
 *
 * INPUT
 *		x: X coordinate of start of text.
 *		y: Y coordinate of top of text.
 *		psz: See HotText.H for explanation of hottext format.
 *		
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *		Returns x coordinate of end of drawn text.
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short DrawHotTextN (short x, short y, char *psz, size_t len)
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "DrawHotTextN";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
{
	Font	*pfont = GetActiveWindow()->Font;
	UBYTE	 ub;
	GSH		*pgsh;
	GSH		**ppgshLetters	= pfont->Letters;
	UBYTE	 first		= pfont->FirstLetter;
	UBYTE	 last		= pfont->LastLetter;
	char 	 chHot = 0;
	char	*psz2 = psz;

	if (*psz++ == '&') {
		if ((chHot = *psz++) != 0) {
			psz2 = psz;
		} else {
			return x; /* Illegal HotText: "&" */
		}
	}

	while (len && *psz2) {
		ub = *((UBYTE *)psz2);
		psz2++;
		if (ub >= first && ub <= last) {
			pgsh = ppgshLetters[ub - first];
			DrawGShape (pgsh, x, y);
			if (chHot == ub) { /* Underline it */
				DrawHLine (x, y + pgsh->Height-1, pgsh->Width);
				chHot = 0;
			}
			x += pgsh->Width;
		}
		len--;
	}

	return x;

}} /* DrawHotTextN */

/*********************************************************************
 *
 * DrawHotText
 *
 * SYNOPSIS
 *		short DrawHotText (short x, short y, char *psz)
 *
 * PURPOSE
 *
 * INPUT
 *		x: X coordinate of start of text.
 *		y: Y coordinate of top of text.
 *		psz: See HotText.H for explanation of hottext format.
 *		
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *		Returns x coordinate of end of drawn text.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short DrawHotText (short x, short y, char *psz)
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "DrawHotText";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
{
	return DrawHotTextN (x, y, psz, strlen(psz) - 2 * (*psz == '&'));

}} /* DrawHotText */

/*********************************************************************
 *
 * HotTextLenN
 *
 * SYNOPSIS
 *		short HotTextLenN (char *psz, size_t len)
 *
 * PURPOSE
 *		To return width of image drawn if DrawHotTextN(x, y, psz, len)
 *		were called.
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short HotTextLenN (char *psz, size_t len)
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "HotTextLenN";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
{
	Font	*pfont = GetActiveWindow()->Font;
	UBYTE	 ub;
	GSH		**ppgshLetters	= pfont->Letters;
	UBYTE	 first		= pfont->FirstLetter;
	UBYTE	 last		= pfont->LastLetter;
	short	 cx = 0;
	char	*psz2 = psz;

	if (*psz++ == '&') {
		if (*psz++) {
			psz2 = psz;
		} else {
			return 0; /* Illegal HotText: "&" */
		}
	}

	while (len && *psz2) {
		ub = *((UBYTE *)psz2);
		psz2++;
		if (ub >= first && ub <= last) {
			cx += ppgshLetters[ub - first]->Width;
		}
		len--;
	}

	return cx;

}} /* HotTextLenN */

/*********************************************************************
 *
 * HotTextLen
 *
 * SYNOPSIS
 *		short HotTextLen (char *psz)
 *
 * PURPOSE
 *		To return width of image drawn if DrawHotText(x, y, psz)
 *		were called.
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short HotTextLen (char *psz)
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "HotTextLen";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
{
	return HotTextLenN (psz, strlen(psz) - 2 * (*psz == '&'));
}} /* HotTextLen */

