/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * RM_TSINF.C
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 03/17/92
 *   MODIFIED : 07/27/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		Displays a dialog for Room or Tileset Info
 *
 * HISTORY
 *
*/

#include <echidna/platform.h>
#include "switches.h"

#include <echidna/strings.h>
#include <echidna/listfunc.h>
#include <echidna/emacros.h>
#include <echidna/eerrors.h>
#include <echidna/grafx.h>
#include <echidna/fonts.h>
#include <echidna/lstgadg.h>
#include <echidna/hitareas.h>
#include <echidna/windows.h>
#include <echidna/eui.h>
#include <echidna/edraw.h>
#include <echidna/events.h>
#include <echidna/mouse.h>
#include <echidna/keyboard.h>

#include "tuglbl.h"

#define LOCAL static
/**************************** C O N S T A N T S ***************************/

#define RT_WHITE	0
#define RT_BLACK	1
#define RT_BLUE		2
#define RT_ORANGE	3

#define RTDLG_WIDTH		299
#define RTDLG_HEIGHT	105
#define RTDLG_LEFT		((SCREEN_WIDTH / 2) - (RTDLG_WIDTH / 2))
#define RTDLG_TOP		((((SCREEN_HEIGHT - 12) / 2) - (RTDLG_HEIGHT / 2)) + 12)

#define TS_HEIGHT		141
#define TS_TOP			((((SCREEN_HEIGHT - 12) / 2) - (TS_HEIGHT / 2)) + 12)

#define RTDLG_TLST_LEFT		3
#define RTDLG_TLST_TOP		27
#define RTDLG_TLST_WIDTH	122
#define RTDLG_TLST_HEIGHT	74
#define RTDLG_TLST_ARROW_W	12
#define RTDLG_TLST_NAME_W	(RMDLG_TLIST_WIDTH - 1 - RMDLG_TLST_ARROW_W)

#define RT_NAME_LEFT		3
#define RT_NAME_TOP			14
#define RT_NAME_WIDTH		292
#define RT_NAME_HEIGHT		11

#define ARROW_WIDTH			11
#define ARROW_HEIGHT		11
#define NUMBER_WIDTH		57
#define NUMBER_HEIGHT		11

#define RTDLG_NUM_LEFT		127
#define RT_TYPE_TOP			27
#define RT_NUM_TOP			41
#define RT_WIDTH_TOP		55
#define	RT_HEIGHT_TOP		69
#define NTI_TOP				83
#define NTI_WIDTH			9
#define	NTI_HEIGHT			9
#define LABEL_LEFT			(RTDLG_NUM_LEFT + ARROW_WIDTH * 2 + NUMBER_WIDTH)

#define CANCEL_LEFT			198
#define CANCEL_TOP			OK_TOP
#define CANCEL_WIDTH		49
#define CANCEL_ID			10
#define OK_LEFT				(CANCEL_LEFT + CANCEL_WIDTH + 2)
#define	OK_TOP				89
#define OK_WIDTH			43
#define OK_ID				11
#define BUTTON_HEIGHT		12
#define TSET_TOP			103
#define	TSET_HEIGHT			37

#define TS_COM_LEFT			3
#define	TS_COM1_TOP			113
#define TS_COM2_TOP			126


/******************************** T Y P E S *******************************/

typedef struct {
	NodeType	 Node;
	short		 Type;
} RT_Type;

/****************************** G L O B A L S *****************************/

LOCAL Window	*RTWin;
LOCAL Font		*RTFont;
LOCAL Font		*RTSFont;
LOCAL UBYTE		 RtColors[4];

LOCAL short		 RTdone;
LOCAL short		 RTsuccess;

LOCAL short		 RTTop;
LOCAL ListType	*RTList;
LOCAL LGAD		*RTlgad;

LOCAL short		 TileWidth = 16;
LOCAL short		 TileHeight = 16;
LOCAL short		 RoomWidth = 20;
LOCAL short		 RoomHeight = 12;

LOCAL ListType	 TileTypeListX;
LOCAL ListType	*TileTypeList = &TileTypeListX;
LOCAL ListType	 RoomTypeListX;
LOCAL ListType	*RoomTypeList = &RoomTypeListX;

char			*RoomTypeGroup = "No Room Type Group";
char			*TileTypeGroup = "No Tile Type Group";

/******************************* M A C R O S ******************************/

#define ED_Button(x,y,w,h)	\
	ED_Rect ((x), (y), (w) - 1, (h) -1),				\
	ED_RelHLine ((x) + 1, (w) - 1, (y) + (h) - 1),		\
	ED_RelVLine ((x) + (w) - 1, (y) + 1, (h) - 1)		\

/******************************* T A B L E S ******************************/

LOCAL char *s0 = "Okay";
LOCAL char *s1 = "Cancel";
LOCAL char *s2 = "Tile Comments";
LOCAL char *s3 = "Save Grafx";

LOCAL char **RtStr[] = {
#define OK_STR	0
	&s0,
#define CANCEL_STR	1
	&s1,
#define COMMENT_STR	2
	&s2,
#define SAVE_STR	3
	&s3,
};

LOCAL struct TextFont **RtFonts[] = {
	(struct TextFont **)&RTFont,
	(struct TextFont **)&RTSFont,
};

LOCAL EPic TSetEPic[] = {
	ED_Font (1),
	ED_DrawMode (JAM2),
	ED_FgColorTbl (RT_WHITE),
	ED_Rect (0, TSET_TOP + 1, RTDLG_WIDTH - 1, TSET_HEIGHT - 2),
	ED_DrawMode (JAM1),
	ED_FgColorTbl (RT_BLACK),
	ED_Button (0, TSET_TOP, RTDLG_WIDTH, TSET_HEIGHT),

	/* comment string gads */
	ED_Rect (TS_COM_LEFT, TS_COM1_TOP, RT_NAME_WIDTH, RT_NAME_HEIGHT),
	ED_Rect (TS_COM_LEFT, TS_COM2_TOP, RT_NAME_WIDTH, RT_NAME_HEIGHT),
	ED_PosText (4, TSET_TOP + 3, COMMENT_STR),

	/* NTI thing */
	ED_Rect (RTDLG_NUM_LEFT, NTI_TOP, NTI_WIDTH, NTI_HEIGHT),
	ED_PosText (RTDLG_NUM_LEFT + NTI_WIDTH + 2, NTI_TOP + 2, SAVE_STR),

	ED_End,
};

LOCAL EPicture RTEPicture = {
	RTEPic,
	RtFonts,
	RtStr,
	RtEPics,
	NULL,
	RtColors,
};

LOCAL EPicture NumEPicture = {
	NumberEPic,
	RtFonts,
	RtStr,
	RtEPics,
	NULL,
	RtColors,
};

LOCAL EPicture TSetEPicture = {
	TSetEPic,
	RtFonts,
	RtStr,
	RtEPics,
	NULL,
	RtColors,
};
	

#define NUM_TYPE_NDX	0
#define NUM_IDNUM_NDX	1
#define NUM_WIDTH_NDX	2
#define NUM_HEIGHT_NDX	3
#define NUM_COM1_NDX	4
#define NUM_COM2_NDX	5
#define NUM_NAME_NDX	6

LOCAL short SaveGrafx;

LOCAL char	Name[EIO_MAXPATH];
LOCAL char	Com1[EIO_MAXPATH];
LOCAL char	Com2[EIO_MAXPATH];
LOCAL char	NumStrings[4][EIO_MAXPATH];

#define NUM_STRING_GADS	7

LOCAL short StrHeights[] = {
	RT_TYPE_TOP + 2, 
	RT_NUM_TOP + 2,
	RT_WIDTH_TOP + 2,
	RT_HEIGHT_TOP + 2,
	TS_COM1_TOP + 2,
	TS_COM2_TOP + 2,
 	RT_NAME_TOP + 2,
};

LOCAL short NewGadWidths[] = {
	ARROW_WIDTH, 
	NUMBER_WIDTH - 2,
	ARROW_WIDTH, 
	ARROW_WIDTH,
	NUMBER_WIDTH - 2,
	ARROW_WIDTH,
	NTI_WIDTH,
	RT_NAME_WIDTH,
	RT_NAME_WIDTH,
};

LOCAL EditString NumString[NUM_STRING_GADS] = {
{ NumStrings[0], (NUMBER_WIDTH - 5) / 8, RTDLG_LEFT + RTDLG_NUM_LEFT + ARROW_WIDTH + 1, RTDLG_TOP + RT_TYPE_TOP + 2, NULL, &NumString[1], },
{ NumStrings[1], (NUMBER_WIDTH - 5) / 8, RTDLG_LEFT + RTDLG_NUM_LEFT + ARROW_WIDTH + 1, RTDLG_TOP + RT_NUM_TOP + 2, NULL, &NumString[2], },
{ NumStrings[2], (NUMBER_WIDTH - 5) / 8, RTDLG_LEFT + RTDLG_NUM_LEFT + ARROW_WIDTH + 1, RTDLG_TOP + RT_WIDTH_TOP + 2, NULL, &NumString[3],},
{ NumStrings[3], (NUMBER_WIDTH - 5) / 8, RTDLG_LEFT + RTDLG_NUM_LEFT + ARROW_WIDTH + 1, RTDLG_TOP + RT_HEIGHT_TOP + 2, NULL, &NumString[4], },
{ Com1, (RT_NAME_WIDTH - 5) / 8, RTDLG_LEFT + TS_COM_LEFT + 2, RTDLG_TOP + TS_COM1_TOP + 2, NULL, &NumString[5], },
{ Com2, (RT_NAME_WIDTH - 5) / 8, RTDLG_LEFT + TS_COM_LEFT + 2, RTDLG_TOP + TS_COM2_TOP + 2, NULL, &NumString[6], },
{ Name, (RT_NAME_WIDTH - 5) / 8, RTDLG_LEFT + RT_NAME_LEFT + 2, RTDLG_TOP + RT_NAME_TOP + 2, NULL, &NumString[0], },
};

typedef EditString	*ESPtr;

LOCAL ESPtr ESTable[4][NUM_STRING_GADS] = {
{ &NumString[1], &NumString[2], &NumString[3], &NumString[4], &NumString[5], &NumString[6], &NumString[0], }, /* tset + new */
{ &NumString[1], &NumString[4], NULL         , NULL         , &NumString[5], &NumString[6], &NumString[0], }, /* tset       */
{ &NumString[1], &NumString[2], &NumString[3], &NumString[6], NULL         , NULL         , &NumString[0], }, /* room + new */
{ &NumString[1], &NumString[6], NULL         , NULL         , NULL         , NULL         , &NumString[0], }, /* room + new */
};

/***************************** R O U T I N E S ****************************/

/*********************************************************************
 *
 * UpdateListMark
 *
 * SYNOPSIS
 *		LOCAL void UpdateListMark (void)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL void UpdateListMark (void)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "UpdateListMark";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		RT_Type	*rt;
		short	 type;
		short	 count;
		short	 pos = (-1);

		count = 0;
		type  = atoi (NumStrings[NUM_TYPE_NDX]);
		rt    = Head (RTList);
		while (!EndOfList (rt)) {
			rt->Node.Priority = 0;
			if (rt->Type == type) {
				type = -30000;
				pos  = count;
				rt->Node.Priority |= LGAD_NOD_SELECTED;
			}
			count++;
			rt = Next (rt);
		}

		if (pos < 0) {
			SetLGADList (RTlgad, RTList, 1000);
		} else {
			SetLGADListPos (RTlgad, pos, LGAD_POS_MID);
		}
	}

} /* UpdateListMark */

/*********************************************************************
 *
 * RedrawList
 *
 * SYNOPSIS
 *		void RedrawList (void)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void RedrawList (void)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "RedrawList";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	UpdateListMark ();
	/*
	SetLGADListPos (RTlgad, UpdateListMark(), LGAD_POS_MID);
	UpdateListMark ();
	SetLGADList (RTlgad, RTList, 1000);
	*/

} /* RedrawList */

/*********************************************************************
 *
 * IncDecNumber
 *
 * SYNOPSIS
 *		LOCAL void IncDecNumber (void) 
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL void IncDecNumber (HitInfo *hi) 
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "IncDecNumber";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		HitArea		*ha  = hi->HitArea;
		short		 num = ha->ID & 0x0F;
		short		 dir = (ha->ID & 0x10) ? (-1) : (1);
		short		 val;

		val  = atoi (NumStrings[num]);
		val += dir;
		val  = max (0, val);
		val  = min (32766, val);

		itoa (val, NumStrings[num], 10);
		BeforeGraphics ();
		UpdateEditString (&NumString[num]);
		if (num == NUM_TYPE_NDX) {
			RedrawList ();
		}
		AfterGraphics ();

	}
} /* IncDecNumber */

/*********************************************************************
 *
 * UpdateNTI
 *
 * SYNOPSIS
 *		void UpdateNTI (void)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void UpdateNTI (void)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "UpdateNTI";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		static short xy[][2] = {
			{ RTDLG_LEFT + RTDLG_NUM_LEFT + 1, NTI_TOP + 1, },
			{ RTDLG_LEFT + RTDLG_NUM_LEFT + 2, NTI_TOP + 2, },
			{ RTDLG_LEFT + RTDLG_NUM_LEFT + 3, NTI_TOP + 3, },
			{ RTDLG_LEFT + RTDLG_NUM_LEFT + 4, NTI_TOP + 4, },
			{ RTDLG_LEFT + RTDLG_NUM_LEFT + 5, NTI_TOP + 5, },
			{ RTDLG_LEFT + RTDLG_NUM_LEFT + 6, NTI_TOP + 6, },
			{ RTDLG_LEFT + RTDLG_NUM_LEFT + 7, NTI_TOP + 7, },
			{ RTDLG_LEFT + RTDLG_NUM_LEFT + 7, NTI_TOP + 1, },
			{ RTDLG_LEFT + RTDLG_NUM_LEFT + 6, NTI_TOP + 2, },
			{ RTDLG_LEFT + RTDLG_NUM_LEFT + 5, NTI_TOP + 3, },
			{ RTDLG_LEFT + RTDLG_NUM_LEFT + 4, NTI_TOP + 4, },
			{ RTDLG_LEFT + RTDLG_NUM_LEFT + 3, NTI_TOP + 5, },
			{ RTDLG_LEFT + RTDLG_NUM_LEFT + 2, NTI_TOP + 6, },
			{ RTDLG_LEFT + RTDLG_NUM_LEFT + 1, NTI_TOP + 7, },
			{ -1, },
		};
		short	i = 0;

		BeforeGraphics ();

		SetPenColor (RtColors[RT_WHITE]);
		DrawRect (
				RTDLG_LEFT + RTDLG_NUM_LEFT + 1,
				RTTop  + NTI_TOP + 1,
				NTI_WIDTH - 2,
				NTI_HEIGHT - 2);

		if (SaveGrafx) {
			SetPenColor (RtColors[RT_BLACK]);
			while (xy[i][0] >= 0) {
				DrawPixel (xy[i][0], RTTop + xy[i][1]);
				i++;
			}
		}

		AfterGraphics ();
	}

} /* UpdateNTI */

/*********************************************************************
 *
 * ToggleNTI
 *
 * SYNOPSIS
 *		LOCAL void ToggleNTI (void) 
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL void ToggleNTI (void) 
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ToggleNTI";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	SaveGrafx = !SaveGrafx;
	UpdateNTI ();

} /* ToggleNTI */

/*********************************************************************
 *
 * void	SelectFunc
 *
 * SYNOPSIS
 *		LOCAL void	SelectFunc (void *pvdata, UWORD fWhy)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL void SelectFunc (void *pvdata, UWORD fWhy)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "void	SelectFunc";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	fWhy = fWhy;
	{
		LGAD *plgad = pvdata;
		RT_Type	*pnod = (RT_Type *)pnodLGADFirstSelected (plgad);

		if (pnod) {
			itoa (pnod->Type, NumStrings[0], 10);
			UpdateEditString (&NumString[0]);
		}
	}
} /* void SelectFunc */

/********************************* Gadgets ********************************/
	
LOCAL HitArea RTHitArea[] = {
	{ RT_NAME_LEFT, RT_NAME_TOP, RT_NAME_WIDTH, RT_NAME_HEIGHT,
		HTA_DOWN | HTA_REPEAT | HTA_INSURE_UP, 0, HighlightString, &NumString[NUM_NAME_NDX], },
#define LIST_GAD	1
	{ RTDLG_TLST_LEFT, RTDLG_TLST_TOP, RTDLG_TLST_WIDTH, RTDLG_TLST_HEIGHT,
		HTA_SUBAREAS, 0, NULL, NULL, },

	/* Type Number */
	{ RTDLG_NUM_LEFT, RT_TYPE_TOP, ARROW_WIDTH, ARROW_HEIGHT,
		HTA_COMPLEMENT, 0x10, IncDecNumber, },
	{ RTDLG_NUM_LEFT + ARROW_WIDTH, RT_TYPE_TOP, NUMBER_WIDTH - 2, NUMBER_HEIGHT,
		HTA_DOWN | HTA_REPEAT | HTA_INSURE_UP, 0, HighlightString, &NumString[NUM_TYPE_NDX], },
	{ RTDLG_NUM_LEFT + ARROW_WIDTH + NUMBER_WIDTH - 1, RT_TYPE_TOP, ARROW_WIDTH, ARROW_HEIGHT,
		HTA_COMPLEMENT, 0x00, IncDecNumber, },

	/* IDNum Number */
	{ RTDLG_NUM_LEFT, RT_NUM_TOP, ARROW_WIDTH, ARROW_HEIGHT,
		HTA_COMPLEMENT, 0x11, IncDecNumber, },
	{ RTDLG_NUM_LEFT + ARROW_WIDTH, RT_NUM_TOP, NUMBER_WIDTH - 2, NUMBER_HEIGHT,
		HTA_DOWN | HTA_REPEAT | HTA_INSURE_UP, 0, HighlightString, &NumString[NUM_IDNUM_NDX], },
	{ RTDLG_NUM_LEFT + ARROW_WIDTH + NUMBER_WIDTH - 1, RT_NUM_TOP, ARROW_WIDTH, ARROW_HEIGHT,
		HTA_COMPLEMENT, 0x01, IncDecNumber, },

	/* Width Number */
#define WIDTH_GADS	8
	{ RTDLG_NUM_LEFT, RT_WIDTH_TOP, ARROW_WIDTH, ARROW_HEIGHT,
		HTA_COMPLEMENT, 0x12, IncDecNumber, },
	{ RTDLG_NUM_LEFT + ARROW_WIDTH, RT_WIDTH_TOP, NUMBER_WIDTH - 2, NUMBER_HEIGHT,
		HTA_DOWN | HTA_REPEAT | HTA_INSURE_UP, 0, HighlightString, &NumString[NUM_WIDTH_NDX], },
	{ RTDLG_NUM_LEFT + ARROW_WIDTH + NUMBER_WIDTH - 1, RT_WIDTH_TOP, ARROW_WIDTH, ARROW_HEIGHT,
		HTA_COMPLEMENT, 0x02, IncDecNumber, },

#define HEIGHT_GADS	11
	/* Height Number */
	{ RTDLG_NUM_LEFT, RT_HEIGHT_TOP, ARROW_WIDTH, ARROW_HEIGHT,
		HTA_COMPLEMENT, 0x13, IncDecNumber, },
	{ RTDLG_NUM_LEFT + ARROW_WIDTH, RT_HEIGHT_TOP, NUMBER_WIDTH - 2, NUMBER_HEIGHT,
		HTA_DOWN | HTA_REPEAT | HTA_INSURE_UP, 0, HighlightString, &NumString[NUM_HEIGHT_NDX], },
	{ RTDLG_NUM_LEFT + ARROW_WIDTH + NUMBER_WIDTH - 1, RT_HEIGHT_TOP, ARROW_WIDTH, ARROW_HEIGHT,
		HTA_COMPLEMENT, 0x03, IncDecNumber, },

#define NTI_GAD	14
	/* NTI Gadget */
	{ RTDLG_NUM_LEFT, NTI_TOP, NTI_WIDTH, NTI_HEIGHT,
		0, 0, ToggleNTI, },

#define COMMENT_GADS	15
	/* Comment Gads */
	{ TS_COM_LEFT, TS_COM1_TOP,  RT_NAME_WIDTH, RT_NAME_HEIGHT,
		HTA_DOWN | HTA_REPEAT | HTA_INSURE_UP, 0, HighlightString, &NumString[NUM_COM1_NDX], },
	{ TS_COM_LEFT, TS_COM2_TOP,  RT_NAME_WIDTH, RT_NAME_HEIGHT,
		HTA_DOWN | HTA_REPEAT | HTA_INSURE_UP, 0, HighlightString, &NumString[NUM_COM2_NDX], },

	/* Ok / Cancel */
	{ OK_LEFT, OK_TOP, OK_WIDTH, BUTTON_HEIGHT,
		HTA_COMPLEMENT, OK_ID, NULL},
	{ CANCEL_LEFT, CANCEL_TOP, CANCEL_WIDTH, BUTTON_HEIGHT,
		HTA_COMPLEMENT, CANCEL_ID, NULL},
	{ HTA_LAST_AREA, HTA_LAST_AREA, HTA_LAST_AREA, HTA_LAST_AREA, },
};

/*********************************************************************
 *
 * RoomNTileInfoReq
 *
 * SYNOPSIS
 *		short RoomNTileInfoReq ()
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short RoomNTileInfoReq (RoomType *room, TileSetType *tst, short new)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "RoomNTileInfoReq";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		SaveAreaType		*sa = NULL;
		HitArea				*ha;
		struct	RastPort	 rp = { NULL, };
		short				 height;
		short				 top;
		LGAD				 lgadx;
		LGAD				*lgad = &lgadx;

		RTlgad    = lgad;
		RTdone    = FALSE;
		RTsuccess = FALSE;
		
		RTWin   = MainWindow;
		RTFont  = &MainFont;
		RTSFont = &SmallFont;
		RtColors[RT_WHITE ] = RTWin->White;
		RtColors[RT_BLACK ] = RTWin->Black;
		RtColors[RT_BLUE  ] = RTWin->Blue;
		RtColors[RT_ORANGE] = RTWin->Orange;

		if (tst) {
			itoa (tst->UserType, NumStrings[NUM_TYPE_NDX], 10);
			itoa (tst->UserNumber, NumStrings[NUM_IDNUM_NDX], 10);
			top    = TS_TOP;
			height = TS_HEIGHT;
			RTList = TileTypeList;
		} else {
			itoa (room->UserType, NumStrings[NUM_TYPE_NDX], 10);
			itoa (room->UserNumber, NumStrings[NUM_IDNUM_NDX], 10);
			top    = RTDLG_TOP;
			height = RTDLG_HEIGHT;
			RTList = RoomTypeList;
		}

		RTTop = top;

		if (PushWindowClipValues (RTWin)) {
			sa = SaveArea (RTDLG_LEFT, top, RTDLG_WIDTH, height);
			if (!sa) {
				GEcatf ("\nOOM: FileReq Area");
				goto dRTcleanup;
			}

			if (InitTextLGAD (
					lgad,
					RTDLG_TLST_WIDTH,
					RTDLG_TLST_HEIGHT,
					RTList,
					LGAD_V_ELEVATOR | LGAD_V_REALTIME_SLIDER | LGAD_EXCLUSIVE,
					RTDLG_TLST_ARROW_W,
					20,20,
					1000,
					SelectFunc,
					lgad)) {

				UpdateListMark ();

				RTHitArea[LIST_GAD].Data = phaLGAD(lgad);

				BeforeGraphics ();

				StartEPicture (&rp, &RTEPicture, RTDLG_LEFT, top);

				DrawLGAD (lgad, RTDLG_LEFT + RTDLG_TLST_LEFT, top + RTDLG_TLST_TOP);

				DrawString (RTFont, RTDLG_LEFT + LABEL_LEFT, top + RT_TYPE_TOP + 2, "User Type");
				DrawString (RTFont, RTDLG_LEFT + LABEL_LEFT, top + RT_NUM_TOP + 2, "User Number");

				/*
				** If new draw Width and Height stuff.
				*/
				if (new) {

					StartEPicture (&rp, &NumEPicture, RTDLG_LEFT + RTDLG_NUM_LEFT, top + RT_WIDTH_TOP);
					StartEPicture (&rp, &NumEPicture, RTDLG_LEFT + RTDLG_NUM_LEFT, top + RT_HEIGHT_TOP);

				} else {
				}

				/*
				** If new or not add or remove Width/Height areas.
				*/
				{
					short	i;

					for (i = 0; i < 6; i++) {
						RTHitArea[WIDTH_GADS + i].Width = (new) ? NewGadWidths[i] : 0;
					}
					for (i = 6; i < 9; i++) {
						RTHitArea[WIDTH_GADS + i].Width = (tst) ? NewGadWidths[i] : 0;
					}
				}

				if (tst) {
					/*
					** If Tilesets draw NTI and COMment stuff.
					*/
					strcpy (Name, tst->Filespec);
					SaveGrafx = !(tst->Flags & DONT_SAVEIMAGES);

					if (tst->Comment[0]) {
						strcpy (Com1, tst->Comment[0]);
					} else {
						Com1[0] = '\0';
					}

					if (tst->Comment[1]) {
						strcpy (Com2, tst->Comment[1]);
					} else {
						Com2[0] = '\0';
					}

					StartEPicture (&rp, &TSetEPicture, RTDLG_LEFT, top);
					DrawString (RTFont, RTDLG_LEFT + RTDLG_WIDTH / 2 - StringLen (RTFont, "Tileset Info") / 2, top + 2, "Tileset Info");
					if (new) {
						itoa (TileWidth, NumStrings[NUM_WIDTH_NDX], 10);
						itoa (TileHeight, NumStrings[NUM_HEIGHT_NDX], 10);

						DrawString (RTFont, RTDLG_LEFT + LABEL_LEFT, top + RT_WIDTH_TOP + 2, "Tile Width");
						DrawString (RTFont, RTDLG_LEFT + LABEL_LEFT, top + RT_HEIGHT_TOP + 2, "Tile Height");
					} else {
						/*
						** Print some tset info?
						*/
					}
					UpdateNTI ();
				} else {
					/*
					** If Rooms remove NTI area.
					*/
					if (room->Name) {
						strcpy (Name, room->Name);
					} else {
						Name[0] = '\0';
					}

					DrawString (RTFont, RTDLG_LEFT + RTDLG_WIDTH / 2 - StringLen (RTFont, "Room Info") / 2, top + 2, "Room Info");
					if (new) {
						itoa (RoomWidth, NumStrings[NUM_WIDTH_NDX], 10);
						itoa (RoomHeight, NumStrings[NUM_HEIGHT_NDX], 10);

						DrawString (RTFont, RTDLG_LEFT + LABEL_LEFT, top + RT_WIDTH_TOP + 2, "Tile Width");
						DrawString (RTFont, RTDLG_LEFT + LABEL_LEFT, top + RT_HEIGHT_TOP + 2, "Tile Height");
					} else {
						/*
						** Print some room info
						*/
					}
				}

				/*
				** Set String Gadget heights and tab order.
				*/
				{
					short i;

					for (i = 0; i < NUM_STRING_GADS ; i++) {
						NumString[i].Y    = top + StrHeights[i];
						NumString[i].Next = ESTable[((tst) ? 0 : 2) + ((new) ? 0 : 1)][i];
					}
				}

				UpdateStrings (MainWindow, &NumString[0], (((new) ? 2 : 0) + ((tst) ? 5 : 3)), NULL);

				AfterGraphics ();

				while (!RTdone) {
					short	status;
					short	key;

					ha = CheckHitAreas (RTHitArea, RTDLG_LEFT, top);
					if (ha->ID == OK_ID) {
						RTdone    = TRUE;
						RTsuccess = TRUE;
						break;
					}
					if (ha->ID == CANCEL_ID) {
						RTdone  = TRUE;
						break;
					}
					status = CheckEditKeys (&key);
					if (status) {
						RTdone = TRUE;
						if (status >= 0) {
							RTsuccess = TRUE;
						}
					}
				}
			}

dRTcleanup:
			if (sa)		RestoreArea (sa);
			FlushKeyboardBuffer ();

			PopClipValues ();
		}

		if (RTsuccess) {

			if (tst) {
				tst->Flags &= (~DONT_SAVEIMAGES);
				tst->Flags |= (SaveGrafx) ? 0 : DONT_SAVEIMAGES;
				repstr (&tst->Filespec, Name);
				repstr (&tst->Comment[0], Com1);
				repstr (&tst->Comment[1], Com2);
				tst->UserType   = atoi (NumStrings[NUM_TYPE_NDX]);
				tst->UserNumber = atoi (NumStrings[NUM_IDNUM_NDX]);
				if (new) {
					tst->Width  = atoi (NumStrings[NUM_WIDTH_NDX]);
					tst->Height = atoi (NumStrings[NUM_HEIGHT_NDX]);
				}
			} else {
				repstr (&room->Name, Name);
				room->UserType   = atoi (NumStrings[NUM_TYPE_NDX]);
				room->UserNumber = atoi (NumStrings[NUM_IDNUM_NDX]);
				if (new) {
					room->Width  = atoi (NumStrings[NUM_WIDTH_NDX]);
					room->Height = atoi (NumStrings[NUM_HEIGHT_NDX]);
				}
			}
		}

		return TRUE;
	}
} /* RoomNTileInfoReq */

/*********************************************************************
 *
 * ReadTypeList
 *
 * SYNOPSIS
 *		LOCAL short ReadTypeList (SectionTracker *sec, ListType *list)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL short ReadTypeList (SectionTracker *sec, ListType *list)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ReadTypeList";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
#define MAX_LINE	80
	{
		ConfigLine		*cl;
		RT_Type			*rt;
		char			 line[MAX_LINE + 1];
		char			*s;
		short			 pos;

		while ((cl = GetNextINILine (sec))) {
			s     = NodeName (cl);
			pos   = 0;
			while (*s && *s != '=') {
				s++;
				pos++;
			}
			if (*s == '=' && pos) {
				strncpy (line, NodeName (cl), pos);
				line[pos] = '\0';
				s  = TrimWhiteSpace (line);
				if (strlen (s)) {
					rt = CreateNode (sizeof (RT_Type), s);
					if (!rt) {
						SetGlobalErr (ERR_OUT_OF_MEMORY);
						GEcatf ("\nOOM: Reading Room/Tileset Types");
						return FALSE;
					}
					rt->Type = atoi (NodeName (cl) + pos + 1);
					AddTail (list, rt);
				}

			}
		}

		return TRUE;
	}
} /* ReadTypeList */

/*********************************************************************
 *
 * InitTypeLists
 *
 * SYNOPSIS
 *		short InitTypeLists (IniList *inilist)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short InitTypeLists (IniList *inilist)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "InitTypeLists";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		SectionTracker	 secx;
		SectionTracker	*sec = &secx;

		SectionTracker	 sec2x;
		SectionTracker	*sec2 = &sec2x;

		char			*s;
		char			 work[100];

		InitList (TileTypeList);
		InitList (RoomTypeList);

		sec = FindSection (sec, inilist, "[Type Groups]");
		if (sec) {

			ResetSection (sec);
			s = FindNextINILine (sec, "TileTypes=");
			if (s) {
				TileTypeGroup = dupstr (s);
				strcpy (work + 1, s);
				work[0] = '[';
				strcat (work, "]");

				sec2 = FindSection (sec2, inilist, work);
				if (sec2) {
					if (!ReadTypeList (sec2, TileTypeList)) {
						return FALSE;
					}
				}
			}

			ResetSection (sec);
			s = FindNextINILine (sec, "RoomTypes=");
			if (s) {
				RoomTypeGroup = dupstr (s);
				strcpy (work + 1, s);
				work[0] = '[';
				strcat (work, "]");

				sec2 = FindSection (sec2, inilist, work);
				if (sec2) {
					if (!ReadTypeList (sec2, RoomTypeList)) {
						return FALSE;
					}
				}
			}
		}

		return TRUE;
	}
} /* InitTypeLists */

