/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * EASYDBUF.C
 *
 * PROGRAMMER : Gregg A. Tavares 
 *    VERSION : 00.000
 *    CREATED : 06/17/90 
 *   MODIFIED : 07/27/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w-----
 *	 E C H I D N A
 *	--------------
 *
 * DESCRIPTION
 *		Routines to do fast (system friendly) double buffering on the amiga
 *
 * HISTORY
 *
*/
#include <echidna/platform.h>
#include "switches.h"

#include <echidna/etypes.h>
#include <exec/tasks.h>
#include <exec/memory.h>
#include <exec/interrupts.h>
#include <hardware/intbits.h>
#include <graphics/gfxbase.h>
#include <intuition/intuition.h>
#include <intuition/intuitionbase.h>
#if LATTICE
#include <proto/exec.h>
#include <proto/graphics.h>
#include <proto/intuition.h>
#include <dos.h>
#include <stdio.h>
#else
#include <functions.h>
#endif
#include <echidna/emacros.h>
#include "sizedefs.h"
#include "easydbuf.h"

#ifdef TEST
#define LOCAL
#else
#define LOCAL static
#endif

#undef DEBUG

#if LATTICE
extern	void	__far	LCVB_DBufInterrupt (void);
#else
#ifndef __chip
#define __chip
#endif
#endif

/**************************** C O N S T A N T S ***************************/

/* Note: Changing these will not work! */
#define DEPTH	5
#define WIDTH	320
#define HEIGHT	200


/******************************** T Y P E S *******************************/

typedef void (*VoidFunc)();

typedef struct cpr_wait {
	UBYTE	VerticalStart;
	UBYTE	HorizontalStart;	/* Bit  0 must be 1!	*/
	UBYTE	VerticalCompare;	/* Bit 15 should be 1	*/
	UBYTE	HorizontalCompare;	/* Bit  0 must be 0!	*/
} cpr_WaitType;

typedef struct cpr_color {
	UWORD	ColorRegister;		/* Bit  0 must be 0!	*/
	UWORD	ColorRGBValue;
} cpr_ColorType;

typedef struct cpr_planeptr {
	UWORD	HiPlaneRegister;
	UWORD	HiPtr;
	UWORD	LoPlaneRegister;
	UWORD	LoPtr;
} cpr_PlanePtrType;

typedef struct cpr_customcopperlist {
	cpr_WaitType	BeginWait;
	cpr_ColorType	Colors[32];
	UWORD		DIWSTRT;
	UBYTE		DisplayWindowVerticalStart;	/* 0x05 */
	UBYTE		DisplayWindowHorizontalStart;	/* 0x81 */
	UWORD		BPLCON0;
	UWORD		colorburston;	/* 0x0200 */
	UWORD		BPLCON2;
	UWORD		priorities;	/* 0x0024 */
	UWORD		DIWSTOP;
	UBYTE		DisplayWindowVerticalStop;	/* 0x40 */
	UBYTE		DisplayWindowHorizontalStop;	/* 0xc1 */
	UWORD		DDFSTRT;
	UBYTE		ddfstrtpad;
	UBYTE		DataFetchStart;	/* 0x0038 */	/* bits 0,1 must be 0 */
	UWORD		DDFSTOP;
	UBYTE		ddfstoppad;
	UBYTE		DataFetchStop;	/* 0x00d0 */	/* bits 0,1 must be 0 */
	UWORD		BPLCON1;
	UBYTE		bplcon1pad;
	UBYTE		ScrollReg;	/* bits 0-3 = playfield 1 scroll reg */
					/* bits 4-7 = playfield 2 scroll reg */
	UWORD		BPL1MOD;
	UWORD		Playfield1Modulo;	/* 0x0000 */
	UWORD		BPL2MOD;
	UWORD		Playfield2Modulo;	/* 0x0000 */
	cpr_PlanePtrType PlanePtrs[5];
	cpr_WaitType	TopWait;
	UWORD		BPLCON0a;
	UWORD		PlaneControl;		/* 0x5200 */
	cpr_WaitType	BottomWait;
	UWORD		BPLCON0b;
	UWORD		PlaneControl2;		/* 0x0200 */
	UWORD		INTREQ;
	UWORD		CauseInterrupt;		/* 0x8008 */
	cpr_WaitType	ForeverWait;
} CustomCopperListType;

/****************************** G L O B A L S *****************************/

/* LATTICE BLINK says duplicate symbol _VBTimer */
/* but we no longer need it 'cause OpenGraphics() installs */
/* VBTimer as a 'timer' (60 Hz) */
/* except when  we want to use the vblank after all. */

#if USE_AMIGA_VBLANK
extern 	ULONG	VBTimer;
#endif

/******************************* M A C R O S ******************************/


/***************************** R O U T I N E S ****************************/


static struct Screen			*DBufScreen;
extern struct GfxBase			*GfxBase;
extern struct IntuitionBase		*IntuitionBase;


/*
 ************************************************
 ** This data MUST BE IN CHIP MEMORY!!!
 **
 ** Note: This copperlist is for a 5 bitplane,
 **       32 color, non-scrolling screen!
 ************************************************
*/

LOCAL UWORD __chip MyCopperList[] = {
  0x2b01, 0xfffe,
  0x0180, 0x0000,
  0x0182, 0x0000,
  0x0184, 0x0000,
  0x0186, 0x0000,
  0x0188, 0x0000,
  0x018a, 0x0000,
  0x018c, 0x0000,
  0x018e, 0x0000,
  0x0190, 0x0000,
  0x0192, 0x0000,
  0x0194, 0x0000,
  0x0196, 0x0000,
  0x0198, 0x0000,
  0x019a, 0x0000,
  0x019c, 0x0000,
  0x019e, 0x0000,
  0x01a0, 0x0000,
  0x01a2, 0x0000,
  0x01a4, 0x0000,
  0x01a6, 0x0000,
  0x01a8, 0x0000,
  0x01aa, 0x0000,
  0x01ac, 0x0000,
  0x01ae, 0x0000,
  0x01b0, 0x0000,
  0x01b2, 0x0000,
  0x01b4, 0x0000,
  0x01b6, 0x0000,
  0x01b8, 0x0000,
  0x01ba, 0x0000,
  0x01bc, 0x0000,
  0x01be, 0x0000,
  0x008e, 0x0581,
  0x0100, 0x0200,
  0x0104, 0x0024,
  0x0090, 0x40c1,
  0x0092, 0x0038,
  0x0094, 0x00d0,
  0x0102, 0x0000,
  0x0108, 0x0000,
  0x010a, 0x0000,
  0x00e0, 0x0000,
  0x00e2, 0x0000,
  0x00e4, 0x0000,
  0x00e6, 0x0000,
  0x00e8, 0x0000,
  0x00ea, 0x0000,
  0x00ec, 0x0000,
  0x00ee, 0x0000,
  0x00f0, 0x0000,
  0x00f2, 0x0000,
  0x2d01, 0xfffe,
  0x0100, 0x5200,
  0xf501, 0xfffe,
  0x0100, 0x0200,
  0x009c, 0x8008,
  0xffff, 0xfffe,
};

LOCAL struct TextAttr NormFont = {
	(UBYTE *)"topaz.font",	/* Font Name		*/
	8,					/* Font Height		*/
	FS_NORMAL,			/* Style		*/
	FPF_ROMFONT };		/* Preferences		*/

LOCAL struct NewScreen NewScreen = {
	0, 0,			/* Left, Top Edge		*/
	WIDTH, HEIGHT,	/* Width, Height		*/
	DEPTH,			/* Depth			*/
	0, 1,			/* DetailPen, BlockPen		*/
	NULL,			/* Display Modes		*/
	CUSTOMSCREEN,	/* Screen Type		*/
	&NormFont,		/* Font				*/
	NULL,			/* Title			*/
	NULL,			/* Screen Gadgets		*/
	NULL,			/* CustomBitMap			*/
};

LOCAL 	struct	RastPort	FrontRpX;
LOCAL 	struct	RastPort	BackRpX;
		struct	RastPort	*FrontRp = &FrontRpX;
		struct	RastPort	*BackRp  = &BackRpX;
LOCAL 	struct	Task		*OurTask;
LOCAL 	struct	Interrupt	IntServ[2] = { NULL, };
LOCAL	struct	cprlist		DBufCprList;
LOCAL	struct	View		DBufView;
LOCAL 	ULONG				ReadyToRenderSignal = -1;
LOCAL 	ULONG				ReadyToRenderSignalMask;
LOCAL	int					Signaled = FALSE;
LOCAL 	DBufScreenTrkType	DBufStuff = { NULL, };
LOCAL 	CustomCopperListType	*CList = NULL;


VOID SetRGB (
	UWORD	 reg,
	UWORD	 color
)
{
	if (reg == 0) {
		GfxBase->copinit->diagstrt[1] = color;
	}

	CList->Colors[reg].ColorRGBValue     = color;

#ifdef DEBUG
	if (DBufScreen) {
		SetRGB4 (&DBufScreen->ViewPort, reg,
			  color >> 8,
			 (color >> 4) & 0x000F,
			  color & 0x000F);
	}
#endif
}

VOID SetRGBBoth (
	UWORD	reg,
	UWORD	color
)
{
	SetRGB (reg, color);
	SetRGB4 (&DBufScreen->ViewPort, (long)reg,
		 (long)(color >> 8),
		 (long)((color >> 4) & 0x000F),
		 (long)(color & 0x000F));
}

VOID LoadRGB (
	UWORD	 reg,
	UWORD	*colors,
	UWORD	 count
)
{
	for (count; count > 0; count--) {
		SetRGB (reg++, *colors++);
	}
}

VOID LoadRGBBoth (
	UWORD	 reg,
	UWORD	*colors,
	UWORD	 count
)
{
	LoadRGB (reg, colors, count);
	for (; count > 0; count--, colors++) {
		SetRGB4 (&DBufScreen->ViewPort, (long)reg++,
			 (long)(*colors >> 8),
			 (long)((*colors >> 4) & 0x000F),
			 (long)(*colors & 0x000F));
	}
}

	
void __regargs AmigaPaletteToHardware (
	UWORD	*colors,
	UWORD	count
)
{
	WORD	index;

	for (index = 0; index < count ; index += 1) {
		if (! index) {
			GfxBase->copinit->diagstrt[1]	= *colors;
		}
		CList->Colors[index].ColorRGBValue	= *colors;
		colors += 1;
	} 
	return;
}


#if LATTICE

/* all function calls from inside this function must be cast as __far */

/* void __far __saveds MyVBlank (void) */
void __far __saveds DBufInterrupt (void)
{
	LOCAL	struct BitMap	*OldBitMap;
	register UWORD					*planes;
	register UWORD					*planeptrs;
	register UWORD					 ndx;

	if (OldBitMap != DBufStuff.FrontBitMapPtr) {
		planes    = (UWORD *)&DBufStuff.FrontBitMapPtr->Planes[0];
		planeptrs = (UWORD *)&CList->PlanePtrs[0].HiPtr;
		for (ndx = 0; ndx < DEPTH; ndx++) {
			*planeptrs = *planes++; planeptrs+=2;
			*planeptrs = *planes++; planeptrs+=2;
		}
		OldBitMap = DBufStuff.FrontBitMapPtr;
		Signal (OurTask, ReadyToRenderSignalMask);
	}

	CList->BeginWait.VerticalStart  = DBufView.DyOffset - 1;
	CList->TopWait.VerticalStart    = DBufView.DyOffset;
	CList->BottomWait.VerticalStart	= min(0xFE, DBufView.DyOffset + HEIGHT);

	CList->DisplayWindowHorizontalStart = DBufView.DxOffset;

	CList->DisplayWindowHorizontalStop  = DBufView.DxOffset + WIDTH + 1;
/*									  ^ */
/*									  | */
/*						        Why is this needed? */

	ndx = (DBufView.DxOffset >> 1) - 8;
	CList->DataFetchStart = ndx;
	CList->DataFetchStop  = ndx + 8 * (((WIDTH + 15) >> 4) - 1);

/*      What should this be and why??  */
/*                                  |  */
/*                                  V  */
	ndx = ((DBufView.DxOffset - 0) & 0x000F);
	CList->ScrollReg = ndx | (ndx << 4);

#if USE_AMIGA_VBLANK
	VBTimer	+= 1;
#endif

	return;
}

#else

static VOID *DBufInterrupt (void)
{
	LOCAL	struct BitMap	*OldBitMap;
	register UWORD					*planes;
	register UWORD					*planeptrs;
	register UWORD					 ndx;
/*
#asm
	movem.l	d1-d7/a0-a6,-(sp)
#endasm
*/

	if (OldBitMap != DBufStuff.FrontBitMapPtr) {
		planes    = (UWORD *)&DBufStuff.FrontBitMapPtr->Planes[0];
		planeptrs = (UWORD *)&CList->PlanePtrs[0].HiPtr;
		for (ndx = 0; ndx < DEPTH; ndx++) {
			*planeptrs = *planes++; planeptrs+=2;
			*planeptrs = *planes++; planeptrs+=2;
		}
		OldBitMap = DBufStuff.FrontBitMapPtr;
		Signal (OurTask, ReadyToRenderSignalMask);
	}

	CList->BeginWait.VerticalStart  = DBufView.DyOffset - 1;
	CList->TopWait.VerticalStart    = DBufView.DyOffset;
	CList->BottomWait.VerticalStart	= min(0xFE, DBufView.DyOffset + HEIGHT);

	CList->DisplayWindowHorizontalStart = DBufView.DxOffset;

	CList->DisplayWindowHorizontalStop  = DBufView.DxOffset + WIDTH + 1;
/*									  ^ */
/*									  | */
/*						        Why is this needed? */

	ndx = (DBufView.DxOffset >> 1) - 8;
	CList->DataFetchStart = ndx;
	CList->DataFetchStop  = ndx + 8 * (((WIDTH + 15) >> 4) - 1);

/*      What should this be and why??  */
/*                                  |  */
/*                                  V  */
	ndx = ((DBufView.DxOffset - 0) & 0x000F);
	CList->ScrollReg = ndx | (ndx << 4);

#if USE_AMIGA_VBLANK
	VBTimer	+= 1;
#endif

/*
#asm
	movem.l	(sp)+,d1-d7/a0-a6
#endasm
*/
	return NULL;
}

#endif


VOID FlipDBufScreen (void)
{
	DBufScreenTrkType	*ds;
	struct	BitMap		*tempbm;
#if 0
	static	struct	Screen	*oldfrontscreen;
#endif

	ds = (DBufScreenTrkType *)DBufScreen->UserData;

	SetSignal (0L, ReadyToRenderSignalMask);
	Signaled = TRUE;

#if 0
	oldfrontscreen = IntuitionBase->FirstScreen;
	if (oldfrontscreen != IntuitionBase->FirstScreen) {
		if (IntuitionBase->FirstScreen == DBufScreen) {
			LoadView (&DBufView);
		}
	}
#endif
#if 1
	if (IntuitionBase->FirstScreen == DBufScreen) {
		if (GfxBase->ActiView != &DBufView) {
			LoadView (&DBufView);
		}
	}
#endif

	tempbm             = ds->FrontBitMapPtr;
	ds->FrontBitMapPtr = ds->BackBitMapPtr;
	ds->BackBitMapPtr  = tempbm;

	FrontRp->BitMap = ds->FrontBitMapPtr;
	BackRp->BitMap  = ds->BackBitMapPtr;

	DBufScreen->ViewPort.RasInfo->BitMap = ds->FrontBitMapPtr;
}

VOID WaitDBufScreen (void)
{
	if (Signaled) {
		Signaled = FALSE;
		Wait (ReadyToRenderSignalMask);
	}
}

VOID RectifyDBufScreen (void)
{
	do {
		WaitDBufScreen ();
		if (FrontRp->BitMap == &DBufScreen->BitMap) {
			break;
		}
		FlipDBufScreen ();
	} while (TRUE);
	ScreenToFront (DBufScreen);
}

VOID DeleteDBufScreen (
	struct	Screen	*scr
)
{
	int	i;

	if (!scr) return;

	for (i = 0; i < 2; i++) {
		if (IntServ[i].is_Data) {
			RemIntServer((long)(i ? INTB_VERTB : INTB_COPER),
				&IntServ[i]);
		}
		IntServ[i].is_Data = NULL;
	}

	if (ReadyToRenderSignal != -1) {
		FreeSignal (ReadyToRenderSignal);
		ReadyToRenderSignal = -1;
	}

	for (i = 0; i < DEPTH; i++) {
		if (DBufStuff.BackBitMap.Planes[i]) {
			FreeMem(DBufStuff.BackBitMap.Planes[i], 
				    (long)RASSIZE(WIDTH, HEIGHT));
		}
	}

	CloseScreen (scr);

	if (CList) {
		FreeMem (CList, sizeof (MyCopperList));
		CList = NULL;
	}
}


struct Screen *CreateDBufScreen (void)
{

static UWORD	BlackPalette [32] = {
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
};

	struct	Screen	*ns;
	struct	Screen	 ws;
	int		i;

	if (!(CList = AllocMem (sizeof (MyCopperList), MEMF_CHIP))) {
		return NULL;
	}

	CopyMem ((char *)MyCopperList, (char *)CList, sizeof (MyCopperList));

	if (!(ns = OpenScreen (&NewScreen))) {
		FreeMem (CList, sizeof (MyCopperList));
		return NULL;
	}
	else {

		/* we want to be sure of a black screen */

		LoadRGB4 (&ns->ViewPort, (void *) BlackPalette, 32);

	}

	DBufScreen = ns;

	ns->UserData = (UBYTE *)&DBufStuff;

	DBufStuff.FrontBitMapPtr = &ns->BitMap;
	InitBitMap (&DBufStuff.BackBitMap, DEPTH, WIDTH, HEIGHT);
	DBufStuff.BackBitMapPtr = &DBufStuff.BackBitMap;

	for (i = 0; i < DEPTH; i++) {
		if (!(DBufStuff.BackBitMap.Planes[i] = (PLANEPTR)AllocMem (RASSIZE(WIDTH, HEIGHT), MEMF_CHIP | MEMF_CLEAR))) {
			DeleteDBufScreen (ns);
			return NULL;
		}
	}

	InitRastPort (FrontRp);
	InitRastPort (BackRp);

	OurTask = FindTask (NULL);

	if ((ReadyToRenderSignal = AllocSignal (-1L)) == -1) {
		return NULL;
	}

	ReadyToRenderSignalMask = (1L << ReadyToRenderSignal);

	for (i = 0; i < 2; i++) {
#if LATTICE
		IntServ[i].is_Data         = (void *) getreg ((int) REG_A4);
		if (i) {	/** VERTB? Use vblank shell **/
			IntServ[i].is_Code         = (VoidFunc) LCVB_DBufInterrupt;
		}
		else {		/** COPER? Don't use vblank shell **/
			IntServ[i].is_Code         = (VoidFunc) DBufInterrupt;
		}
#else
		IntServ[i].is_Data		   = (APTR)TRUE;
		IntServ[i].is_Code         = (VoidFunc)&DBufInterrupt;
#endif
		IntServ[i].is_Node.ln_Type = NT_INTERRUPT;
		IntServ[i].is_Node.ln_Name = "DBuf Interrupt";
		AddIntServer ((long)(i ? INTB_VERTB : INTB_COPER),
				&IntServ[i]);
	}

	DBufCprList.start    = (UWORD *)CList;
	DBufCprList.MaxCount = sizeof (*CList) >> 2;

	if (GetScreenData ((char *)&ws, sizeof (ws), WBENCHSCREEN, NULL)) {
		LoadRGB (0, (UWORD *)ws.ViewPort.ColorMap->ColorTable, min((1 << DEPTH), 32));
	}

	InitView (&DBufView);
	DBufView.LOFCprList = &DBufCprList;
	LoadView (&DBufView);

	FlipDBufScreen ();
/*	RectifyDBufScreen (); */

	return ns;
}

VOID PositionDBufScreen (
	UWORD	 x,
	UWORD	 y
)
{
	DBufView.DxOffset = min(130, max(x, 0x62));
	DBufView.DyOffset = max(y, 0x19);
}

void RestoreView (void)
{
	LoadView (&DBufView);
}

#ifdef TEST
PrintCListStuff()
{
	printf ("CList->DisplayWindowHorizontalStart = 0x%02x\n", 
		CList->DisplayWindowHorizontalStart);
	printf ("CList->DisplayWindowHorizontalStop = 0x%02x\n", 
		CList->DisplayWindowHorizontalStop);
	printf ("CList->DisplayWindowVerticalStart = 0x%02x\n", 
		CList->DisplayWindowVerticalStart);
	printf ("CList->DisplayWindowVerticalStop = 0x%02x\n", 
		CList->DisplayWindowVerticalStop);
	printf ("CList->DataFetchStart = 0x%04x\n",
		CList->DataFetchStart);
	printf ("CList->DataFetchStop = 0x%04x\n",
		CList->DataFetchStop);
	printf ("CList->BeginWait.VerticalStart = 0x%02x\n",
		CList->BeginWait.VerticalStart);
	printf ("CList->TopWait.VerticalStart = 0x%02x\n",
		CList->TopWait.VerticalStart);
	printf ("CList->BottomWait.VerticalStart = 0x%02x\n",
		CList->BottomWait.VerticalStart);
}
#endif

