;* 
;* The contents of this file are subject to the Mozilla Public 
;* License Version 1.1 (the "License"); you may not use this file 
;* except in compliance with the License. You may obtain a copy of 
;* the License at http://www.mozilla.org/MPL/ 
;* 
;* Software distributed under the License is distributed on an "AS 
;* IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
;* implied. See the License for the specific language governing 
;* rights and limitations under the License. 
;* 
;* The Original Code is tUME: the Universal Map Editor, released 
;* June 18, 2000. 
;* 
;* The Initial Developer of the Original Code is Echidna. 
;* Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
;* All Rights Reserved. 
;* 
;* Contributor(s): 
;* 

;**
;** EGFX.asm
;**
;** PROGRAMMER : Gregg A. Tavares
;**    VERSION : 00.000
;**    CREATED : 12/15/89
;**   MODIFIED : 10/02/92
;**       TABS : 09 15 40 44
;**
;**    	     \|///-_
;**	     \oo///_
;**	-----w/-w------
;**	 E C H I D N A
;**	---------------
;**
;** DESCRIPTION
;**    Various Graphics Routines for EGA mode.
;**    
;** HISTORY
;**
;**

	IDEAL

;********************************* EXTERNS *********************************
	EXTRN _EGA_OpenGraphics:PROC
	EXTRN _EGA_CloseGraphics:PROC
	EXTRN _EGA_ConvertShape:PROC
	EXTRN _EGA_FreeShape:PROC
	EXTRN _EGA_DrawShapeNoTrans:PROC
	EXTRN _EGA_DrawRect:PROC
	EXTRN _EGA_GetGShapeLoadBuf:PROC
	EXTRN _EGA_UsePaletteN:PROC
	EXTRN _EGA_FadeToPaletteN:PROC
	EXTRN _EGA_UseColorCycleSet:PROC   
	EXTRN _EGA_FadeToColorCycleSet:PROC
	EXTRN _EGA_UsePaintBoxN:PROC
	EXTRN _EGA_FadeToPaintBoxN:PROC
	EXTRN _EGA_OpenPainter:PROC
	EXTRN _EGA_ClosePainter:PROC       

;********************************* SEGMENT *********************************
	INCLUDE "GFXMDL.inc"
	WSEG	EGFX_TEXT

;**************************** GLOBAL VARIABLES *****************************
	PUBLIC _EGARoutineTable
	PUBLIC _EGADataTable

;***************************** PUBLIC ROUTINES *****************************
	PUBLIC _EGA_ASMOpenGraphics
	PUBLIC _EGA_ASMCloseGraphics
	PUBLIC _EGA_WaitRetrace
	PUBLIC _EGA_SetColorReg
	PUBLIC _EGA_ASMSetPalette

;******************************** VARIABLES ********************************

EClipLeft	 dw 0
EClipTop	 dw 0
EClipWidth	 dw 320
EClipHeight	 dw 200
EClipRight	 dw 319
EClipBottom	 dw 199
EGrSeg		 DW ?
EPenColor	 DB ?
EPaperColor	 DB ?
EWorkBuffer	 DW 0
EBackBuffer	 DW 0
EDisplayBuffer DW 0A000h
_EGAEvenPatTbl DB 000h ; 0
		DB 011h ; 1
		DB 022h ; 2
		DB 033h ; 3
		DB 044h ; 4
		DB 055h ; 5
		DB 066h ; 6
		DB 077h ; 7
		DB 088h ; 8
		DB 099h ; 9
		DB 0AAh ;10
		DB 0BBh ;11
		DB 0CCh ;12
		DB 0DDh ;13
		DB 0EEh ;14
		DB 0FFh ;15
		DB 240 DUP (?)
_EGAOddPatTbl	DB 000h ; 0
		DB 011h ; 1
		DB 022h ; 2
		DB 033h ; 3
		DB 044h ; 4
		DB 055h ; 5
		DB 066h ; 6
		DB 077h ; 7
		DB 088h ; 8
		DB 099h ; 9
		DB 0AAh ;10
		DB 0BBh ;11
		DB 0CCh ;12
		DB 0DDh ;13
		DB 0EEh ;14
		DB 0FFh ;15
		DB 240 DUP (?)

	MACRO Rptr Routine

	DW	OFFSET Routine, SEG Routine

	ENDM

_EGADataTable:
	RPtr	EClipLeft
	RPtr	EClipTop 
	RPtr	EClipWidth
	RPtr	EClipHeight
	RPtr	EClipRight 
	RPtr	EClipBottom
	RPtr	EPenColor
	RPtr	EPaperColor
	Rptr	EGrSeg
	RPtr	EWorkBuffer
	RPtr	EBackBuffer
	RPtr	EDisplayBuffer
	RPtr	_EGAEvenPatTbl
	RPtr	_EGAOddPatTbl

_EGARoutineTable:
	Rptr	_EGA_OpenGraphics
	Rptr	_EGA_CloseGraphics
	Rptr	_EGA_DrawPixel
	Rptr	_EGA_DrawRect
	Rptr	_EGA_ClearScreen
	Rptr	_EGA_DrawShape
	Rptr	_EGA_Restore
	Rptr	_EGA_UnRestore
	Rptr	_EGA_ShowRect
	Rptr	_EGA_ConvertShape
	Rptr	_EGA_FreeShape
	Rptr	_EGA_DrawShapeNoTrans
	Rptr	_EGA_GetGShapeLoadBuf
	Rptr	_EGA_UsePaletteN
	Rptr	_EGA_FadeToPaletteN
	Rptr	_EGA_UseColorCycleSet   
	Rptr	_EGA_FadeToColorCycleSet
	Rptr	_EGA_UsePaintBoxN
	Rptr	_EGA_FadeToPaintBoxN
	Rptr	_EGA_OpenPainter
	Rptr	_EGA_ClosePainter       
	Rptr	0

	LABEL LineSt  WORD              ; Lookup table for line start adresses
x       =     0
        REPT 200
        dw  x*40
x       =   x + 1
        ENDM


PMasks	DB	080h
	DB	040h
	DB	020h
	DB	010h
	DB	008h
	DB	004h
	DB	002h
	DB	001h

;=====================================================================
;**
;** _EGA_ASMCloseGraphics
;**
;** PURPOSE
;**	Free EGA resources and return to text mode.
;**
;** INPUT
;**	NONE
;**
;** REGISTERS USED
;**	AX,BX,ES
;**
;** EFFECTS
;**	OBuffer and BBuffer 64000 bytes * 2 freed.
;**
;** RETURN VALUE
;**	NONE
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	
;**
	MPROC _EGA_ASMCloseGraphics
	MPROC EGA_ASMCloseGraphics

	mov	dx,03CEh
	mov	ax,0000h	;0
	out	dx,ax
	inc	al	;1
	out	dx,ax
	inc	al	;2
	out	dx,ax
	inc	al	;3
	out	dx,ax
	inc	al	;4
	out	dx,ax
	inc	al	;5
	out	dx,ax
	inc	al	;6
	out	dx,ax
	mov	ax,0F07h	;7
	out	dx,ax
	mov	ax,0FF08h;	;8
	out	dx,ax

	mov	ax,03h	; Return to Text mode
	int	10h

@@no_obuf:
	mov	ax,0	; Return 0 for OpenGraphics Failure.
	ret
	ENDP
	ENDP

;=====================================================================
;**
;** EGA_ASMOpenGraphics
;** _EGA_ASMOpenGraphics
;**
;** PURPOSE
;**	Initialize EGA Graphics Routines.
;**
;** INPUT
;**	NONE
;**
;** REGISTERS USED
;**	AX, BX, ES
;**
;** EFFECTS
;**	Allocates 2 64000 byte buffers and enters EGA mode
;**
;** RETURN VALUE
;**	AX = 0  failed
;**	   = !0 success
;**
;** LIMITS/BUGS
;**		
;**
;** HISTORY
;**		
;**

	MPROC _EGA_ASMOpenGraphics
	MPROC EGA_ASMOpenGraphics

	mov	ax,0Dh	; Start EGA mode.
	int	10h

	ret
	ENDP
	ENDP


;=====================================================================
;**
;** EGA_DrawPixel
;**
;** PURPOSE
;**	Draw one Pixel on the current work page.
;**
;** INPUT
;**	AX       = xpos
;**	BX       = ypos
;**	PenColor = color
;**
;** REGISTERS USED
;**	AX, BX, CX, DX, DI, ES
;**
;** EFFECTS
;**	
;**
;** RETURN VALUE
;**	NONE
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	
;**
	MPROC EGA_DrawPixel

	mov	cl,bl	; Save Ypos
	mov	ch,al	; Save Xpos

	shl	bx,1
	mov	di,[CS:LineSt+BX]	; Get Starting address of y row
	mov	bx,ax
	shr	bx,1
	shr	bx,1
	shr	bx,1
	add	di,bx	; add x column.
	and	al,07h	; get pixel num (0-7);
	mov	bl,al
	xor	bh,bh
	mov	ah,[CS:PMasks+bx]

	mov	dx,03CEh	; Set EGA BitMask
	mov	al,08h
	out	dx,ax

	mov	ax,0005h	; Select EGA RMD 0/WMD 0
	out	dx,ax

	mov	ax,0F01h	; Set Enable Set/Reset
	out	dx,ax

	mov	dx,03C4h	; write to all planes
	mov	ax,0F02h
	out	dx,ax

	mov	dx,03CEh	; Set DX back to standard EGA regs.

	mov	bx,[CS:EGrSeg]	; Get Segment of current page.
	mov	es,bx

	xor	bh,bh
	mov	bl,[CS:EPenColor]	; Get Pen Color.
	test	cl,01h	; odd or even line?
	jnz	@@oddline
	mov	ah,[CS:_EGAEvenPatTbl+bx]
	jmp	short @@gotit
@@oddline:
	mov	ah,[CS:_EGAOddPatTbl+bx]
@@gotit:

	test	ch,01h	; odd of even pixel?
	jnz	@@oddpixel
	mov	cl,4
	shr	ah,cl
	jmp	short @@gotcolor

@@oddpixel:
	and	ah,0Fh

@@gotcolor:
	mov	al,00h	; Set Set/Reset
	out	dx,ax

	or	[es:di],al

	ret

	ENDP


;=====================================================================
;**
;** _EGA_DrawPixel
;**
;** PURPOSE
;**	C Glue to EGA_DrawPixel
;**
;** INPUT
;**	C parms: EGA_DrawPixel (int x, int y)
;**
;** REGISTER USED
;**	
;**
;** EFFECTS
;**	
;**
;** RETURN VALUE
;**	NONE
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	
;**

	MPROC _EGA_DrawPixel
	ARG  XPOS:WORD, YPOS:WORD

	push	bp
	mov	bp,sp

	push	di

	mov	bx,[YPOS]
	mov	ax,[XPOS]
	call	EGA_DrawPixel

	pop	di

	pop	bp
	ret

	ENDP

;=====================================================================
;**
;** _EGA_WaitRetrace
;**
;** PURPOSE
;**	
;**
;** INPUT
;**	
;**
;** REGISTER USED
;**	
;**
;** EFFECTS
;**	
;**
;** RETURN VALUE
;**	
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	
;**

	MPROC	_EGA_WaitRetrace 	; INPUT     : NONE
			; USED REGS : NONE
	mov   dx, 3dah	;
again1:			;
	in    al, dx	;
	test  al, 00001000b	;
	jnz   again1	;
again:			;
	in    al, dx	;
	test  al, 00001000b	;
	jz    again	;
			;
	ret		;
	ENDP

;=====================================================================
;**
;** _EGA_SetColorReg (short regnum, UBYTE color)
;**
;** PURPOSE
;**	
;**
;** INPUT
;**	
;**
;** REGISTER USED
;**	
;**
;** EFFECTS
;**	
;**
;** RETURN VALUE
;**	
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	
;**

ClrCnvTbl	DB  0h, 1h, 2h, 3h, 4h, 5h, 6h, 7h
		DB 10h,11h,12h,13h,14h,15h,16h,17h

	MPROC _EGA_SetColorReg
	ARG  REGNUM:WORD, COLOR:BYTE

	push	bp
	mov	bp,sp

	xor	bh,bh
	mov	bl,[COLOR]
	mov	bh,[CS:ClrCnvTbl+bx]
	mov	ax,[REGNUM]
	mov	bl,al
	mov	ax,1000h
	int	10h

	if 0
	mov	ax,40h
	mov	es,ax	; ES := video BIOS data segment
	mov	dx,[es:63h]	; DX := CRTC address reg (3x4h)
	add	dl,6	; DX := Status Reg (3xAh)
	push	dx	; preserve this value
	cli
	in	al,dx	; reset Attribute Controller Address
			;  flip-flop
	mov	dl,0C0h	; DX := 3C0h
	mov	ax,[REGNUM]
	out	dx,al	; update one palette register
	xor	bh,bh
	mov	bl,[COLOR]
	mov	al,[CS:ClrCnvTbl+bx]
	out	dx,al
	pop	dx	; DX := Status register port
	in	al,dx	; reset the flip-flop
	mov	dl,0C0h
	mov	al,20h
	out	dx,al	; Set bit 5 of
			; Attribute Controller address reg
	sti
	endif

	pop	bp
	ret

	ENDP


;=====================================================================
;**
;** _EGA_ASMSetPaletteg (UBYTE *color)
;**
;** PURPOSE
;**	
;**
;** INPUT
;**	
;**
;** REGISTER USED
;**	
;**
;** EFFECTS
;**	
;**
;** RETURN VALUE
;**	
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	
;**
CTemp	DB	16 DUP (?)
	DB	0

	MPROC _EGA_ASMSetPalette
	ARG	SpOff:WORD,SpSeg:WORD

	push	bp
	mov	bp,sp

	push	di
	push	si
	push	ds

	mov	ds,[SpSeg]
	mov	si,[SpOff]

	mov	ax,cs
	mov	es,ax
	mov	di,OFFSET CTemp
	mov	dx,di
	
	xor	bh,bh
	mov	cx,10h
	cld
@@covloop:
	lodsb
	mov	bl,al
	mov	al,[CS:ClrCnvTbl+bx]
	stosb
	loop @@covloop

	mov	ax,1002h
	int	10h

	pop	ds
	pop	si
	pop	di
	pop	bp
	ret

	ENDP


;=====================================================================
;**
;** ClipBox
;**
;** PURPOSE
;**	Clip x, y, width, height to Clipping parameters:
;**		EClipLeft, EClipRight, EClipTop, EClipBottom.
;**
;** INPUT
;**	AX = x
;**	BX = y
;**	CX = width
;**	DX = height
;**
;** REGISTER USED
;**	SI, DI
;**
;** EFFECTS
;**	
;**
;** RETURN VALUE
;**	AX,BX,CX,DX = new clipped box
;**	carry clear = box was entirely clipped
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	
;**

	MPROC ClipBox NEAR

	;** Quit early rect is completely outside clip window.
	cmp	ax,[CS:EClipRight]
	jg	@@no_rect	; starts past right edge.
	cmp	bx,[CS:EClipBottom]
	jg	@@no_rect	; starts past bottom edge
	mov	si,ax	; get 1 past right side
	add	si,cx
	cmp	si,[CS:EClipLeft]
	jle	@@no_rect	; ends before left edge.
	mov	di,bx	; get 1 below bottom side 
	add	di,dx
	cmp	di,[CS:EClipTop]
	jle	@@no_rect	; ends before top edge.
	
	push	si

	;* Clip sides that overlap clip window 
	mov	si, ax
	sub	si,[CS:EClipLeft]	; check left edge.
	jge	@@left_ok
	add	cx,si	; clip width.
	sub ax,si	; clip left.

@@left_ok:
	mov	si,bx
	sub	si,[CS:EClipTop]	; check top edge.
	jge	@@top_ok
	add	dx,si	; clip height.
	sub	bx,si	; clip top.

@@top_ok:
	pop	si
	dec	si
	sub	si,[CS:EClipRight]	; check right edge.
	jle	@@right_ok
	sub	cx,si	; clip width.	

@@right_ok:
	dec	di
	sub	di,[CS:EClipBottom]	; check bottom edge.
	jle	@@bottom_ok
	sub	dx,di	; clip height.	
@@bottom_ok:
	
	stc
	ret

@@no_rect:
	clc
	ret

	ENDP


;******************************** INCLUDES *********************************
	INCLUDE "ESHAPE.asm"
	INCLUDE "ERESTORE.asm"
	INCLUDE "ERECT.asm"
	INCLUDE "ECLEAR.asm"
	INCLUDE "EWRTCREG.asm"

	ENDS
	END

