;* 
;* The contents of this file are subject to the Mozilla Public 
;* License Version 1.1 (the "License"); you may not use this file 
;* except in compliance with the License. You may obtain a copy of 
;* the License at http://www.mozilla.org/MPL/ 
;* 
;* Software distributed under the License is distributed on an "AS 
;* IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
;* implied. See the License for the specific language governing 
;* rights and limitations under the License. 
;* 
;* The Original Code is tUME: the Universal Map Editor, released 
;* June 18, 2000. 
;* 
;* The Initial Developer of the Original Code is Echidna. 
;* Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
;* All Rights Reserved. 
;* 
;* Contributor(s): 
;* 

;**
;** ERESTORE.asm
;**
;** PROGRAMMER : Gregg A. Tavares
;**    VERSION : 00.000
;**    CREATED : 03/06/90
;**   MODIFIED : 10/02/92
;**       TABS : 10 18 40 44
;**
;**	     \|///-_
;**	     \oo///_
;**	-----w/-w------
;**	 E C H I D N A
;**	---------------
;**
;** DESCRIPTION
;**	ShowRect/Restore for EGA mode.
;**
;** HISTORY
;**
;**

;=====================================================================
;**
;** EGA_CopyInPlace
;**
;** PURPOSE
;**	Copy a byte aligned rectangle of graphics from an one buffer
;**	to another.  (an area in one buffer to the SAME area in another)
;**
;** INPUT
;**	DS = source buffer segment
;**	ES = dest   buffer segment
;**	AX = xpos
;**	BX = ypos
;**	CX = width
;**	DX = height
;**
;** REGISTER USED
;**	ALL of them
;**
;** EFFECTS
;**	
;**
;** RETURN VALUE
;**	
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	
;**
	MPROC EGA_CopyInPlace NEAR

	call	ClipBox
	jc	@@coordsokay
	ret

@@coordsokay:
	shl	bx,1

	add	cx,ax	; compute last pixel
	dec	cx
	shr	cx,1	; compute last byte
	shr	cx,1
	shr	cx,1
	shr	ax,1	; compute first byte
	shr	ax,1
	shr	ax,1
	sub	cx,ax	; compute width in bytes
	inc	cx

	mov	bp,cx	; save width in bytes

	push	dx
	push	ax
	mov	dx,03C4h	; write to all planes
	mov	ax,0F02h
	out	dx,ax

	mov	dx,03CEh
	mov	ax,0105h	; Select EGA RMD 0/WMD 1
	out	dx,ax
	pop	ax
	pop	dx

@@copyrow:
	mov	cx,bp	; restore width
	mov	di,[cs:LineSt+bx]	; compute starting addr of source
	add	di,ax
	mov	si,[cs:LineSt+bx]	; compute starting addr of dest
	add	si,ax

	cld
	rep	movsb	; copy the bytes.

	add	bx,2	; move index to next line
	dec	dx	; next line?
	jg	@@copyrow
	ret

	ENDP

;
; C PROTOTYPE:
;
; void far EGA_Restore(int xmin, int ymin, int xmax, int ymax);
;
; _Restore - restores from background buffer to the current work page.
; segment of background buffer is in [cs:Pages+4]
; segment of current work page is in [cs:GrPage]
; by Daniel Chang, 07/23/89.

	MPROC	_EGA_Restore
	ARG	xmin:WORD, ymin:WORD, rwidth:WORD, height:WORD=AUTO_SIZE

	push	bp
	mov	bp, sp
	sub	sp, AUTO_SIZE

	push	ds
	push	es
	push	di
	push	si

	mov	ax, [xmin]
	mov	bx, [ymin]
	mov	cx, [rwidth]
	mov	dx, [height]

	mov	ds, [cs:EBackBuffer]	;source segment
	mov	es, [cs:EWorkBuffer]	;destination segment

	call	EGA_CopyInPlace

	pop	si
	pop	di
	pop	es
	pop	ds

	add	sp, AUTO_SIZE
	pop	bp
	ret
	ENDP	_EGA_Restore


;
; C PROTOTYPE:
;
; void far EGA_UnRestore(int xmin, int ymin, int xmax, int ymax);
;
; _UnRestore - 'unrestores' from the current work page to background buffer.
; segment of background buffer is in [cs:Pages+4]
; segment of current work page is in [cs:GrPage]
; by Daniel Chang, 07/23/89.
; Modified from EGA_Restore 7/08/91 (RGM)

	MPROC	_EGA_UnRestore
	ARG	xmin:WORD, ymin:WORD, rwidth:WORD, height:WORD=AUTO_SIZE

	push	bp
	mov	bp, sp
	sub	sp, AUTO_SIZE

	push	ds
	push	es
	push	di
	push	si

	mov	ax, [xmin]
	mov	bx, [ymin]
	mov	cx, [rwidth]
	mov	dx, [height]

	mov	ds, [cs:EWorkBuffer]	;source segment
	mov	es, [cs:EBackBuffer]	;destination segment

	call	EGA_CopyInPlace

	pop	si
	pop	di
	pop	es
	pop	ds

	add	sp, AUTO_SIZE
	pop	bp
	ret
	ENDP	_EGA_UnRestore


;
; C PROTOTYPE:
;
; void far EGA_ShowRect(int xmin, int ymin, int width, int height);
;
; _Restore - copies from work page to the display page.
; segment of current work page is in [cs:GrPage]
; segment of display buffer is in [cs:Pages]
; by Daniel Chang, 07/23/89.

	MPROC	_EGA_ShowRect
	ARG	xmin:WORD, ymin:WORD, rwidth:WORD, height:WORD=AUTO_SIZE

	push	bp
	mov	bp, sp
	sub	sp, AUTO_SIZE

	push	ds
	push	es
	push	di
	push	si

	mov	ax, [xmin]
	mov	bx, [ymin]
	mov	cx, [rwidth]
	mov	dx, [height]

	mov	ds, [cs:EWorkBuffer]		;source segment
	mov	es, [cs:EDisplayBuffer]	;destination segment

	call	EGA_CopyInPlace

	pop	si
	pop	di
	pop	es
	pop	ds

	add	sp, AUTO_SIZE
	pop	bp
	ret
	ENDP	_EGA_ShowRect

