;* 
;* The contents of this file are subject to the Mozilla Public 
;* License Version 1.1 (the "License"); you may not use this file 
;* except in compliance with the License. You may obtain a copy of 
;* the License at http://www.mozilla.org/MPL/ 
;* 
;* Software distributed under the License is distributed on an "AS 
;* IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
;* implied. See the License for the specific language governing 
;* rights and limitations under the License. 
;* 
;* The Original Code is tUME: the Universal Map Editor, released 
;* June 18, 2000. 
;* 
;* The Initial Developer of the Original Code is Echidna. 
;* Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
;* All Rights Reserved. 
;* 
;* Contributor(s): 
;* 

;**
;** MRECT.asm
;**
;** PROGRAMMER : Gregg A. Tavares
;**    VERSION : 00.000
;**    CREATED : 12/13/89
;**   MODIFIED : 10/02/92
;**       TABS : 10 18 40 44
;**
;**	     \|///-_
;**	     \oo///_
;**	-----w/-w------
;**	 E C H I D N A
;**	---------------
;**
;** DESCRIPTION
;**	Routine to draw a rectangle in current color on MCGA screen.
;**
;**
;** HISTORY
;**
;**

	IDEAL

;=====================================================================
;**
;** MCGA_DrawRect
;**
;** PURPOSE
;**	Draws a rectangle on the specified rastport in the current color
;**
;** INPUT
;**	DS:SI RastPort			:Not Implemented
;**	AX = starting x pos
;**	BX = starting y pos
;**	CX = width
;**	DX = height
;**
;** EFFECTS
;**	AX,BX,CX,DX,ES,DI,SI,DS
;**
;** RETURN VALUE
;**	NONE
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	
;**

	MPROC MCGA_DrawRect

	BeforeGraphics

	cld
	shl	bx,1
	mov	di,[CS:LineSt+BX]	; Get Starting address of y row
	add	di,ax	; add x column.

	mov	ax,[CS:MGrSeg]	; Get Segment of current page.
	mov	es,ax

	if USE_GRAFX
	mov	bl,[CS:MPenColor]	; Get Pen Color.
	sub	bh,bh	; Look up thru RemapTable
	mov	al,[CS:_RemapTable+bx]
	mov	ah,al	; put it in both halfs of AX
	else
	mov	al,[CS:MPenColor]	; Get Pen Color.
	inc	al	; Shift color up one.
	mov	ah,al	; put it in both halfs of AX
	endif

	mov	bx,320	; compute modulo of rect.
	sub	bx,cx	; 320 - width.

	mov	si,cx	; save width.

@@draw_row:
	mov	cx,si	; Get Width
	shr	cx,1	; (in words.)

	rep	stosw	; Draw a row of pixels

	test	si,1	; if width is odd plot one more pixel
	jz	@@not_odd

	stosb

@@not_odd:
	add	di,bx	; move to next row.
	dec	dx
	jg	@@draw_row

@@no_rect:
	AfterGraphics
	ret

	ENDP MCGA_DrawRect

;=====================================================================
;**
;** MCGA_RectClip
;**
;** PURPOSE
;**	Clipping for rect routines.
;**
;** INPUT
;**	
;**
;** REGISTER USED
;**	
;**
;** EFFECTS
;**	
;**
;** RETURN VALUE
;**	
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	
;**
	MPROC	MCGA_RectClip

	;** Clip the top left width and height.
	cmp	ax,[CS:MClipRight]
	jg	@@no_rect	; starts past right edge.
	cmp	bx,[CS:MClipBottom]
	jg	@@no_rect	; starts past bottom edge
	mov	si,ax	; get 1 past right side
	add	si,cx
	cmp	si,[CS:MClipLeft]
	jle	@@no_rect	; ends before left edge.
	mov	di,bx	; get 1 below bottom side 
	add	di,dx
	cmp	di,[CS:MClipTop]
	jle	@@no_rect	; ends before top edge.

	push	si

	;* Clip sides that overlap clip window 
	mov	si, ax
	sub	si,[CS:MClipLeft]	; check left edge.
	jge	@@left_ok
	add	cx,si	; clip width.
	sub	ax,si	; clip left.

@@left_ok:
	mov	si,bx
	sub	si,[CS:MClipTop]	; check top edge.
	jge	@@top_ok
	add	dx,si	; clip height.
	sub	bx,si	; clip top.

@@top_ok:
	pop	si
	dec	si
	sub	si,[CS:MClipRight]	; check right edge.
	jle	@@right_ok
	sub	cx,si	; clip width.	

@@right_ok:
	dec	di
	sub	di,[CS:MClipBottom]	; check bottom edge.
	jle	@@bottom_ok
	sub	dx,di	; clip height.	
@@bottom_ok:
	stc
	ret

@@no_rect:
	clc
	ret

	ENDP
;=====================================================================
;**
;** _MCGA_DrawRect(x, y, width, height)
;**
;** PURPOSE
;**	Draws a rectangle on the current workpage in the current color
;**
;** INPUT
;**	From stack using C conventions.
;**
;** RETURN VALUE
;**	NONE
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	
;**
	PUBLIC _MCGA_DrawRectAsm
	MPROC	_MCGA_DrawRectAsm
	ARG	_LEFT:WORD, _TOP:WORD, _WIDTH:WORD, _HEIGHT:WORD

	push	bp
	mov	bp,sp

	push	di	; Save SI and DI
	push	si

	mov	dx,[_HEIGHT]	; Get C Parms
	mov	cx,[_WIDTH]
	mov	bx,[_TOP]
	mov	ax,[_LEFT]

	call	MCGA_RectClip
	jnc	@@no_rect

	call	MCGA_DrawRect	; Draw the Rect

@@no_rect:
	pop	si	; Restore SI and DI
	pop	di

	pop	bp
	ret

	ENDP _MCGA_DrawRectAsm


	if	USE_GRAFX
;=====================================================================
;**
;** MCGA_XORRect
;**
;** PURPOSE
;**	Draws a rectangle on the specified rastport XORed in the
;**	current color
;**
;** INPUT
;**	DS:SI RastPort			:Not Implemented
;**	AX = starting x pos
;**	BX = starting y pos
;**	CX = width
;**	DX = height
;**
;** EFFECTS
;**	AX,BX,CX,DX,ES,DI,SI,DS
;**
;** RETURN VALUE
;**	NONE
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	
;**

	MPROC MCGA_XORRect

	BeforeGraphics

	if 0
	cmp	dx,0	; Skip if height = 0
	je	@@no_rect
	cmp	cx,0	; Skip if width = 0
	je	@@no_rect
	endif

	shl	bx,1
	mov	di,[CS:LineSt+BX]	; Get Starting address of y row
	add	di,ax	; add x column.

	mov	ax,[CS:MGrSeg]	; Get Segment of current page.
	mov	es,ax

	mov	al,[CS:MPenColor]	; Get Pen Color.
	mov	ah,al	; put it in both halfs of AX

	mov	bx,320	; compute modulo of rect.
	sub	bx,cx	; 320 - width.

	mov	si,cx	; save width.

@@draw_row:
	mov	cx,si	; Get Width
	shr	cx,1	; (in words.)
	je	@@skippairs

@@xorloop:
	xor	[es:di],ax	; XOR row of pixels.
	add	di,2
	loop	@@xorloop

@@skippairs:
	test	si,1	; if width is odd plot one more pixel
	jz	@@not_odd

	xor	[es:di],al
	inc	di

@@not_odd:
	add	di,bx	; move to next row.
	dec	dx
	jg	@@draw_row

@@no_rect:
	AfterGraphics
	ret

	ENDP MCGA_XORRect

;=====================================================================
;**
;** _XORRect(x, y, width, height)
;**
;** PURPOSE
;**	Draws a rectangle on the current workpage in the current color
;**
;** INPUT
;**	From stack using C conventions.
;**
;** RETURN VALUE
;**	NONE
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	
;**
	PUBLIC  _MCGA_XORRect
	MPROC	_MCGA_XORRect
	ARG	_LEFT:WORD, _TOP:WORD, _WIDTH:WORD, _HEIGHT:WORD

	push	bp
	mov	bp,sp

	push	di	; Save SI and DI
	push	si

	mov	dx,[_HEIGHT]	; Get C Parms
	mov	cx,[_WIDTH]
	mov	bx,[_TOP]
	mov	ax,[_LEFT]

	call	MCGA_RectClip
	jnc	@@no_rect

	call	MCGA_XORRect	; Draw the Rect

@@no_rect:
	pop	si	; Restore SI and DI
	pop	di

	pop	bp
	ret

	ENDP _MCGA_XORRect

 	endif

