/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * TDBUF.c
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 03/05/90
 *   MODIFIED : 07/27/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		TGA specific DBUF routines.
 *
 * HISTORY
 *
*/

#include <echidna/platform.h>
#include "switches.h"

#include <echidna/etypes.h>
#include <echidna/memory.h>
#include <echidna/eerrors.h>
#include <echidna/dbuf.h>
#include "dbufarea.h"

/***************************** EXTERNAL STUFF *****************************/
extern volatile WORD far		 TGA_RetraceBegun;
extern UWORD TDMemTbl[];
extern UWORD TDAcsTbl[];
extern UBYTE TDispVal;

/**************************** C O N S T A N T S ***************************/
#define GHEADERSIZE	(GSHAPESIZE + TGASDATASIZE)

#define DISPB	0
#define WORKB	1
#define BACKB	2

/*********************************************************************
 *
 * TGA_CloseDBufGraphics
 *
 * SYNOPSIS
 *		void TGA_CloseDBufGraphics (void)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
static void *TGATemp;

void TGA_CloseDBufGraphics (void)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "TGA_CloseDBufGraphics";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	if (TDMemTbl[BACKB])	FreeMemOnPara (MK_FP(TDMemTbl[BACKB], 0));
	DestroyTempBuffer (TGATemp, 16384U + 32768U);

} /* TGA_CloseDBufGraphics */


/*********************************************************************
 *
 * TGA_OpenDBufGraphics
 *
 * SYNOPSIS
 *		short TGA_OpenDBufGraphics (short mode, size_t size)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short TGA_OpenDBufGraphics (short mode, size_t size)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "TGA_OpenDBufGraphics";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	mode = mode;

	TGATemp = CreateTempBuffer (16384U + 32768U);
	if (!TGATemp) {
		SetGlobalErr (ERR_OUT_OF_MEMORY);
		GEprintf ("OOM: allocating TGA gfx memory");
		return FALSE;
	}
	TDMemTbl[BACKB] = FP_SEG(AllocMemOnPara (max (size, 8000U * 4U + 192U * 3U)));	/* work space too :-) */
	if (!TDMemTbl[BACKB]) {
		SetGlobalErr (ERR_OUT_OF_MEMORY);
		GEprintf ("OOM: allocating TGA backbuffer");
		return FALSE;
	}
	return TRUE;
} /* TGA_OpenDBufGraphics */

/*********************************************************************
 *
 * TGA_DBufPeepRect
 *
 * SYNOPSIS
 *		void TGA_DBufPeepRect (
 *			GShape	*shapeptr,
 *			short	 x,
 *			short	 y,
 *			short	 width,
 *			short	 height,
 *		)
 *
 * PURPOSE
 *		To draw a rect in shape's size ^ onto the background buffer, and work buffer and
 *		and enter the area into the shape area list so it will be copied
 *		to the display buffer on the next call to DBufFlipScreen().
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void TGA_DBufPeepRect (
	short	 x,
	short	 y,
	short	 width,
	short	 height
)
{
	/* saptr:			Points to the succesive fields 
	 *					of the current 'AreaRecType' struct pointed to
	 *					by 'SAdrawPtr'. The fields are: 'x', 'width',  
	 *					'y', 'height' and 'backflag'.
	*/	
	/* dataptr:			Data Pointer. Points to the succesive fields of the
	 *					shape data header: <x offset>, <width>,
	 *					<y offset>, <height>. 
	*/

	WORD	*saptr 		= (WORD *)SAdrawPtr++;

	/** Record in draw list the area rectangle where shape will be drawn **/
	*(saptr++) 	= x;
	*(saptr++)	= width;
	*(saptr++) 	= y;
	*(saptr++) 	= height;
  	*saptr		= PEEP_FLAG_MASK;	/* Erase next frame */    

	DrawRect (x, y, width, height);

} /* TGA_DBufPeepRect */

/*********************************************************************
 *
 * TGA_DBufPeepShape
 *
 * SYNOPSIS
 *		void TGA_DBufPeepShape (
 *			GShape	*shapeptr,
 *			short x,
 *			short y
 *		) 
 *
 * PURPOSE
 *		To record the rectangular area where the shape will be drawn in the
 *		shape area draw list.  
 *		Then to draw the shape via call to 'DrawGShape'. This shape will
 *		stay on the display screen for only one frame.
 *
 * INPUT
 *		shapeptr: 	Far pointer to shape data.
 *		x, y:		Screen coordinates for drawing shape.	
 *		SAdrawPtr:	(GLOBAL) Pointer to first free record in Shape Area Draw
 *					List.  
 *  
 * AFFECTS
 *		SAdrawPtr:	(GLOBAL) Incremented to point to next free record.
 *
 * RETURN VALUE
 *		None.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void TGA_DBufPeepShape (
	GShape	*shapeptr,
	short x,
	short y
) 
{
	/* saptr:			Points to the succesive fields 
	 *					of the current 'AreaRecType' struct pointed to
	 *					by 'SAdrawPtr'. The fields are: 'x', 'width',  
	 *					'y', 'height' and 'backflag'.
	*/	
	/* dataptr:			Data Pointer. Points to the succesive fields of the
	 *					shape data header: <x offset>, <width>,
	 *					<y offset>, <height>. 
	*/
	UWORD	*saptr		= (UWORD *)SAdrawPtr++;
	UWORD	*dataptr	= (UWORD *)shapeptr->Data.TGA.ts[x & 0x01];

	/** Record in draw list the area rectangle where shape will be drawn **/
	*(saptr++) 	= x + (*dataptr++);		/* Area->x = x + shape->xoffset */
	*(saptr++)	= (*dataptr++) << 1;	/* Area->width = shape->width   */
	*(saptr++) 	= y + *dataptr++;		/* Area->y = y + shape->yoffset */
	*(saptr++) 	= *dataptr;				/* Area->height = shape->height */
  	*saptr		= PEEP_FLAG_MASK;		/* Erase next frame */

	DrawGShape (shapeptr, x, y);

} /* TGA_DBufPeepShape */

/*********************************************************************
 *
 * TGA_DBufStampShape
 *
 * SYNOPSIS
 *		void TGA_DBufStampShape (
 *			GShape	*shapeptr,
 *			short	 x,
 *			short	 y
 *		)
 *
 * PURPOSE
 *		To draw a shape onto the background buffer, and work buffer and
 *		and enter the area into the shape area list so it will be copied
 *		to the display buffer on the next call to DBufFlipScreen().
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void TGA_DBufStampShape (
	GShape	*shapeptr,
	short	 x,
	short	 y
)
{
	/* saptr:			Points to the succesive fields 
	 *					of the current 'AreaRecType' struct pointed to
	 *					by 'SAdrawPtr'. The fields are: 'x', 'width',  
	 *					'y', 'height' and 'backflag'.
	*/	
	/* dataptr:			Data Pointer. Points to the succesive fields of the
	 *					shape data header: <x offset>, <width>,
	 *					<y offset>, <height>. 
	*/

	UWORD	*saptr		= (UWORD *)SAdrawPtr++;
	UWORD	*dataptr	= (UWORD *)shapeptr->Data.TGA.ts[x & 0x01];

	/** Record in draw list the area rectangle where shape will be drawn **/
	*(saptr++) 	= x + (*dataptr++);		/* Area->x = x + shape->xoffset */
	*(saptr++)	= (*dataptr++) << 1;	/* Area->width = shape->width   */
	*(saptr++) 	= y + *dataptr++;		/* Area->y = y + shape->yoffset */
	*(saptr++) 	= *dataptr;				/* Area->height = shape->height */
  	*saptr		= STAMP_FLAG_MASK;		/* Do not Erase next frame */    

	SetWorkPage (BackBuffer);
	DrawGShape (shapeptr, x, y);	/* in back */
	
	SetWorkPage (WorkBuffer);
	DrawGShape (shapeptr, x, y);	/* in work */

} /* TGA_DBufStampShape */

/*********************************************************************
 *
 * TGA_DBufStampShapeNoTrans
 *
 * SYNOPSIS
 *		void TGA_DBufStampShapeNoTrans (
 *			GShape	*shapeptr,
 *			short	 x,
 *			short	 y
 *		)
 *
 * PURPOSE
 *                   without transparency.
 *		To draw a shape ^ onto the background buffer, and work buffer and
 *		and enter the area into the shape area list so it will be copied
 *		to the display buffer on the next call to DBufFlipScreen().
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void TGA_DBufStampShapeNoTrans (
	GShape	*shapeptr,
	short	 x,
	short	 y
)
{
	/* saptr:			Points to the succesive fields 
	 *					of the current 'AreaRecType' struct pointed to
	 *					by 'SAdrawPtr'. The fields are: 'x', 'width',  
	 *					'y', 'height' and 'backflag'.
	*/	
	/* dataptr:			Data Pointer. Points to the succesive fields of the
	 *					shape data header: <x offset>, <width>,
	 *					<y offset>, <height>. 
	*/

	WORD	*saptr 		= (WORD *)SAdrawPtr++;
	UWORD	 width		= shapeptr->Width;
	UWORD	 height		= shapeptr->Height;

	/** Record in draw list the area rectangle where shape will be drawn **/
	*(saptr++) 	= x;
	*(saptr++)	= width;			/* Area->width = shape->Width   */
	*(saptr++) 	= y;
	*(saptr++) 	= height;			/* Area->height = shape->Height */
  	*saptr		= STAMP_FLAG_MASK;	/* Do not Erase next frame */    

	SetWorkPage (BackBuffer);
	DrawGShapeNoTrans (shapeptr, x, y);	/* in back */
	
	SetWorkPage (WorkBuffer);
	DrawGShapeNoTrans (shapeptr, x, y);	/* in work */

} /* TGA_DBufStampShapeNoTrans */

/*********************************************************************
 *
 * TGA_DBufStampClearShape
 *
 * SYNOPSIS
 *		void TGA_DBufStampClearShape (
 *			GShape	*shapeptr,
 *			short	 x,
 *			short	 y
 *		)
 *
 * PURPOSE
 *		To draw a rect in shape's size ^ onto the background buffer, and work buffer and
 *		and enter the area into the shape area list so it will be copied
 *		to the display buffer on the next call to DBufFlipScreen().
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void TGA_DBufStampClearShape (
	GShape	*shapeptr,
	short	 x,
	short	 y
) {
	/* saptr:			Points to the succesive fields 
	 *					of the current 'AreaRecType' struct pointed to
	 *					by 'SAdrawPtr'. The fields are: 'x', 'width',  
	 *					'y', 'height' and 'backflag'.
	*/	
	/* dataptr:			Data Pointer. Points to the succesive fields of the
	 *					shape data header: <x offset>, <width>,
	 *					<y offset>, <height>. 
	*/

	WORD	*saptr 		= (WORD *)SAdrawPtr++;
	WORD	width		= shapeptr->Width, 
			height		= shapeptr->Height;

	/** Record in draw list the area rectangle where shape will be drawn **/
	*(saptr++) 	= x;
	*(saptr++)	= width;			/* Area->width = shape->width   */
	*(saptr++) 	= y;
	*(saptr++) 	= height;			/* Area->height = shape->height */
  	*saptr		= STAMP_FLAG_MASK;	/* Do not Erase next frame */    

	SetWorkPage (BackBuffer);
	
	SetPaperColor (255);
#if 0
	EvenPatternTbl[255] = shapeptr->TColor;
	OddPatternTbl[255]  = shapeptr->TColor >> 8;
#else
	EvenPatternTbl[255] = shapeptr->Data.TGA.TPatEven;
	OddPatternTbl[255]  = shapeptr->Data.TGA.TPatOdd;
#endif
	DrawRect (x, y, shapeptr->Width, shapeptr->Height);
	
	SetWorkPage (WorkBuffer);
	DrawRect (x, y, shapeptr->Width, shapeptr->Height);

} /* TGA_DBufStampClearShape */

/*********************************************************************
 *
 * TGA_DBufStampRect
 *
 * SYNOPSIS
 *		void TGA_DBufStampRect (
 *			GShape	*shapeptr,
 *			short	 x,
 *			short	 y,
 *			short	 width,
 *			short	 height,
 *		)
 *
 * PURPOSE
 *		To draw a rect in shape's size ^ onto the background buffer, and work buffer and
 *		and enter the area into the shape area list so it will be copied
 *		to the display buffer on the next call to DBufFlipScreen().
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void TGA_DBufStampRect (
	short	 x,
	short	 y,
	short	 width,
	short	 height
)
{
	/* saptr:			Points to the succesive fields 
	 *					of the current 'AreaRecType' struct pointed to
	 *					by 'SAdrawPtr'. The fields are: 'x', 'width',  
	 *					'y', 'height' and 'backflag'.
	*/	
	/* dataptr:			Data Pointer. Points to the succesive fields of the
	 *					shape data header: <x offset>, <width>,
	 *					<y offset>, <height>. 
	*/

	WORD	*saptr 		= (WORD *)SAdrawPtr++;

	/** Record in draw list the area rectangle where shape will be drawn **/
	*(saptr++) 	= x;
	*(saptr++)	= width;
	*(saptr++) 	= y;
	*(saptr++) 	= height;
  	*saptr		= STAMP_FLAG_MASK;	/* Do not Erase next frame */    

	SetWorkPage (BackBuffer);
	DrawRect (x, y, width, height);

	SetWorkPage (WorkBuffer);
	DrawRect (x, y, width, height);

} /* TGA_DBufStampRect */

/*********************************************************************
 *
 * RestorePeepAreas
 *
 * SYNOPSIS
 *		void RestorePeepAreas (
 *			AreaRecType	*arealist
 *		)
 *
 * PURPOSE
 *		To copy the rectangular peep areas in 'arealist' from the Back screen 
 *		to the Work screen via call to 'Restore'.  
 *
 * INPUT
 *		arealist:	Array of AreaRecType structures with end of list marked
 *					by entry with height field of 0.
 *		
 * AFFECTS
 *		Copies areas from background buffer to work buffer. 
 *
 * RETURN VALUE
 *		None.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *		Restore()
 *
*/

static void RestorePeepAreas (
	AreaRecType	*arealist
)
{
	WORD al_height;

	for ( ; (al_height = arealist->height) != 0; arealist++) {
			if (arealist->flags & PEEP_FLAG_MASK) {
				Restore (arealist->x, arealist->y, 
							arealist->width, al_height);		
			}
	}

} /* RestorePeepAreas */
	
/*********************************************************************
 *
 * RestoreStampAreas
 *
 * SYNOPSIS
 *		void RestoreStampAreas (
 *			AreaRecType	*arealist
 *		)
 *
 * PURPOSE
 *		To copy the rectangular areas in 'arealist' from the Background
 *		screen to the Work screen via call to 'Restore'.  
 *
 * INPUT
 *		arealist:	Array of AreaRecType structures with end of list marked
 *					by entry with height field of 0.
 *
 * AFFECTS
 *		Copies areas from background buffer to work buffer. 
 *
 * RETURN VALUE
 *		None.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *		Restore()
 *
*/
static void RestoreStampAreas (
	AreaRecType	*arealist
)
{
	WORD al_height;

	for ( ; (al_height = arealist->height) != 0; arealist++) {
		if (arealist->flags & STAMP_FLAG_MASK) {
			Restore (arealist->x, arealist->y, 
						arealist->width, al_height);		
		}
	}

} /* RestoreStampAreas */

/*********************************************************************
 *
 * TGA_DBufFlipScreen
 *
 * SYNOPSIS
 *		void TGA_DBufFlipScreen (void)
 *
 * PURPOSE
 *		To ShapeFlip double buffering screens and update the display.
 *
 * INPUT
 *		(GLOBALS)
 *		SAList1, SAList2:	Shape Area lists (arrays).
 *		SAeraseList:		List from which to restore areas of Display  
 *							screen where old shapes are.
 *		SAdrawList:			List from which to update areas of Display
 *							screen where new shapes are. List from which
 *							to restore areas of Work screen where new shapes
 *							were drawn.
 *		SAerasePtr:			Pointer to end of SAerasList.
 *		SAdrawPtr:			Pointer to end of SAdrawList.
 *
 * AFFECTS
 *		(GLOBALS)
 *		'SAeraseList' and 'SAdrawList' are swapped.
 *		'SAerasePtr' is set to final value of 'SAdrawPtr'.
 *		'SAdrawPtr' is set to start of new 'SAdrawList'.
 *
 * RETURN VALUE
 *		None.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *		DisplayWorkAreas (), RestoreBackAreas (),
 *		SWAP_RESET_SALISTS ()
 *
*/
void TGA_DBufFlipScreen (void)
{
	UWORD	temp;

	/* Mark end of area lists */
		SAerasePtr->height = 0;
		SAdrawPtr->height = 0; 

	/* Flip Display and Work Buffers */
		SetWorkPage (WorkBuffer);
		temp        = TDAcsTbl[1];
		TDAcsTbl[1] = TDAcsTbl[0];
		TDAcsTbl[0] = temp;

		TDispVal = (temp >> 3) | 0300; /* octal */
		outportb (0x03DF, TDispVal);

	/* Wait for Display to flip */
		TGA_RetraceBegun = FALSE;
		while (!TGA_RetraceBegun) {}
	

	/* Copy Old Shape areas from Work to Display */
		RestoreStampAreas (SAdrawList);

	/* Copy New Shape areas from Work to Display */
		RestorePeepAreas (SAeraseList);

	/* Reset shape save areas */
		SWAP_RESET_SALISTS ();
} /* TGA_DBufFlipScreen */

typedef void (far *RoutinePtr)();

static RoutinePtr TGADBufRoutineTable[] = {
	(RoutinePtr)TGA_OpenDBufGraphics,
	(RoutinePtr)TGA_CloseDBufGraphics,
	(RoutinePtr)TGA_DBufPeepShape,
	(RoutinePtr)TGA_DBufStampShape,
	(RoutinePtr)TGA_DBufStampShapeNoTrans,
	(RoutinePtr)TGA_DBufStampClearShape, 
	(RoutinePtr)TGA_DBufStampRect,
	(RoutinePtr)TGA_DBufPeepRect,
	(RoutinePtr)TGA_DBufFlipScreen,
};


/*********************************************************************
 *
 * GetTGADBufRoutineTable
 *
 * SYNOPSIS
 *		RoutinePtr *GetTGADBufRoutineTable (void)
 *
 * PURPOSE
 *		Returns pointer to TGADBufRoutineTable.  Calling this
 *		routine should force loading if TGA_DBufRoutines as an
 *		overlay.
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
RoutinePtr *GetTGADBufRoutineTable (void)
{
	return TGADBufRoutineTable;

} /* GetTGADBufRoutineTable */

