/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * XMS.C
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 10/30/91
 *   MODIFIED : 10/06/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		XMS Memory Routines
 *
 * HISTORY
 *		01/27/92 (dcc) - handle case of empty bk->FreeList in FreeXMS()
 *		02/21/92 (dcc) - ActivateXMSatWin() checks whether new request
 *						 is for a block that is already activated
 *		03/14/92 (dcc) - Added pXmsNew() & pXmsDelete() to allocate
 *						 several XMSPntrs at once. This save about
 *						 6 bytes per XMSPntr generated
 *		04/03/92 (dcc) - ReadXMS() sees if request is already in main
 *						 memory, and copies from there if it exists
 *		04/03/92 (dcc) - WriteXMS() sees if XMS is paged into main
 *						 memory, and copies to there as well if it exists
 *		04/16/92 (dcc) - fix code that checks for access to already
 *						 activated blocks
 *		05/05/92 (dcc) - UpdateXMS() sees if XMS block is paged into
 *						 another physical page, and updates other
 *						 physical pages as well
 *		05/06/92 (dcc) - FreeXMS() sees if XMS block is paged into
 *						 a physical page, and marks block as no
 *						 longer being paged in
 *		08/29/92 (GAT) - Added SetMaxXMSWindows()
 *		08/29/92 (GAT) - Added support for GMEMLIB and it's
 *						 restrictions on freeing memory.  Books
 *						 are now freed when they are no longer
 *						 in use and if GMEMLIB then the XMSPntr
 *						 mechanism is compiled out.
 *		11/18/92 (GAT) - changed AvailXMS() to return -1 if
 *						 not used.
 *		05/11/93 (dcc) - change ActivateXTRAatWin() so if XMS block
 *						 is already in another window, then simply
 *						 return a pointer to that XMS block.
 *						 This ensures the same XMS block appears
 *						 only once.
 *		05/18/93 (dcc) - Add ReleaseXMS() which clears pxmsLast[].
 *						 Add debug warning to ActivateXMSatWin().
 *						 Add xmsUsage[] to stack Activates/Releases.
 *		05/18/93 (dcc) - HOO BOY. 05/11/93 changes breaks tUME in
 *						 a big way. Someone needs to do more testing...
 *						 Anyhow, change UpdateXMS() so it clears
 *						 pxmsLast[] after it does its work.
 *		03/07/94 (dcc) - Add SetXMSMemoryPoolSize()
 *		10/06/94 (dcc) - updated to use <echidna/alloc.h>
 *
 *	THEORY
 *		ActivateXMSatWin() increases xmsUsage[] for the particular XMS window
 *		frame by one. UpdateXTRA() and ReleaseXTRA() decreases xmsUsage[] for
 *		the XMS window frame by one.
 *
 *		So, ActivateXMSatWin() will activate a previously activated XMS window
 *		if the xmsUsage[] for that window is non-zero; otherwise it will
 *		ActivateXMSatWin() in the specified window.
*/

#include <echidna/platform.h>
#include "switches.h"

#include <stdio.h>

#include <echidna/etypes.h>
#include <echidna/alloc.h>
#include <echidna/eerrors.h>
#include <echidna/listfunc.h>
#include <echidna/memory.h>
#include <echidna/exit.h>
#include <echidna/xms.h>

#include <dos.h>
#include <alloc.h>

#define LOCAL static

/**************************** C O N S T A N T S ***************************/

#define MAX_BOOK_KBYTES	512U
#define WINDOW_SIZE		16384U

#define BUNCHOFXMSPNTR	160

#define iWindowsMax		(4)

/******************************** T Y P E S *******************************/

typedef struct {
	NodeType	 Node;
	UWORD		 XMSHandle;
	ULONG		 Size;
	ListType	 FreeListX;
	ListType	*FreeList;
} Book;

typedef struct {
	NodeType	 Node;
	ULONG		 Offset;
	ULONG		 Size;
} FreeMemType;

typedef union {
	void far32	*Data;
	ULONG		 Offset;
} MemPos;

typedef struct {
	ULONG	 Length;
	UWORD	 SrcHandle;
	MemPos	 SrcPos;
	UWORD	 DstHandle;
	MemPos	 DstPos;
} MoveRequest;

typedef struct {
	NodeType	 Node;
	XMSPntr		*XMS;
} XMSTracker;

/****************************** G L O B A L S *****************************/


/******************************* M A C R O S ******************************/


/******************************* L O C A L S ******************************/

LOCAL UWORD cXMSMaxKBytes = 0;			/* How much XMS we may allocate */
LOCAL UWORD cXMSKBytesAllocated = 0;	/* # XMS KBytes already allocated */

LOCAL ListType		 BookListX;
LOCAL ListType		*BookList = &BookListX;
LOCAL ListType		 XMSStackX;
LOCAL ListType		*XMSStack = &XMSStackX;
LOCAL XMSPntr		*LastXMS;
LOCAL short			 XMSInit = FALSE;
LOCAL UBYTE			*XMSBuffer = NULL;
LOCAL ListType		 lstFreeXMSX;
LOCAL ListType		*lstFreeXMS = &lstFreeXMSX;
LOCAL short			 iWindows = iWindowsMax;

/* pxmsLast - shows what XMS memory area each XMS window currently holds. */

static XMSPntr pxmsLast[iWindowsMax] =
{	{0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0}	};

static short xmsUsage[iWindowsMax] = {0, };

extern UWORD	 CallXMS (UWORD ax, UWORD dx, UWORD ds, UWORD si);
extern void far32	*XMSFunc;
extern UWORD	 XMS_DX;
extern UWORD	 XMS_BX;


/***************************** R O U T I N E S ****************************/


/*********************************************************************
 *
 * SetXMSMemoryPoolSize
 *
 * PURPOSE
 *		Call this routine to set the maximum amount of XMS (in Kbytes)
 *		that may be allocated via AllocXMS().
 *
 *		It is NOT necessary to call this routine. It is probably
 *		most useful to assist in debugging low XMS memory conditions.
 *
 *		If you call this routine with 0, it will use all available XMS.
 *
 * INPUT
 *		UWORD cXMSKBytes	: number of K bytes to limit XMS memory
 *
 * ASSUMES
 *		
 *
 * RETURN VALUE
 *		NONE.
 *
 * HISTORY
 *		03/07/94 Monday (dcc) - created.
 *
*/
void SetXMSMemoryPoolSize(UWORD cXMSKBytes)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "SetXMSMemoryPoolSize";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	cXMSMaxKBytes = cXMSKBytes;

} /* SetXMSMemoryPoolSize */


/*********************************************************************
 *
 * SetMaxXMSWindows
 *
 * SYNOPSIS
 *		void SetMaxXMSWindows (short maxwins)
 *
 * PURPOSE
 *		Set up the maximum 'windows' the XMS routines will use.
 *		Normally, the XMS routines need to allocate a 64K byte
 *		buffer to emulate the EMS routines but if you call this
 *		routine you can tell the XMS routines you need less windows
 *		and if you are only ever going to call ReadXTRA/WriteXTRA
 *		then you can call this with '0' and have no buffer allocated.
 *
 * INPUT
 *		maxwins	=	(0 - 4)
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void SetMaxXMSWindows (short maxwins)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "SetMaxXMSWindows";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	iWindows = maxwins;

} /* SetMaxXMSWindows */


/*********************************************************************
 *
 * SameXMSLast
 *
 * SYNOPSIS
 *		LOCAL BOOL SameXMSLast(XMSPntr *xp, short window)
 *
 * PURPOSE
 *		Returns TRUE if XMS pointer was the last one activated
 *		in <window>.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL BOOL SameXMSLast(XMSPntr *xp, short window)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "SameXMSLast";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	if (pxmsLast[window].Size		== xp->Size &&
		pxmsLast[window].XMSHandle	== xp->XMSHandle &&
		pxmsLast[window].XMSOffset	== xp->XMSOffset)
	{
		UWORD uw = xp->Size;

		/* If block is larger than WINDOW_SIZE, make sure subsequent windows
			have not been re-activated (pxmsLast[].Size == 0xFFFFU). */

		while (uw > WINDOW_SIZE)
		{
			uw -= WINDOW_SIZE;
			window++;
			if (window > iWindows)
				return TRUE;

			if (pxmsLast[window].Size		!= 0xFFFFU ||
				pxmsLast[window].XMSHandle	!= 0 ||
				pxmsLast[window].XMSOffset	!= 0)
			{
				return FALSE;
			}
		}
		return TRUE;
	}
	return FALSE;
} /* SameXMSLast */


/*********************************************************************
 *
 * ClearXMSLast
 *
 * SYNOPSIS
 *		LOCAL void ClearXMSLast(short window)
 *
 * PURPOSE
 *		Mark local variables to indicate that <window> no longer
 *		contains any activated XMS memory.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL void ClearXMSLast(short window)
{
	UWORD uw;

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ClearXMSLast";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	uw = pxmsLast[window].Size;

	xmsUsage[window]			= 0;
	pxmsLast[window].Size		= 0;
	pxmsLast[window].XMSHandle	= 0;
	pxmsLast[window].XMSOffset	= 0;

	/* If block is larger than WINDOW_SIZE, mark subsequent windows
		as being no longer in use as well. */

	while (uw > WINDOW_SIZE)
	{
		uw -= WINDOW_SIZE;
		window++;
		if (window > iWindows)
			return;

		xmsUsage[window]			= 0;
		pxmsLast[window].Size		= 0;
		pxmsLast[window].XMSHandle	= 0;
		pxmsLast[window].XMSOffset	= 0;
	}

} /* ClearXMSLast */


/*********************************************************************
 *
 * MarkXMSLast
 *
 * SYNOPSIS
 *		LOCAL void MarkXMSLast(XMSPntr *xp, short window)
 *
 * PURPOSE
 *		Mark local variables to keep track of last XMS pointer
 *		activated in <window>.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL void MarkXMSLast(XMSPntr *xp, short window)
{
	short i;
	UWORD uw;

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "MarkXMSLast";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	/* Clear previously Activated then Released/Updated in other windows */

	for (i = 0; i < iWindows; i++)
		if (i != window)
			if (SameXMSLast(xp, i))
				ClearXMSLast(i);			/* ASSERT: xmsUsage[i] == 0! */

							uw	= xp->Size;
	pxmsLast[window].Size		= uw;
	pxmsLast[window].XMSHandle	= xp->XMSHandle;
	pxmsLast[window].XMSOffset	= xp->XMSOffset;
	xmsUsage[window]++;

	/* If block is larger than WINDOW_SIZE, mark subsequent windows
		as being in use as well (set pxmsLast[].Size = 0xFFFFU). */

	while (uw > WINDOW_SIZE)
	{
		uw -= WINDOW_SIZE;
		window++;
		if (window > iWindows)
			return;

		pxmsLast[window].Size		= 0xFFFFU;
		pxmsLast[window].XMSHandle	= 0;
		pxmsLast[window].XMSOffset	= 0;
		xmsUsage[window]++;
	}

} /* MarkXMSLast */


/*********************************************************************
 *
 * pXmsNew
 *
 * SYNOPSIS
 *		LOCAL XMSPntr *pXmsNew (void)
 *
 * PURPOSE
 *		Return a XMSPntr from lstFreeXMS if available,
 *		otherwise, allocate a new block of XMSPntr.
 *
 *		Rationale: currently, sizeof (XMSPntr) == 10. Turbo C allocates
 *		in 16 byte units. Therefore, if you allocate XMSPntr one at a
 *		time, you will waste 6 bytes each time (16 - 10 = 6). Thus we
 *		allocate BUNCHOFXMSPNTR at once.
 *
 *		Note: even though a XMSPntr does not have a node header, we still
 *		add it to XMSFreeList. There is an implied type conversion from
 *		struct XMSPntr to struct NodeType.
 *
 *		Note: even though a NodeType is bigger than a XMSPntr,
 *		we are only interested in the first 9 bytes of NodeType, so
 *		everything works (for now...)
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *		Pointer to a free XMSPntr to use.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL XMSPntr *pXmsNew (void)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "pXmsNew";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	#if USE_GAMEMEM
	return calloc (1, sizeof (XMSPntr));
	#else
	{
		UBYTE		*pMem;

		if ((pMem = RemTail(lstFreeXMS)) != NULL)
		{
			memset(pMem, 0, sizeof(XMSPntr));	/* zero out (crap) Node info */
			return (XMSPntr *) pMem;
		}
		else
		{
			UBYTE		*pMem;

			pMem = MEM_calloc (BUNCHOFXMSPNTR, sizeof(XMSPntr));
			if (pMem)
			{
				int i;

				for (i = 1; i < BUNCHOFXMSPNTR; i++)
				{
					AddTail(lstFreeXMS, pMem + i * sizeof(XMSPntr));
				}
				return (XMSPntr *) pMem;
			}
		}
		return NULL;
	}
	#endif

} /* pXmsNew */


/*********************************************************************
 *
 * pXmsDelete
 *
 * SYNOPSIS
 *		LOCAL void pXmsDelete (XMSPntr *pXMS)
 *
 * PURPOSE
 *		Add pXMS to XMSFreeList.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL void pXmsDelete (XMSPntr *pXMS)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "pXmsDelete";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	#if USE_GAMEMEM
	free (pXMS);
	#else
	((NodeType *)(pXMS))->Type = 0;	/* Fix so AddTail does generate error */
	AddTail(lstFreeXMS, pXMS);
	#endif

} /* pXmsDelete */


/*********************************************************************
 *
 * AddNewBook
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *		03/07/94 Monday (dcc) - add support for cXMSKBytesAllocated
 *		03/07/94 Monday (dcc) - also set bk->Size (which wasn't set B4 !?)
 *
 * SEE ALSO
 *
*/
LOCAL short AddNewBook (void)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "AddNewBook";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		Book		*bk;
		UWORD		 kbytes;
		FreeMemType	*fmt;

		bk = CreateNode (sizeof (Book), NULL);
		if (!bk) {
			return FALSE;
		}

		kbytes = CallXMS (0x08, 0, 0, 0);
		if (!kbytes) {
			DeleteNode (bk);
			return FALSE;
		}
		
		#if	0
		printf ("%dK XMS Free DX\n", XMS_DX);
		printf ("%dK largest Free\n", kbytes);
		#endif

		kbytes = min (kbytes, MAX_BOOK_KBYTES);

		/* See if there is a limit on how much XMS we may allocate */

		if (cXMSMaxKBytes && cXMSMaxKBytes >= cXMSKBytesAllocated)
			kbytes = min(kbytes, cXMSMaxKBytes - cXMSKBytesAllocated);

		if (!CallXMS (0x09, kbytes, 0, 0)) {
			DeleteNode (bk);
			return FALSE;
		}
		cXMSKBytesAllocated += kbytes;

		bk->XMSHandle = XMS_DX;
		bk->Size = kbytes * 1024;
		bk->FreeList = &bk->FreeListX;
		InitList (bk->FreeList);
		AddTail (BookList, bk);

		fmt = CreateNode (sizeof (FreeMemType), NULL);
		if (!fmt) {
			DeleteNode (bk);
			return FALSE;
		}
		fmt->Size = kbytes * 1024L;
		AddTail (bk->FreeList, fmt);

		return TRUE;
	}
} /* AddNewBook */


/*********************************************************************
 *
 * FreeBook
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *		03/07/94 Monday (dcc) - add support for cXMSKBytesAllocated
 *
 * SEE ALSO
 *
*/
LOCAL void FreeBook (Book *bk)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "FreeBook";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	CallXMS (0x0A, bk->XMSHandle, 0, 0);
	cXMSKBytesAllocated -= (UWORD) (bk->Size / 1024);
	EmptyList (bk->FreeList);
	DeleteNode (bk);

} /* FreeBook */


/*********************************************************************
 *
 * FreeBooks
 *
 * SYNOPSIS
 *		void FreeBooks (void)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL void FreeBooks (void)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "FreeBooks";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		Book	*bk;

		while ((bk = RemTail (BookList)) != NULL) {
			FreeBook (bk);
		}
		EmptyList (XMSStack);
	}
} /* FreeBooks */


/*********************************************************************
 *
 * FreeUnusedBooks
 *
 * SYNOPSIS
 *		void FreeUnusedBooks (void)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void FreeUnusedBooks (void)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "FreeUnusedBooks";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

{
	Book	*bk;

	bk = Tail (BookList);
	while (!StartOfList (bk))
	{
		Book		*prev;
		FreeMemType	*fmt;

		prev = Prev (bk);

		fmt = Head (bk->FreeList);
		if (!EndOfList (fmt))
		{
			if (fmt->Size == bk->Size)
			{
				Remove (bk);
				FreeBook (bk);
			}
		}

		bk = prev;
	}
}	
} /* FreeUnusedBooks */


/*********************************************************************
 *
 * InitXMS
 *
 * SYNOPSIS
 *		LOCAL short InitXMS (short windows)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short InitXMS (short windows)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "InitXMS";
#endif
#if REQUIRE
	if (windows < 0 || windows > iWindows) {
		SetGlobalErr (ERR_GENERIC);
		GEcatf ("InitXMS (windows must be 0-4)");
		return FALSE;
	}
#endif
/*-------------------------------------------------------------------------*/

	{
		union REGS		inregs;
		union REGS		outregs;
		struct SREGS	segregs;

		inregs.x.ax = 0x4300;
		int86 (0x2F, &inregs, &outregs);
		if (outregs.h.al != 0x80) {
			/* No XMS */
			return FALSE;
		}

		inregs.x.ax = 0x4310;
		int86x (0x2F, &inregs, &outregs, &segregs);

		XMSFunc = MK_FP (segregs.es, outregs.x.bx);

		if (windows) {
			XMSBuffer = AllocMemOnPara (windows * WINDOW_SIZE - 1); /* Kludge */
			if (!XMSBuffer) {
				/* No XMS Buffer available */
				return FALSE;
			}
		}

		InitList (BookList);
		InitList (XMSStack);
		InitList (lstFreeXMS);

		if (!AddNewBook ()) {
			if (XMSBuffer)
				FreeMemOnPara (XMSBuffer);
			return FALSE;
		}
		AddExitFunc (FreeBooks);

		XMSInit = TRUE;
	}
	return TRUE;

} /* InitXMS */



/*********************************************************************
 *
 * ActivateXMSatWin
 *
 * SYNOPSIS
 *		void far32 *ActivateXMSatWin (XMSPntr *xp, short window)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void far32 *ActivateXMSatWin (XMSPntr *xp, short window)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ActivateXMSatWin";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		MoveRequest		 mr;
		UBYTE huge32		*dest;
		WORD i;

		if (XMSBuffer == NULL)
			return NULL;

#if 0
		if (xmsUsage[window] != 0)
		{
			puts("Should have been ReleaseXMS()'ed\n");
		}
#endif

		/* Search all windows first to see if already activated */

		for (i = 0; i < iWindows; i++)
		{
			/* See if new request is for block we have already activated? */

			if (SameXMSLast(xp, i) && xmsUsage[i])
			{
				MarkXMSLast(xp, i);		/* updates pxmsLast[] */
				xp->XMSWindow = (UBYTE) i;
				LastXMS = xp;
				return (void far32 *) (XMSBuffer + (i * WINDOW_SIZE));
			}
		}
		/* Otherwise see if previously Activated
			then Released/Updated in the same window */

		if (SameXMSLast(xp, window))
		{
			MarkXMSLast(xp, window);	/* updates pxmsLast[] */
			xp->XMSWindow = (UBYTE) window;
			LastXMS = xp;
			return (void far32 *) (XMSBuffer + (window * WINDOW_SIZE));
		}

		dest = XMSBuffer + (window * WINDOW_SIZE);

		MarkXMSLast(xp, window);		/* updates pxmsLast[] */
		mr.Length        = xp->Size;
		mr.SrcHandle     = xp->XMSHandle;
		mr.SrcPos.Offset = xp->XMSOffset;
		mr.DstHandle     = 0;
		mr.DstPos.Data   = (void far32 *)dest;

		CallXMS (0x0B, 0, FP_SEG (&mr), FP_OFF (&mr));

		xp->XMSWindow = (UBYTE) window;
		LastXMS = xp;

		return (void far32 *)dest;
	}
} /* ActivateXMSatWin */



/*********************************************************************
 *
 * FreeXMS
 *
 * SYNOPSIS
 *		void FreeXMS (XMSPntr *xp)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void FreeXMS (XMSPntr *xp)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "FreeXMS";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		Book		*bk;
		FreeMemType	*fmt;
		FreeMemType	*nextfmt;
		WORD i;

		/* If XMS memory block is paged into some window, then
			mark block as no longer being paged in. */

		for (i = 0; i < iWindows; i++)
		{
			if (SameXMSLast(xp, i))
			{
				ClearXMSLast(i);
			}
		}

		bk = Head (BookList);
		while (!EndOfList (bk)) {
			if (xp->XMSHandle == bk->XMSHandle) {
				fmt = Head (bk->FreeList);
				if (EndOfList (fmt)) {
					nextfmt = CreateNode (sizeof(FreeMemType), NULL);
					if (!nextfmt) {
						SetGlobalErr (ERR_OUT_OF_MEMORY);
						GEcatf ("\n FreeXMS () no memory for FreeMemType");
						return;
					}
					nextfmt->Offset = xp->XMSOffset;
					nextfmt->Size = xp->Size;
					AddTail(bk->FreeList, nextfmt);
					pXmsDelete (xp);
					return;
				}
				while (!EndOfList (fmt)) {
					if (xp->XMSOffset + xp->Size < fmt->Offset) {
						nextfmt = CreateNode (sizeof(FreeMemType), NULL);
						if (!nextfmt) {
							SetGlobalErr (ERR_OUT_OF_MEMORY);
							GEcatf ("\n FreeXMS () no memory for FreeMemType");
							return;
						}
						nextfmt->Offset = xp->XMSOffset;
						nextfmt->Size   = xp->Size;
						InsertBefore (fmt, nextfmt);
						pXmsDelete (xp);
						return;
					}
					if (xp->XMSOffset + xp->Size == fmt->Offset) {
						fmt->Offset -= xp->Size;
						fmt->Size += xp->Size;
						nextfmt = Prev (fmt);
						if (!StartOfList (nextfmt)) {
							if (nextfmt->Offset + nextfmt->Size == fmt->Offset) {
								nextfmt->Size += fmt->Size;
								Remove (fmt);
								DeleteNode (fmt);
							}
						}
						pXmsDelete (xp);
						return;
					}
					if (xp->XMSOffset == fmt->Offset + fmt->Size) {
						fmt->Size += xp->Size;
						nextfmt = Next (fmt);
						if (!EndOfList (nextfmt)) {
							if (fmt->Offset + fmt->Size == nextfmt->Offset) {
								fmt->Size += nextfmt->Size;
								Remove (nextfmt);
								DeleteNode (nextfmt);
							}
						}
						pXmsDelete (xp);
						return;							
					}
					if (xp->XMSOffset > fmt->Offset + fmt->Size) {
						if (EndOfList (Next(fmt))) {
							nextfmt = CreateNode (sizeof(FreeMemType), NULL);
							if (!nextfmt) {
								SetGlobalErr (ERR_OUT_OF_MEMORY);
								GEcatf ("\n FreeXMS () no memory for FreeMemType");
								return;
							}
							nextfmt->Offset = xp->XMSOffset;
							nextfmt->Size   = xp->Size;
							InsertAfter (fmt, nextfmt);
							pXmsDelete (xp);
							return;
						}
					}
					fmt = Next (fmt);
				}
			}
			bk = Next (bk);
		}

		FreeUnusedBooks ();

		SetGlobalErr (ERR_GENERIC);
		GEcatf ("\nFreeXMS(): Bad XMSPntr, no matching book.");
	}
} /* FreeXMS */


/*********************************************************************
 *
 * FindFree
 *
 * SYNOPSIS
 *		LOCAL short FindFree (Book *bk, size_t size, XMSPntr *xp)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL short FindFree (Book *bk, size_t size, XMSPntr *xp)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "FindFree";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		FreeMemType	*fmt;

		fmt = Head (bk->FreeList);
		while (!EndOfList (fmt)) {
			if (fmt->Size >= size) {
				xp->Size      = size;
				xp->XMSHandle = bk->XMSHandle;
				xp->XMSOffset = fmt->Offset;
				fmt->Offset  += size;
				fmt->Size    -= size;
				if (!fmt->Size) {
					Remove (fmt);
					DeleteNode (fmt);
				}
				return TRUE;
			}
			fmt = Next (fmt);
		}
		return FALSE;
	}
} /* FindFree */


/*********************************************************************
 *
 * AllocXMS
 *
 * SYNOPSIS
 *		XMSPntr *AllocXMS (size_t size)
 *
 * PURPOSE
 *		Allocate upto 64K of XMS memory;
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
XMSPntr *AllocXMS (size_t size)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "AllocXMS";
#endif
#if REQUIRE
	if (!size) {
		GEprintf ("Called AllocXMS() with a ZERO size");
		return NULL;
	}
#endif
/*-------------------------------------------------------------------------*/
	{
		Book		*bk;
		XMSPntr		*xp;

		if (!XMSInit) {
			if (!InitXMS (iWindows)) {
				return NULL;
			}
		}

		xp = pXmsNew();
		if (xp) {
			size = size + 15 & 0xFFF0;
			bk   = Head (BookList);
			while (!EndOfList (bk)) {
				if (FindFree (bk, size, xp)) {
					return xp;
				}
				bk = Next (bk);
			}
			if (AddNewBook ()) {
				if (FindFree (Tail (BookList), size, xp)) {
					return xp;
				}
			}
			pXmsDelete (xp);
		}
		return NULL;
	}
} /* AllocXMS */


/*********************************************************************
 *
 * UpdateXMS
 *
 * SYNOPSIS
 *		void UpdateXMS (XMSPntr *xp)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void UpdateXMS (XMSPntr *xp)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "UpdateXMS";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		MoveRequest		 mr;

		if (XMSBuffer == NULL)
			return;

		mr.Length        = xp->Size;
		mr.SrcHandle     = 0;
		mr.SrcPos.Data   = XMSBuffer + (xp->XMSWindow * WINDOW_SIZE);
		mr.DstHandle     = xp->XMSHandle;
		mr.DstPos.Offset = xp->XMSOffset;
		
		CallXMS (0x0B, 0, FP_SEG (&mr), FP_OFF (&mr));

		if (xmsUsage[xp->XMSWindow])
			xmsUsage[xp->XMSWindow]--;
	}

} /* UpdateXMS */


/*********************************************************************
 *
 * ReleaseXMS
 *
 * SYNOPSIS
 *		void ReleaseXMS (XMSPntr *xp)
 *
 * PURPOSE
 *		Frees window used by <xp>. Does not update XMS with copy
 *		in window. Just releases it so subsequent ActivateXTRA does
 *		NOT try to use the copy in a different window.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void ReleaseXMS (XMSPntr *xp)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ReleaseXMS";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	if (xmsUsage[xp->XMSWindow])
		xmsUsage[xp->XMSWindow]--;

} /* ReleaseXMS */


/*********************************************************************
 *
 * ReadXMS
 *
 * SYNOPSIS
 *		void ReadXMS (XMSPntr *xp, UWORD uwOffset, UWORD uwLength,
 *					void far32 *pMem)
 *
 * PURPOSE
 *		Copies <uwLength> bytes from <uwOffset> from the start of
 *		XMS memory chunk <xp> to <pMem>.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void ReadXMS (XMSPntr *xp, UWORD uwOffset, UWORD uwLength, void far32 *pMem)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ReadXMS";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		MoveRequest		 mr;
		UBYTE ubSaveMem;

		if (SameXMSLast(xp, xp->XMSWindow))
		{
			memcpy(pMem, XMSBuffer + (xp->XMSWindow * WINDOW_SIZE) + uwOffset,
						uwLength);
			return;
		}

		mr.Length        = uwLength;

		if (uwLength & 1)				/* If length is odd, then... */
		{
			/* Save 1st byte following the last byte to read in ubSaveMem. */

			ubSaveMem = *((UBYTE *) pMem + uwLength);

			/* Read an extra byte (so read request is even. */

			mr.Length++;
		}

		mr.SrcHandle     = xp->XMSHandle;
		mr.SrcPos.Offset = xp->XMSOffset + uwOffset;
		mr.DstHandle     = 0;
		mr.DstPos.Data   = pMem;

		CallXMS (0x0B, 0, FP_SEG (&mr), FP_OFF (&mr));

		if (uwLength & 1)
		{
			/* Restore 1st byte following the last byte to read. */

			*((UBYTE *) pMem + uwLength) = ubSaveMem;
		}
	}

} /* ReadXMS */


/*********************************************************************
 *
 * WriteXMS
 *
 * SYNOPSIS
 *		void WriteXMS (void far32 *pMem, UWORD uwLength,
 *						XMSPntr *xp, UWORD uwOffset)
 *
 * PURPOSE
 *		Copies <uwLength> bytes from <pMem> to
 *		<uwOffset> from the start of XMS memory chunk <xp>.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void WriteXMS (void far32 *pMem, UWORD uwLength, XMSPntr *xp, UWORD uwOffset)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "WriteXMS";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		MoveRequest		 mr;
		UBYTE ubSaveMem[2];
		WORD i;

		for (i = 0; i < iWindows; i++)
		{
			if (SameXMSLast(xp, i))
			{
				memcpy(XMSBuffer + (i * WINDOW_SIZE) + uwOffset, pMem,
							uwLength);
			}
		}

		if (uwLength & 1)				/* If length is odd, then... */
		{
			/* Read last byte and byte+1 from XMS into ubSaveMem[]. */

			mr.Length        = 2;
			mr.SrcHandle     = xp->XMSHandle;
			mr.SrcPos.Offset = xp->XMSOffset + uwOffset + (uwLength-1);
			mr.DstHandle     = 0;
			mr.DstPos.Data   = ubSaveMem;

			CallXMS (0x0B, 0, FP_SEG (&mr), FP_OFF (&mr));

			/* Save last byte to write in ubSaveMem[0]. */

			ubSaveMem[0] = *((UBYTE *) pMem + (uwLength-1));
		}

		if (uwLength > 1)
		{
			mr.Length        = uwLength & 0xFFFE;
			mr.SrcHandle     = 0;
			mr.SrcPos.Data   = pMem;
			mr.DstHandle     = xp->XMSHandle;
			mr.DstPos.Offset = xp->XMSOffset + uwOffset;

			CallXMS (0x0B, 0, FP_SEG (&mr), FP_OFF (&mr));
		}

		if (uwLength & 1)
		{
			/* Write last byte to XMS and byte+1 (from XMS) back to XMS. */

			mr.Length        = 2;
			mr.SrcHandle     = 0;
			mr.SrcPos.Data   = ubSaveMem;
			mr.DstHandle     = xp->XMSHandle;
			mr.DstPos.Offset = xp->XMSOffset + uwOffset + (uwLength-1);

			CallXMS (0x0B, 0, FP_SEG (&mr), FP_OFF (&mr));
		}
	}

} /* WriteXMS */


/*********************************************************************
 *
 * PushXMS
 *
 * SYNOPSIS
 *		short PushXMS (void)
 *
 * PURPOSE
 *		Push the current XMS active pages
 *		NOTE: You will need to call UpdateXMS() before you call Push
 *		if you've changed and data un the current XMS.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short PushXMS (void)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "PushXMS";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		XMSTracker		*xms;

		xms = CreateNode (sizeof (XMSTracker), NULL);
		if (!xms) {
			return FALSE;
		}
		xms->XMS = LastXMS;
		AddTail (XMSStack, xms);

		return TRUE;
	}

} /* PushXMS */


/*********************************************************************
 *
 * AvailXMS
 *
 * PURPOSE
 *		Return available XMS memory in K bytes.
 *		If XMS memory has not been initialized, this routine
 *		returns -1, even if plenty of XMS exists.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *		Available XMS memory in K bytes, or -1 if not used.
 *
 * HISTORY
 *		03/07/94 Monday (dcc) - add support for cXMSKBytesAllocated
 *
 * SEE ALSO
 *
*/
long AvailXMS(void)
{

	long ulAvail = -1L;

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "AvailXMS";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	if (XMSInit)
	{
		Book *bk = Head (BookList);
		UWORD uw;

		ulAvail = 0;
		while (!EndOfList (bk))
		{
			FreeMemType *fmt = Head (bk->FreeList);

			while (!EndOfList (fmt))
			{
				ulAvail += fmt->Size;

				fmt = Next (fmt);
			}
			bk = Next (bk);
		}
		ulAvail /= 1024;

		CallXMS(0x08, 0, 0, 0);
		uw = XMS_DX;

		/* See if there is a limit on how much XMS we may allocate */

		if (cXMSMaxKBytes)
			uw = min(uw, cXMSMaxKBytes - cXMSKBytesAllocated);

		ulAvail += uw;
	}

	return ulAvail;
} /* AvailXMS */


/*********************************************************************
 *
 * PopXMS
 *
 * SYNOPSIS
 *		void PopXMS (void)
 *
 * PURPOSE
 *		Pop the most recently Pushed XMS pages
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void PopXMS (void)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "PopXMS";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		XMSTracker	*xms;

		xms = RemTail (XMSStack);
		ActivateXMS (xms->XMS);
		DeleteNode (xms);

	}

} /* PopXMS */

