/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * EXIT.C
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 01/18/91
 *   MODIFIED : 09/21/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		Exit/Cleanup routines
 *
 * HISTORY
 *	03/19/93 Friday (dcc) - make SIGTERM exit silently.
 *
 *	05/12/92 Tuesday (dcc) - Try to make Ctrl-C not interrupt AbortExit()
 *		by adding setcbrk(0) to the function, and restoring setcbrk()
 *		after all exit functions have been called.
 *
 *	05/07/92 Thursday (GAT) - The aliens are back and they are messing me
 *		up because they made me believe that the atexit() function gets
 *		called on all cases of exiting a program but the definition of
 *		atexit() sez it only gets called on NORMAL termination from a
 *		program, a fact that the aliens hid from me.  Fortunately Captain
 *		Greggman came to the rescue by adding support for the signal()
 *		function which handles ABBY-NORMAL program termination.!
 *
 *	05/26/91 Sunday (RGM) - Added include for <stdlib.h>, added fn prototype
 *		for atexit 'cause of the way how SAS/Lattice's <stdlib.h> don' have
 *		it, even though it's in the compiler manual.  Sheesh.
 *
 *	12/10/91 Tuesday (GAT) - Found the alien plot to rule the world by
 *		puting stupid bugs in my code.  Boy I fried that space turkey good.
 *		Bug was that 'init' never got set in AddExitFunc().  Uuugghhh!
 *
*/

#include <echidna/platform.h>
#include "switches.h"

#include <echidna/listfunc.h>
#include <echidna/exit.h>
#if __MSDOS__
#include <dos.h>
#endif/*__MSDOS__*/
#include <stdlib.h>
#include <signal.h>
#include <stdio.h>

#if LATTICE
#ifndef atexit
extern int	atexit (void (*fcn)(void));
#endif
#endif

#define LOCAL static

/**************************** C O N S T A N T S ***************************/


/******************************** T Y P E S *******************************/

typedef struct {
	NodeType	 Node;
	ExitFunc	 Exit;
} ExitNode;

/****************************** G L O B A L S *****************************/

ListType	 ExitListX;
ListType	*ExitList = &ExitListX;

#if __MSDOS16__
LOCAL		 fCtrlBreakState;		/* Save state of BREAK Off/On flag */
#endif/*__MSDOS16__*/

/******************************* M A C R O S ******************************/


/***************************** R O U T I N E S ****************************/


#if __WATCOMC__
/*********************************************************************
 *
 * getcbrk
 *
 * PURPOSE
 *		Get the current control-break setting.
 *
 * INPUT
 *		
 *
 * ASSUMES
 *		
 *
 * RETURN VALUE
 *		
 *
 * HISTORY
 *		07/08/93 Thursday (dcc) - created.
 *
*/
int getcbrk(void)
{
	union REGS regs;

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "getcbrk";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	regs.w.ax = 0x3300;
	intdos(&regs, &regs);
	return regs.h.dl;

} /* getcbrk */


/*********************************************************************
 *
 * setcbrk
 *
 * PURPOSE
 *		Set control-break setting to <cbrkvalue>.
 *
 * INPUT
 *		
 *
 * ASSUMES
 *		
 *
 * RETURN VALUE
 *		Returns <cbrkvalue>
 *
 * HISTORY
 *		07/08/93 Thursday (dcc) - created.
 *
*/
int setcbrk(int cbrkvalue)
{
	union REGS regs;

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "setcbrk";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	regs.w.ax = 0x3301;
	regs.h.dl = cbrkvalue;
	intdos(&regs, &regs);
	return cbrkvalue;

} /* setcbrk */
#endif /*__WATCOMC__*/


/*********************************************************************
 *
 * CallExitFuncs
 *
 * SYNOPSIS
 *		LOCAL void CallExitFuncs (void)
 *
 * PURPOSE
 *		Call all the exit routines.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL void CallExitFuncs (void)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "CallExitFuncs";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		ExitNode	*en;

		while ((en = RemTail (ExitList)) != NULL) {
			en->Exit ();
			DeleteNode (en);
		}

#if __MSDOS16__
		setcbrk(fCtrlBreakState);	/* Restore BREAK Off/On state */
#endif/*__MSDOS16__*/
	}
} /* CallExitFuncs */


/*********************************************************************
 *
 * AbortExit
 *
 * SYNOPSIS
 *		LOCAL void AbortExit (int sigtype)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL void AbortExit (int sigtype)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "AbortExit";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		char	*msg = NULL;

#if __MSDOS16__
		setcbrk(0);					/* Turn BREAK OFF while doing exit stuff */
#endif/*__MSDOS16__*/

		switch (sigtype) {
		case SIGABRT:
			msg = "abnormal termination";
			break;
		case SIGFPE :
			msg = "Divide by zero/overflow";
			break;
		case SIGILL :
			msg = "Illegal Instruction";
			break;
		case SIGINT :
			msg = "Ctrl-C Break";
			break;
		case SIGSEGV:
			msg = "Illegal Storage Access";
			break;
		case SIGTERM:
			break;
		default:
			msg = "unknown signal (abort)";
			break;
		}
		if (msg)
			puts (msg);
		exit (EXIT_FAILURE);
	}
} /* AbortExit */


/*********************************************************************
 *
 * RemoveExitFunc
 *
 * SYNOPSIS
 *		void RemoveExitFunc (ExitFunc ef)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void RemoveExitFunc (ExitFunc ef)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "RemoveExitFunc";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		ExitNode	*en;

		en = Tail (ExitList);
		while (!IsSOList (en)) {
			if (en->Exit == ef) {
				Remove (en);
				DeleteNode (en);
				return;
			}
			en = Prev (en);
		}
	}
} /* RemoveExitFunc */


/*********************************************************************
 *
 * AddExitFunc
 *
 * SYNOPSIS
 *		short AddExitFunc (ExitFunc ef)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short AddExitFunc (ExitFunc ef)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "AddExitFunc";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		static short init = FALSE;
		ExitNode	*en;

		if (!init) {
			InitList (ExitList);
			atexit (CallExitFuncs);
#if __WATCOMC__
			signal (SIGBREAK,AbortExit);
#endif /*__WATCOMC__*/
			signal (SIGABRT, AbortExit);
			signal (SIGFPE , AbortExit);
			signal (SIGILL , AbortExit);
			signal (SIGINT , AbortExit);
			signal (SIGSEGV, AbortExit);
			signal (SIGTERM, AbortExit);
#if __MSDOS16__
			fCtrlBreakState = getcbrk();	/* Get BREAK Off/On state */
#endif/*__MSDOS16__*/
			init = TRUE;
		}
		en = CreateNode (sizeof (ExitNode), NULL);
		if (!en) {
			return FALSE;
		}
		en->Exit = ef;
		AddTail (ExitList, en);
		return TRUE;
	}
} /* AddExitFunc */

