/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * RMDOS.C
 *
 * PROGRAMMER : Dan Chang
 *    VERSION : 00.000
 *    CREATED : 11/21/93
 *   MODIFIED : 07/27/94
 *       TABS : 04 07
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w-----
 *	 E C H i D N A
 *	--------------
 *
 * DESCRIPTION
 *		Protected-mode equivalents for some real-mode <dos.h> routines.
 *
 * HISTORY
 *		11/21/93 Sunday (dcc) - created.
 *
*/

#include <echidna/platform.h>

#if (__WATCOMC__ && __MSDOS32X__)

#include <dos.h>
#include <string.h>

#include <echidna/rmdos.h>

/**************************** C O N S T A N T S ***************************/

#define DPMI_INT 0x31

/******************************** T Y P E S *******************************/

typedef struct {
	long edi;
	long esi;
	long ebp;
	long Reserved;
	long ebx;
	long edx;
	long ecx;
	long eax;
	short flags;
	short es,ds,fs,gs,ip,cs,sp,ss;
} RMI;

/****************************** G L O B A L S *****************************/

static RMI rmi;

/******************************* M A C R O S ******************************/


/***************************** R O U T I N E S ****************************/


/*********************************************************************
 *
 * _dos_getrmvect
 *
 * PURPOSE
 *		Returns the address of the real-mode interrupt handler for
 *		interrupt number <iInterrupt>. This uses the DPMI support.
 *
 * INPUT
 *		
 *
 * ASSUMES
 *		
 *
 * RETURN VALUE
 *		
 *
 * HISTORY
 *		11/21/93 Sunday (dcc) - created.
 *
*/
//void (__interrupt __far *rm_dos_getvect(int _intnum))()
void *rm_dos_getvect(int _intnum)
{
	union REGS regs;

	regs.w.ax = 0x0200;
	regs.h.bl = _intnum;
	int386(DPMI_INT, &regs, &regs);
	return (void *) rmMK_FP32(regs.w.cx, regs.w.dx);
} /* _dos_getrmvect */


/*********************************************************************
 *
 * rmint86
 *
 * PURPOSE
 *		Executes a real-mode 8086 software interrupt specified by
 *		<iInt>. Before executing the software interrupt, the CPU
 *		registers are copied from <pregsIn>. Following the interrupt,
 *		the CPU registers are copied to <pregsOut>.
 *
 * INPUT
 *		iInt		: real-mode software interrupt to execute
 *		pregsIn	: input CPU registers
 *		pregsOut	: output CPU registers
 *
 * ASSUMES
 *		
 *
 * RETURN VALUE
 *		CPU AX value.
 *
 * HISTORY
 *		10/11/93 Monday (dcc) - created.
 *
*/
int rmint86(int iInt, const union rmREGS *pregsIn, union rmREGS *pregsOut)
{
	union REGS regs;
	struct SREGS sregs;
	RMI far *prmiOut;

	memset(&sregs, 0, sizeof(sregs));
	memset(&rmi, 0, sizeof(rmi));

	rmi.edi = pregsIn->x.di;
	rmi.esi = pregsIn->x.si;
	rmi.ebx = pregsIn->x.bx;
	rmi.edx = pregsIn->x.dx;
	rmi.ecx = pregsIn->x.cx;
	rmi.eax = pregsIn->x.ax;
	rmi.flags = pregsIn->x.cflag;

	regs.w.ax = 0x0300;
	regs.h.bl = iInt;
	regs.h.bh = 0;
	regs.w.cx = 0;
	sregs.es = FP_SEG(&rmi);
	regs.x.edi = FP_OFF(&rmi);

	int386x(DPMI_INT, &regs, &regs, &sregs);
	prmiOut = MK_FP(sregs.es, regs.x.edi);

	pregsOut->x.ax = prmiOut->eax;
	pregsOut->x.bx = prmiOut->ebx;
	pregsOut->x.cx = prmiOut->ecx;
	pregsOut->x.dx = prmiOut->edx;
	pregsOut->x.si = prmiOut->esi;
	pregsOut->x.di = prmiOut->edi;
	pregsOut->x.cflag = prmiOut->flags & 1;
	pregsOut->x.flags = prmiOut->flags;

	return pregsOut->x.ax;

} /* rmint86 */


/*********************************************************************
 *
 * rmint86x
 *
 * PURPOSE
 *		Executes a real-mode 8086 software interrupt specified by
 *		<iInt>. Before executing the software interrupt, the CPU
 *		registers are copied from <pregsIn>, and the DS and ES segment
 *		registers are copied from <psregs>. Following the interrupt,
 *		the CPU registers are copied to <pregsOut>.
 *
 * INPUT
 *		iInt		: real-mode software interrupt to execute
 *		pregsIn	: input CPU registers
 *		pregsOut	: output CPU registers
 *		psregs	: input and output segment registers
 *
 * ASSUMES
 *		
 *
 * RETURN VALUE
 *		CPU AX value.
 *
 * HISTORY
 *		10/11/93 Monday (dcc) - created.
 *
*/
int rmint86x(int iInt, const union rmREGS *pregsIn, union rmREGS *pregsOut,
				struct rmSREGS *psregs)
{
	union REGS regs;
	struct SREGS sregs;
	RMI far *prmiOut;

	memset(&sregs, 0, sizeof(sregs));
	memset(&rmi, 0, sizeof(rmi));

	rmi.edi = pregsIn->x.di;
	rmi.esi = pregsIn->x.si;
	rmi.ebx = pregsIn->x.bx;
	rmi.edx = pregsIn->x.dx;
	rmi.ecx = pregsIn->x.cx;
	rmi.eax = pregsIn->x.ax;
	rmi.flags = pregsIn->x.cflag;
	rmi.es = psregs->es;
	rmi.ds = psregs->ds;

	regs.w.ax = 0x0300;
	regs.h.bl = iInt;
	regs.h.bh = 0;
	regs.w.cx = 0;
	sregs.es = FP_SEG(&rmi);
	regs.x.edi = FP_OFF(&rmi);

	int386x(DPMI_INT, &regs, &regs, &sregs);
	prmiOut = MK_FP(sregs.es, regs.x.edi);

	pregsOut->x.ax = prmiOut->eax;
	pregsOut->x.bx = prmiOut->ebx;
	pregsOut->x.cx = prmiOut->ecx;
	pregsOut->x.dx = prmiOut->edx;
	pregsOut->x.si = prmiOut->esi;
	pregsOut->x.di = prmiOut->edi;
	pregsOut->x.cflag = prmiOut->flags & 1;
	pregsOut->x.flags = prmiOut->flags;
	psregs->es = prmiOut->es;
	psregs->ds = prmiOut->ds;

	return pregsOut->x.ax;

} /* rmint86x */

#endif /*(__WATCOMC__ && __MSDOS32X__)*/

