/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * LOADPBX.C
 *
 * PROGRAMMER : Juan M. Alvarado
 *    VERSION : 00.000
 *    CREATED : 04/15/91
 *   MODIFIED : 10/07/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		
 *
 * HISTORY
 *		05/29/91 (RGM) - Fixed LoadPBX() so that it doesn't try to
 *						 return a 'short PaintBoxType *' (why didn't
 *						 bcc catch this when the prototype was just
 *						 'PaintBoxType *' in <echidna/stdload.h> ?).
 *		10/07/94 (dcc) - updated to use <echidna/alloc.h>
 *
*/

#include <echidna/platform.h>
#include "switches.h"

#include <echidna/alloc.h>
#include <echidna/graphics.h>
#include <echidna/stdload.h>
#include "xpaksup.h"

/**************************** C O N S T A N T S ***************************/


/******************************** T Y P E S *******************************/


/****************************** G L O B A L S *****************************/


/******************************* M A C R O S ******************************/


/***************************** R O U T I N E S ****************************/


/*********************************************************************
 *
 * LoadPBX
 *
 * SYNOPSIS
 *		PaintBoxType	*LoadPBX (
 *			char	*filename,
 *			short	 selector
 *		)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
PaintBoxType	*LoadPBX (
	char	*filename,
	short	 selector
)
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "LoadPBX";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
{
	PaintBoxType	*paintbox;

	/** Open File **/
	if (! FPF_open (filename, selector))
	{
		/* report error msg from FPF_open */
		return NULL;
	}

	paintbox = ReadPBX ((size_t)FPF_BytesLeft ());
	
	FPF_close ();

	return paintbox;
}
} /* LoadPBX */



/*********************************************************************
 *
 * FreePBX
 *
 * SYNOPSIS
 *		void FreePBX (PaintBoxType *paintbox)
 *
 * PURPOSE
 *		Free Structure allocated by ReadPBX().
 *
 * INPUT
 *		paintbox:	pointer to structure allocated by ReadPaintBox().
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void FreePBX (PaintBoxType *paintbox)
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "FreePBX";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
{
	ColorCycleSetType	*cycleset;
	PaletteType			*palette;

	if (paintbox) 
	{
		if ((cycleset = paintbox->CycleSet) != NULL)
		{
			if (cycleset->Cycles) MEM_free (cycleset->Cycles);
			if (cycleset->Registers) MEM_free (cycleset->Registers);
			if (cycleset->Colors) MEM_free (cycleset->Colors);
			MEM_free (cycleset);
		}
		if ((palette = paintbox->Palette) != NULL)
		{
			#if __MSDOS__
				switch (GraphicsMode) {
				case MODE_EGA: case MODE_TANDY:
					MEM_free (palette->Data.EGA);
					break;
				case MODE_MCGA:
					MEM_free (palette->Data.RGB);
					break;
				}
			#elif __AMIGAOS__
				MEM_free (palette->Data.RGB);
			#else
				#error Error: Platform missing FreePBX() case.
			#endif
		}
	
		MEM_free (paintbox);
	}
}
} /* FreePBX */

