/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * PixlScan.C
 *
 * PROGRAMMER : Juan M. Alvarado
 *    VERSION : 00.000
 *    CREATED : 02/11/90
 *   MODIFIED : 02/05/95
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		Global variables and routines that allow flipping and color remapping
 *		of an unpacked raw mcga shape as it is scanned from a buffer.
 *
 * HISTORY
 *		02/05/95 (dcc) - #include <echidna/pixlscan.h> to verify 
 *						 function prototypes.
 *
*/
#include <stdlib.h>
#include <echidna/etypes.h>
#include <echidna/graphics.h>
#include <echidna/memory.h>
#include <echidna/pixlscan.h>			// verify function prototypes

/**************************** C O N S T A N T S ***************************/
#define PATTERN_SIZE	4

/******************************** T Y P E S *******************************/


/****************************** G L O B A L S *****************************/


/******************************* M A C R O S ******************************/


/***************************** R O U T I N E S ****************************/

/*********************************************************************
 *
 * OpenPixlScan
 *
 * SYNOPSIS
 *		short OpenPixlScan(maxcolors)												
 *
 * PURPOSE
 *		To allocate tabels used in scanning (color remapping and pattern
 *		mapping).	
 *
 * INPUT
 *		maxcolors:	Maximum number of colors contained in shapes to be 
 *					scanned. Must be greater than 0.
 *
 * EFFECTS
 *		Allocates space for tables:
 *			Table				Size
 *			-----------------	----
 *			Color Remapping		maxcolors bytes
 *			Pattern Remapping	maxcolors * PATTERN_SIZE bytes.
 *
 * RETURN VALUE
 *		Returns non-zero if succesful, else returns 0.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *		ClosePixlScan()
*/
short OpenPixlScan (
	ConvertSType	*cs,
	short			 maxcolors
)																		
{
	short r;

	if ((cs->ColorMap = (UBYTE *)CreateTempBuffer (maxcolors)) != NULL) {
		for (r = 0; r < maxcolors; r++) {
			cs->ColorMap[r] = r;
		}
	} else {
			return FALSE;
	}

	return TRUE;
} /* OpenPixlScan */

